package cnh.crb;

import java.io.File;
import java.io.IOException;
import java.util.Formatter;

import org.geotools.coverage.grid.GridCoverage2D;
import org.geotools.coverage.grid.io.AbstractGridCoverage2DReader;
import org.geotools.coverage.grid.io.AbstractGridFormat;
import org.geotools.coverage.grid.io.GridFormatFinder;
import org.geotools.data.DataSourceException;

import ucar.nc2.constants.FeatureType;
import ucar.nc2.dt.GridDataset;
import ucar.nc2.ft.FeatureDataset;
import ucar.nc2.ft.FeatureDatasetFactoryManager;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.Polygon;

public class CNHCRBUtil {
	public static final double SIX_MIN_SIDE = 0.1;
	public static final double SIX_MIN_DELTA = SIX_MIN_SIDE / 2;

	public static Polygon createPolygonCenteredOn(GeometryFactory fac,
			double lat, double lon) {
		return fac.createPolygon(create6MinuteCell(lat, lon));
	}

	public static Polygon createPolygonCenteredOn(double lat, double lon) {
		return createPolygonCenteredOn(new GeometryFactory(), lat, lon);
	}

	protected static Coordinate[] create6MinuteCell(double lat, double lon) {
		Coordinate southWest = new Coordinate(lon - SIX_MIN_DELTA, lat
				- SIX_MIN_DELTA);
		Coordinate northWest = new Coordinate(lon - SIX_MIN_DELTA, lat
				+ SIX_MIN_DELTA);
		Coordinate northEast = new Coordinate(lon + SIX_MIN_DELTA, lat
				+ SIX_MIN_DELTA);
		Coordinate southEast = new Coordinate(lon + SIX_MIN_DELTA, lat
				- SIX_MIN_DELTA);
		Coordinate[] result = { southWest, northWest, northEast, southEast,
				southWest };
		return result;
	}

	public static GridCoverage2D gettingASC(String filePath) {
//		filePath = "/Volumes/PassportT/Data_Folder/CNH/6minDataFromRichard/ASCII-Files/direction.asc";

		try {
			File f = new File(filePath);
			AbstractGridFormat format = GridFormatFinder.findFormat(f);
			AbstractGridCoverage2DReader reader = format.getReader(f);

			// ArcGridReader reader = new ArcGridReader(is);
			return reader.read(null);

		} catch (DataSourceException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	public static void main(String[] args) {
//		GridCoverage2D cov = gettingASC(null);
//		int[] vals = null;
//		DirectPosition pos = new DirectPosition2D(-111.41562798969461,
//				31.77046259737428);
//		vals = cov.evaluate(pos, vals);
//		for (int i : vals) {
//			System.out.println(i);
//		}

//		GridDataset gds = readNetCDF("/Volumes/PassportT/Data_Folder/CNH/6minDataFromRichard/NAMSTNetwork207Potential06min_v2.nc");
	}

	public static GridDataset readNetCDF(String location) {
		Formatter errlog = new Formatter();
		FeatureDataset fdataset;
		try {
			fdataset = FeatureDatasetFactoryManager.open(FeatureType.GRID,
					location, null, errlog);
			if (fdataset == null) {
				System.out.printf("**failed on %s %n --> %s %n", location,
						errlog);
				return null;
			}
			FeatureType ftype = fdataset.getFeatureType();
			assert (ftype == FeatureType.GRID);
			GridDataset gds = (GridDataset) fdataset;
			return gds;
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}
}
