package cnh.crb;

import java.awt.geom.Point2D;
import java.io.File;
import java.text.DecimalFormat;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import repast.simphony.context.Context;
import cnh.crb.wbm.WBMConnector;
import cnh.crb.wbm.WBMNaturalWaterSource;
import cnh.crb.wbm.WBMWaterSourceCollection;
import cnh.crb.wbm.WBMWaterSourceManager;

public class WaterSourceLoader {
	
	private static Logger LOGGER = Logger.getLogger(WaterSourceLoader.class);
	
	public static Map<String, String> nameMap  = new HashMap<String, String>(); 
	public static Map<String, String> coordMap = new HashMap<String, String>();
	
	public static Set<Point2D.Double> fixedPoints = new HashSet<Point2D.Double>();
	
	private static boolean useFixedPoints = false;
	
	public static void init(){
		nameMap     = new HashMap<String, String>();
		coordMap    = new HashMap<String, String>();
		fixedPoints = new HashSet<Point2D.Double>();
	}
	
	public static void load(String filename, Context<Object> context, WBMWaterSourceCollection wsColl){
		load(filename, context, wsColl, false);
	}
	
	public static void load(String filename, Context<Object> context, WBMWaterSourceCollection wsColl, boolean manageIrr){
		int count = 0;
		
		// Need to trim these
		String decs = ("#########").substring(0, WBMConnector.DECIMAL_COORDINATES);
		DecimalFormat df = new DecimalFormat("#." + decs);
		df.setMaximumFractionDigits(WBMConnector.DECIMAL_COORDINATES);

		try {
			File inputFile = new File(filename);
			DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
			Document doc = docBuilder.parse(inputFile);
		 
			doc.getDocumentElement().normalize();
			Node root = doc.getElementsByTagName("NodeCollection").item(0);
			NodeList nodeList = root.getChildNodes();

			// Establish fixed points
			if(useFixedPoints){
				for(int i = 0; i < nodeList.getLength(); i++){
					Node node = nodeList.item(i);
					if(node.getNodeType() == Node.ELEMENT_NODE){
						Element element = (Element) node;
						String type = element.getAttribute("type");
						// Only two types: definitions of managers and redirects to sub-files:
						switch(type){
						  // Theoretically could define and create natural water sources, non-WBM
						  case "wbmNaturalWaterSource":
							String destType = element.getAttribute("destType");
							double  lat              = java.lang.Double.parseDouble(element.getAttribute("lat"));
							double  lon              = java.lang.Double.parseDouble(element.getAttribute("lon"));
							double  flow             = java.lang.Double.parseDouble(element.getAttribute("defaultFlow"));
							boolean manageIrrigation = manageIrr && !(element.getAttribute("manageIrrigation").compareToIgnoreCase("false") == 0);
							
							if((destType.compareTo("city")     == 0) ||
							   (destType.compareTo("recharge") == 0)){
								
							  if(destType.compareTo("city") == 0)  fixedPoints.add( new Point2D.Double(lat, lon)); // Only add cities to 'merge' points
							  // Also need to add these to the name map and coord map
							  String name = element.getAttribute("name");
							  
							  String latLonKey = df.format(lat) + "_" + df.format(lon);
							  
							  count++;
							  WBMNaturalWaterSource temp = 
									  wsColl.createWaterSource(name + "SOURCE", java.lang.Double.parseDouble(df.format(lat)),java.lang.Double.parseDouble(df.format(lon)));
							  LOGGER.debug("Setting manage irrigation (A) for " + temp.getId() + " to " + manageIrrigation);
							  temp.setManageIrrigation(manageIrrigation);
							  
							  // Must also create a manager
							  if(temp != null){
								temp.setDefaultFlow(flow);
								WBMWaterSourceManager tmpMgr = new WBMWaterSourceManager(name, temp);
								context.add(temp);
								context.add(tmpMgr);
							  }
							  nameMap.put(name,  name);
							  coordMap.put(latLonKey,  name);
							}
							break;
						  case "file":
						  default:
						}
					}
				}
				
			}
			
			
			for(int i = 0; i < nodeList.getLength(); i++){
				Node node = nodeList.item(i);
				if(node.getNodeType() == Node.ELEMENT_NODE){
					Element element = (Element) node;
					String type = element.getAttribute("type");
					// Only two types: definitions of managers and redirects to sub-files:
					switch(type){
					  // Theoretically could define and create natural water sources, non-WBM
					  case "wbmNaturalWaterSource":
						String name              = element.getAttribute("name");
						double lat  = java.lang.Double.parseDouble(element.getAttribute("lat"));
						double lon  = java.lang.Double.parseDouble(element.getAttribute("lon"));
						double flow = java.lang.Double.parseDouble(element.getAttribute("defaultFlow"));
						boolean manageIrrigation = manageIrr && !(element.getAttribute("manageIrrigation").compareToIgnoreCase("false") == 0);
						
						// Rewrite coords
						String destType = element.getAttribute("destType");
						if((destType != null) &&
							destType.compareTo("other") == 0){
							Point2D.Double closest = getClosest(lat, lon);
							if((closest.x != lat) && (closest.y != lon)){
								LOGGER.debug("For element " + name + " coordinates changed from " + lat + "," + lon + " to closest fixed point at " + closest.toString());
								lat = closest.x;
								lon = closest.y;
							}
						}
						String latLonKey = df.format(lat) + "_" + df.format(lon);
						
						if(!coordMap.containsKey(latLonKey)){
							count++;
							WBMNaturalWaterSource temp = wsColl.createWaterSource(name + "SOURCE", java.lang.Double.parseDouble(df.format(lat)),java.lang.Double.parseDouble(df.format(lon)));
							if(temp != null){
								temp.setDefaultFlow(flow);
								LOGGER.debug("Setting manage irrigation (B) for " + temp.getId() + " to " + manageIrrigation);
								temp.setManageIrrigation(manageIrrigation); // TODO: This actually doesn't work: it is possible to end up with irrigation not managed when it should be
								                                            // The issue is that a 'manage irrigation' source should take priority over all 'don't manage' sources,
								                                            // but it really doesn't...
								// Must also create a manager
								WBMWaterSourceManager tmpMgr = new WBMWaterSourceManager(name, temp);
								context.add(temp);
								context.add(tmpMgr);
							}
							nameMap.put(name, name);
							coordMap.put(latLonKey, name);
						}
						else{
							// Record that this was not created and the original one should be used
							LOGGER.warn("Found a duplicate coordinate water source: " + name + " is dup of " + coordMap.get(latLonKey));
							nameMap.put(name,  coordMap.get(latLonKey));
						}
						
						break;
					  case "file":
					  default:
						String fileToLoad = element.getAttribute("name");
						load(fileToLoad, context, wsColl);
					}
				}
			}
			LOGGER.info("DONE CREATING WATER SOURCES...");
		} catch (Exception e) {
			LOGGER.error("ERROR OCCURRED IN CREATEWATERMANAGERS", e);
		}
		LOGGER.info("WaterSourceLoader loaded " + count + " water sources.");

	}
	
	private static Point2D.Double getClosest(double lat, double lon){
		Point2D.Double compare = new Point2D.Double(lat, lon);
		if(fixedPoints.size() == 0) return compare;
		
		double dist = java.lang.Double.MAX_VALUE; // Not sure why max value won't compile here... 
		Point2D.Double ret = null;
		for(Point2D.Double comp: fixedPoints){
			double cDist = compare.distance(comp); // Note: treating as x,y even though really lat/lon
			if(cDist < dist) ret = comp; 
		}
		return ret;
	}

	private static String getName(double lat, double lon){
		String decs = ("#########").substring(0, WBMConnector.DECIMAL_COORDINATES);
		DecimalFormat df = new DecimalFormat("#." + decs);
		df.setMaximumFractionDigits(WBMConnector.DECIMAL_COORDINATES);

		return "SOURCE_[" + df.format(lat) + "," + df.format(lon) + "]";
	}
	
	public static void create(double NWLat, double NWLon, double SELat, double SELon, double NSRes, double EWRes, Context<Object> context, WBMWaterSourceCollection wsColl){
		create(NWLat, NWLon, SELat, SELon, NSRes, EWRes, context, wsColl, false);
	}

	public static void create(double NWLat, double NWLon, double SELat, double SELon, double NSRes, double EWRes, Context<Object> context, WBMWaterSourceCollection wsColl, boolean manageIrr){
		String decs = ("#########").substring(0, WBMConnector.DECIMAL_COORDINATES);
		DecimalFormat df = new DecimalFormat("#." + decs);
		df.setMaximumFractionDigits(WBMConnector.DECIMAL_COORDINATES);

		for(double lat = SELat; lat <= NWLat; lat+= NSRes){
			for(double lon = SELon; lon >= NWLon; lon -= EWRes){
				String name = getName(lat, lon);
				WBMNaturalWaterSource temp = wsColl.createWaterSource(name + "_SOURCE", java.lang.Double.parseDouble(df.format(lat)),java.lang.Double.parseDouble(df.format(lon)));
				temp.setManageIrrigation(manageIrr);
				if(temp!= null){
					WBMWaterSourceManager tmpMgr = new WBMWaterSourceManager(name, temp);
					context.add(temp);
					context.add(tmpMgr);
				}
				LOGGER.info("CREATING " + name);
			}
		}
	}
	
	
}
