package cnh.crb.agent;

import java.util.Date;

import simphony.util.messages.MessageCenter;
import cnh.crb.hydro.DemandRequest;
import cnh.crb.hydro.FlowHydrograph;
import cnh.crb.hydro.FlowManager;
import cnh.crb.hydro.FlowRate;
import cnh.crb.hydro.SourceFlowNode;
import cnh.crb.hydro.Storage;

public class ConstantDemandWaterManager extends BasicWaterManager{

  static MessageCenter msg = MessageCenter.getMessageCenter(ConstantDemandWaterManager.class);
	
  public double constantDemand = 0;
  public FlowRate.FlowRateUnits units;

  public ConstantDemandWaterManager(Storage storage, String managerID, double constantDemand, FlowRate.FlowRateUnits units) {
	super(storage, managerID);
	this.constantDemand = constantDemand;
	this.units = units;
  }
	
  protected NotificationStatus calculateAmountsToDemandFromSources(Date start, Date end, FlowManager flowManager){
	  // Assume only one source (for now)
	  SourceFlowNode src = waterSources.get(0);
	  // The amount of the demand will be the sum of all requested amounts
	  FlowHydrograph H = getDemand(start, end);
	  expectedInflow.put(src, new DemandRequest(H, null)); // Assume it will be met
	  return new NotificationStatus(false, false);
  }
  
  public FlowHydrograph getDemand(Date start, Date end){
	  super.getDemand(start, end); // Calculate as per usual
	  return new FlowHydrograph(start, end, constantDemand, units); // But actual demand is constant
  }
  
  public void flow(Date end){
	  super.flow(end);
	  msg.info(getId() + "- My Reservoir Volume is: " + this.storage.getCurrentVolume());
  }
	

}
