package cnh.crb.agent;

import cnh.crb.hydro.FlowRate;
import cnh.crb.hydro.Storage;
import cnh.crb.hydro.UnitsAndConverters;

public class DamManager extends BasicWaterManager {

	protected double   storageOptimum =  0.90d;
	protected double   alpha          =  0.6667;
	protected double   beta           = 10.0;
	protected double   qMin           = 0.2; // Fraction of annual Flow
	
	public DamManager(Storage storage, String managerID) {
		super(storage, managerID);
	}
	
	public double release(FlowRate avgInflow){
		double ret = 0;
		if(storage.getCurrentVolume() / storage.getMaxVolume() < storageOptimum){
			double regCap = avgInflow.getRateAFY() / (UnitsAndConverters.M3ToAF(storage.getMaxVolume()));
			double k = (1/Math.pow(storageOptimum, alpha / Math.sqrt(regCap))) *
					(Math.exp(1 - qMin * regCap) - 1);
			ret = qMin * regCap + Math.log((k * (Math.pow(storageOptimum, alpha / Math.sqrt(regCap))) ) + 1);
			
		}
		else{
			ret = Math.exp(beta * (Math.pow(storage.getCurrentVolume()/storage.getMaxVolume(),2)));
		}
		return ret;
	}
	
	
	
	

}
