package cnh.crb.agent;

import java.util.Calendar;

import org.apache.log4j.Logger;

import cnh.crb.econ.Bill;
import cnh.crb.econ.HouseholdBillCalculator;
import cnh.crb.household.Household;
import cnh.crb.hydro.FlowNode;
import cnh.crb.hydro.Reservoir;
import cnh.crb.hydro.Tap;

/**
 * Class for flow manager that must bill residential customers.
 * 
 * @author murphy
 * 
 */
public class ResidentialWaterManager extends BasicWaterManager {

	private HouseholdBillCalculator billCalculator;

	public ResidentialWaterManager(Reservoir reservoir, HouseholdBillCalculator billCalculator,
			Calendar initialDate, String managerID) {
		super(reservoir, managerID);
		this.billCalculator = billCalculator;
	}

	public void billCustomers(Calendar billingDate) {
		Logger.getLogger(getClass()).info("Residential Water Manager " + id + " is billing "
				+ waterRecipients.size() + " customers");
		
		
		for (FlowNode node : waterRecipients) {
			if (node instanceof Household) {
				Household household = (Household) node;
				Tap.Usage monthlyUsage = household.getMeteredWaterUse("MONTHLY");
				Bill bill = billCalculator.getBill(billingDate.getTime(), monthlyUsage, household);
				household.receiveBill(bill);
			}
		}
	}

}
