package cnh.crb.analysis;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.TreeMap;

import cnh.crb.analysis.datasets.MidwayDataSetSource;
import cnh.crb.analysis.datasets.MonthlyAverageResampler;
import cnh.crb.analysis.datasets.SingleDataSet;
import cnh.crb.analysis.parametersets.BatchParametersSet;
import repast.simphony.util.collections.Pair;


/***
 * Takes as arguments:
 * 0: Library of run data, e.g. /project/johntmurphy/CNH/run150807
 * 1: Output file name pre-timestamp, e.g. output-subcontractor-agg
 * 2: batch parameters set pathname, e.g. /project/johntmurphy/CNH/run150807/batchParam.txt
 * @author chchow
 */
public class MidwayRankRunsByShortage {
    public static void main(String[] args){

//		MidwayDataSetSource dsSource = 
//				new MidwayDataSetSource(
//						"",                                           // Bulk data does not exist for merged runs (doesn't matter?)
//						"/home/chunho/Desktop/ArizonaWaterProject/pypandas-test/out",//"/project/johntmurphy/run150807",             // Library of run data           
//						"output-subcontractor-agg");                  // Output file name pre-timestamp
		MidwayDataSetSource dsSource = 
				new MidwayDataSetSource(
						"",                                           // Bulk data does not exist for merged runs (doesn't matter?)
						args[0], //e.g. "/project/johntmurphy/run150807"              // Library of run data           
						args[1]);//e.g. "output-subcontractor-agg"                    // Output file name pre-timestamp
		
		// load batch parameter set
//		BatchParametersSet batchParams = new BatchParametersSet("/home/chunho/Desktop/ArizonaWaterProject/pypandas-test/out/batchParam.txt");
		BatchParametersSet batchParams = new BatchParametersSet(args[2]);//e.g. "/project/johntmurphy/CNH/run150807/batchParam.txt"
		
    	// Create and initialize Calendar instance
    	Calendar start = Calendar.getInstance();
		start.set(1993, 0, 1, 0, 0, 0);
		start.set(Calendar.MILLISECOND, 0);
		
		Map<String, Set<String>> valuesVaried = batchParams.getDistinctValues("run");
		System.out.println("Values varied:");
		int combinations = 1;
		for(String param: valuesVaried.keySet()){
			System.out.print(param + ": ");
			combinations *= valuesVaried.get(param).size();
			for(String val: valuesVaried.get(param)) System.out.print(val + " ");
			System.out.println();
		}
		System.out.println("Total possible combinations: " + combinations);
		System.out.println("Total runs: " + batchParams.getCountOfEntries());
		
		MonthlyAverageResampler maResampler = new MonthlyAverageResampler(true);
		
		// This holds the map of water shortage (avg over randomSeeds) to list of param-combinations (since there may be multiple for each value)
		TreeMap<Double, List< Map<String, String> > > rankedParamCombinations = new TreeMap<Double, List< Map<String, String> > >();
		
		// Find the runs that have the same parameters, except for randomSeed

		List<Pair<Map<String, String>, List<Integer>>> commonParamRunsList = batchParams.listRunsWithCommonParameters("run","randomSeed");
		for(Pair<Map<String, String>, List<Integer>> commonParamRuns: commonParamRunsList) {
			List<Integer> runsToExamine = commonParamRuns.getSecond();
			
//			System.out.println("runs to examine: ");
//			runsToExamine.forEach(System.out::println);
			
			// merge the run columns into one SingleDataSet, then average them
			SingleDataSet dsAveraged = new SingleDataSet(dsSource.getDataSet(runsToExamine.get(0)), false);
			dsAveraged.setStartDate(start);
			dsAveraged = dsAveraged.resample(maResampler);
			
			int seedCount = 0;
			ArrayList<String> columnNamesList = new ArrayList<String>(); //  the array containing all the seed#_Shortage name-strings
			for(Integer run: runsToExamine){
				if (seedCount==0) {
					//don't do merging because the column is already copied over above. We just rename it
					dsAveraged.renameColumn("Shortage", "seed0_Shortage");
					
					columnNamesList.add("seed0_Shortage");
				}
				else { // else if seedCount>0
					SingleDataSet ds = dsSource.getDataSet(run);
					ds.setStartDate(start);
					ds = ds.resample(maResampler);
					dsAveraged.merge(ds, "seed"+Integer.toString(seedCount)+"_" );
					
					columnNamesList.add("seed"+Integer.toString(seedCount)+"_Shortage");
				}
				seedCount++;
			}
			
//			System.out.println("averaging (over randomSeeds) current set of runs...");
			// if there's only one seed value, it won't create the "avgShortage" column, so we do this if-else:
			if (seedCount==1) {
				dsAveraged.renameColumn("seed0_Shortage", "avgShortage");
			} else { // else if seedCount>1
			dsAveraged.addCalculatedColumn("avgShortage", SingleDataSet.Function.AVERAGE, columnNamesList.toArray(new String[columnNamesList.size()]) );
			}
			//dsAveraged.report(true);
			
			double totalShortage = dsAveraged.getAggregate(SingleDataSet.Function.SUM, "avgShortage");
			//System.out.println(totalShortage);
			
			if(!rankedParamCombinations.containsKey(totalShortage)) rankedParamCombinations.put(totalShortage, new ArrayList<Map<String, String>>());
			rankedParamCombinations.get(totalShortage).add(commonParamRuns.getFirst());
		}
		
		// What are the parameters?
		List<String> paramNames = new ArrayList<String>();
		paramNames.addAll(rankedParamCombinations.entrySet().iterator().next().getValue().iterator().next().keySet());
		
		
//		// Now just find the top/"best" N entries in the treemap, and output them
		// We want ALL entries printed out, instead of just the top N
//		int N = 10; // We want the N best entries
//		System.out.println(rankedParamCombinations.entrySet());
		
		Set<Entry<Double, List<Map<String, String>>>> set = rankedParamCombinations.entrySet();
		Entry<Double, List<Map<String, String>>>[] entryArray = new Entry[set.size()];
		set.toArray(entryArray);
		
//		System.out.println("Set size: " + set.size());
//		System.out.println("entryArray length: " + entryArray.length);
		
		
		System.out.print("Score");
		for(String param: paramNames) System.out.print("," + param);
		System.out.println();

//		for(int i = set.size()-1; i > set.size() - N && i>0; i--){
		for(int i = set.size()-1; i>=0; i--){
			
//			System.out.print(i + " ");
			
			for (Map<String, String> paramMap: entryArray[i].getValue()){
				System.out.print(entryArray[i].getKey());
				for(String param: paramNames) System.out.print("," + paramMap.get(param));
				System.out.println();
			}
		}
    }
}
