package cnh.crb.analysis;

import java.util.Calendar;

import cnh.crb.analysis.datasets.ColumnRemoverResampler;
import cnh.crb.analysis.datasets.DataSetSource;
import cnh.crb.analysis.datasets.DataTrimmerResampler;
import cnh.crb.analysis.datasets.SingleDataSet;

public class RetrieveHistogramData {
	
	public static void main(String[] args){
		// There will be a common calendar used for all data sets:
		Calendar start = Calendar.getInstance();
		start.set(1993, 0, 1, 0, 0, 0);
		start.set(Calendar.MILLISECOND, 0);
		
		// Enter the list of runs to be analyzed here
		int[] runs = { 
				      1125, 
				      222, 
				      837 
				      };
		
		// Select the kind of analysis:
		boolean showPriceFactor   = false;
		boolean showSocialFactor  = false;
		boolean showServiceCounts = true;
		boolean showWaterUsage    = false;
		
		
		// A 'DataSetSource' is a wrapper around the big output file; asking for one run
		// may write other runs to the library so they don't have to be extracted again when they are
		// requested
		DataSetSource priceFactorDataSource  = new DataSetSource("./out/combined_data_2014_09_30/price_factor_histogram.csv", "./out/analysis/2014_Sep_30/PRICE_FACTORS/", "PRICE_FACTOR");
		DataSetSource socialFactorDataSource = new DataSetSource("./out/combined_data_2014_09_30/social_factor_histogram.csv", "./out/analysis/2014_Sep_30/SOCIAL_FACTORS/", "SOCIAL_FACTOR");
		DataSetSource serviceCountDataSource = new DataSetSource("./out/combined_data_2014_09_30/service_counts_histogram.csv", "./out/analysis/2014_Sep_30/SERVICE_COUNTS/", "SERVICE_COUNT");
		DataSetSource waterUsageDataSource   = new DataSetSource("./out/combined_data_2014_09_30/usage_histogram.csv", "./out/analysis/2014_Sep_30/USAGE_HISTOGRAM/", "USAGE_HIST");
		
		// Create the filter to remove 
		ColumnRemoverResampler removeBatch = new ColumnRemoverResampler("batch");
		Calendar startTrim = Calendar.getInstance();
		startTrim.set(1996, 0, 1, 0, 0, 0);
		startTrim.set(Calendar.MILLISECOND, 0);
		
		DataTrimmerResampler   trimDates   = new DataTrimmerResampler(startTrim.getTime(), null);
		
		for(int i: runs){
			if(showPriceFactor){
				SingleDataSet priceFactorDataSet = priceFactorDataSource.getDataSet(i); // Get the specific run
				priceFactorDataSet.setStartDate(start); // Set the start date
				
				// Remove the 'batch' column
				priceFactorDataSet = priceFactorDataSet.resample(removeBatch);
				
				// Write to the console
				System.out.println("Run: " + i + " Price Factor:");
				priceFactorDataSet.report(true);
			}
			
			if(showSocialFactor){
				SingleDataSet socialFactorDataSet = socialFactorDataSource.getDataSet(i);
				socialFactorDataSet.setStartDate(start);
				socialFactorDataSet = socialFactorDataSet.resample(removeBatch);
				System.out.println("Run: " + i + " Social Factor:");
				socialFactorDataSet.report(true);
			}
			
			if(showServiceCounts){
				SingleDataSet serviceCountsDataSet = serviceCountDataSource.getDataSet(i);
				
				serviceCountsDataSet.setStartDate(start);
				serviceCountsDataSet = serviceCountsDataSet.resample(removeBatch, trimDates);
				System.out.println("Run: " + i + " Service Counts:");
				serviceCountsDataSet.report(true);
			}
			
			if(showWaterUsage){
				SingleDataSet waterUsageDataSet = waterUsageDataSource.getDataSet(i);
				
				waterUsageDataSet.setStartDate(start);
				waterUsageDataSet = waterUsageDataSet.resample(removeBatch);
				System.out.println("Run: " + i + " Usage Histogram:");
				waterUsageDataSet.report(true);
			}
		}
	}
	
	
}
