package cnh.crb.analysis.datasets;



/**
 * Resamples data from a given run.
 * 
 * Instances of this class emulate a different 
 * data collection strategy that could have been 
 * implemented in the simulation but wasn't, but 
 * that can be reconstructed from the data in the 
 * output file specified.
 * 
 * For example, suppose data were collected
 * daily, but weekly averages are now preferred.
 * This class will allow data to be 'resampled',
 * so that:
 * 
 *  Tick, Value
 *  1, 10
 *  2, 10
 *  3, 10
 *  4, 20
 *  5, 30
 *  6, 30
 *  7, 30
 *  
 *  Becomes:
 *  
 *  Tick, Value
 *  7, 20
 *  
 *  reflecting the average for ticks 1-7.
 *  
 *  Additional manipulations can be done, such
 *  as rolling averages or simple filtering
 *  by eliminating ticks or columns.
 *  
 * @author murphy
 *
 */
public abstract class DataResampler {

  public abstract SingleDataSet resample(SingleDataSet dataSet);	
	
}
