package cnh.crb.analysis.datasets;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

import repast.simphony.util.collections.Pair;
import cnh.crb.analysis.parametersets.BatchParametersSet;

public class DataSetSource {

	private String             batchFile;
	private String             archiveDirectory;
	protected OutputFileSplitter splitter;
	private String             baseName;
	private String             batchParamsFileName;
	private String             batchPrefix;
	
	
	public void mergeAndCollapse(List<DataSetSource> sources, SingleDataSet.Function function, String ... parametersToIgnore){
		System.out.println("Initializing...");
		// Get the collection of parameters sets
		// And a map associating each one with a data source
		Map<BatchParametersSet, DataSetSource> paramMap     = new HashMap<BatchParametersSet, DataSetSource>();
		List<BatchParametersSet>               paramSetList = new ArrayList<BatchParametersSet>();
		for(DataSetSource source: sources){
			System.out.println("Loading Parameter Set " + source.batchParamsFileName);
			BatchParametersSet set = source.getParametersSet();
			paramMap.put(set, source);
			paramSetList.add(set);
		}
		// Get the list of unique parameter combinations 
		// and the runs (from each set) associated with them
		System.out.println("Getting lists of runs with common parameters...");
		List<Pair<Map<String, String>, Map<BatchParametersSet, List<Integer>>>> commonRuns = BatchParametersSet.listRunsWithCommonParameters(paramSetList, parametersToIgnore);

		// Loop through this (messy) thing
		System.out.println("Performing main loop...");
		int run = 1;
		BatchParametersSet newParametersSet = null;
		for(Pair<Map<String, String>, Map<BatchParametersSet, List<Integer>>> runSet: commonRuns){
			// Add the values to the parameter set
			if(newParametersSet == null){
				List<String> parameters = new ArrayList<String>();
				parameters.addAll(runSet.getFirst().keySet());
				newParametersSet = new BatchParametersSet(parameters);
			}
			newParametersSet.addRun(run, runSet.getFirst());
			
			// Create a list of the data sets
			List<SingleDataSet> dataSets = new ArrayList<SingleDataSet>();
			String listOfRuns = "";
			for(BatchParametersSet paramSet: runSet.getSecond().keySet()){
				    for(Integer runNumber: runSet.getSecond().get(paramSet)){
				    	SingleDataSet dataSet = paramMap.get(paramSet).getDataSet(runNumber);
					    dataSets.add(dataSet);
					    listOfRuns += dataSet.id;
				    }
			}
			
			// Now call the merge function
			SingleDataSet[] dataSetArray = new SingleDataSet[dataSets.size()];
			dataSetArray = dataSets.toArray(dataSetArray);
			SingleDataSet newDataSet = SingleDataSet.aggregateDataSets(function, true, dataSetArray);
			
			// Write the data set to the output directory
			if(splitter.getOutputFile(archiveDirectory, baseName, run) == null) System.out.println("NULL WHEN TRYING TO GET " + archiveDirectory + " " + baseName + " " + run);
			
			newDataSet.writeToFile(splitter.getOutputFile(archiveDirectory, baseName, run));
			run++;
			System.out.print(".");
			if(run % 100 == 0) System.out.println (" " + run);
		}
		
		// Write the parameters file to the specified destination
		if(newParametersSet != null) newParametersSet.writeToFile(batchParamsFileName);
	}
	
	public DataSetSource(String batchFileName, String archiveDirectoryName, String baseFileName){
		this(batchFileName, archiveDirectoryName, baseFileName, null);
	}
	
	
	public DataSetSource(String batchFileName, String archiveDirectoryName, String baseFileName, String paramsFileName){
		batchFile           = batchFileName;
		archiveDirectory    = archiveDirectoryName;
		baseName            = baseFileName;
		splitter            = new OutputFileSplitter();
		batchParamsFileName = paramsFileName;
	}
	
	public boolean isInArchive(int runNumber){
		return(new File(splitter.getOutputFile(archiveDirectory, baseName, runNumber))).exists();
	}
	
	public SingleDataSet getDataSet(int runNumber){
	  Set<Integer> runNumbers = new TreeSet<Integer>();
	  runNumbers.add(runNumber);
	  Map<Integer, SingleDataSet> dataSets = getDataSets(runNumbers);
	  if(dataSets.size() > 0){
		  SingleDataSet ret = dataSets.entrySet().iterator().next().getValue();
		  ret.id = this.batchPrefix + "_" + runNumber;
		  return ret;
	  }
	  else return null; 
	}
	
	public BatchParametersSet getParametersSet(){
		return batchParamsFileName == null ? null : new BatchParametersSet(batchParamsFileName);
	}
	
	public void splitAll(){
		splitter.split(batchFile, archiveDirectory, baseName, null, true);
	}
	
	public Map<Integer, SingleDataSet> getDataSets(Set<Integer> runNumbers){
		Map<Integer, SingleDataSet> ret = new TreeMap<Integer, SingleDataSet>();
		
		// If the run is not yet in the archive, will have to write it
		List<Integer> runsToRetrieve = new ArrayList<Integer>();
		for(Integer i: runNumbers) if(!isInArchive(i)) runsToRetrieve.add(i);
		splitter.verbose = true;
		splitter.split(batchFile, archiveDirectory, baseName, runsToRetrieve, true);
		// Now all requested runs should be in the archive
		for(Integer i: runNumbers){
			SingleDataSet toReturn = new SingleDataSet(splitter.getOutputFile(archiveDirectory, baseName, i));
			toReturn.id = this.batchPrefix + "_" + i;
			ret.put(i, toReturn);
		}
		return ret;
	}
	
	
	public static void main(String[] args){
		System.out.println("Running...");
		
		String kind = "USAGE_HISTOGRAM";
		String name = "USAGE_HIST";
		String inFile = "usage_histogram.csv";
		
		DataSetSource dsSourceI  = new DataSetSource("./out/combined_data_2014_09_08/" + inFile, "./out/analysis/2014_Sep_08/" + kind + "/", name, "./out/combined_data_2014_09_08/ModelOutput.2014.Sep.08.02_42_01.batch_param_map.txt");
		dsSourceI.batchPrefix = "SET_1";
		DataSetSource dsSourceII = new DataSetSource("./out/combined_data_2014_09_30/" + inFile, "./out/analysis/2014_Sep_30/" + kind + "/", name, "./out/combined_data_2014_09_30/ModelOutput.2014.Oct.01.04_03_52.batch_param_map.txt");
		dsSourceII.batchPrefix = "SET_2";
		
		DataSetSource dest       = new DataSetSource("./out/analysis/MERGED/merged_params.txt", "./out/analysis/MERGED/" + kind + "/", name, "./out/analysis/MERGED/merged_params.txt");
		
		List<DataSetSource> sources = new ArrayList<DataSetSource>();
		sources.add(dsSourceI);
		sources.add(dsSourceII);
		System.out.println("MERGING...");
		dest.mergeAndCollapse(sources, SingleDataSet.Function.AVERAGE, "randomSeed", "run");
		
		System.out.println("Done.");
		
	}
	
}
