package cnh.crb.analysis.datasets;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;
import java.util.Map.Entry;

public class MonthlyAverageResampler extends DataResampler {

	private boolean useNumberOfDaysInMonth;
	
	public MonthlyAverageResampler(){
		useNumberOfDaysInMonth = false;
	}
	
	public MonthlyAverageResampler(boolean useNumberOfDays){
		useNumberOfDaysInMonth = useNumberOfDays;
	}
	
	public SingleDataSet resample(SingleDataSet dataSet) {
		SingleDataSet ret = new SingleDataSet(dataSet);
		if(dataSet.data.size() == 0) return ret;
		
		// Create a bin for averages
		double[] avg = new double[ret.headers.size()];
		
		int    currentMonth = -1;
		double currentTick  = -1;
		int    countOfDays  = 0;
		Iterator<Entry<Double, List<String>>> iterator = dataSet.data.entrySet().iterator();
		do{
			Entry<Double, List<String>> entry = iterator.next();
			countOfDays++;
			double newTick  = entry.getKey();
			int    newMonth = dataSet.convertTickToCalendar(newTick).get(Calendar.MONTH);
			if(newMonth != currentMonth){
				if(currentMonth > -1){ // Skip this the first time
					if(useNumberOfDaysInMonth) countOfDays = dataSet.convertTickToCalendar(currentTick).getActualMaximum(Calendar.DAY_OF_MONTH);
					for(int i = 0; i < avg.length; i++)	if(!Double.isNaN(avg[i]) && countOfDays > 0) avg[i] = avg[i] / countOfDays;
					List<String> listToAdd = new ArrayList<String>();
					for(int i = 0; i < avg.length; i++) listToAdd.add("" + avg[i]);
					ret.data.put(currentTick, listToAdd);
				}
				for(int i = 0; i < avg.length; i++) avg[i] = 0;
				countOfDays = 0;
			}
			for(int i = 0; i < avg.length; i++){
				if(!Double.isNaN(avg[i])){
					double toAdd = Double.NaN;
					try{
						toAdd = Double.parseDouble(entry.getValue().get(i));
					}
					catch(NumberFormatException E){}
					avg[i] = Double.isNaN(toAdd) ? Double.NaN : avg[i] + toAdd;
				}
			}
			
			currentTick     = newTick;
			currentMonth    = newMonth;
			
		}while(iterator.hasNext());
		// Must make last entry
		if(useNumberOfDaysInMonth) countOfDays = dataSet.convertTickToCalendar(currentTick).getActualMaximum(Calendar.DAY_OF_MONTH);
		for(int i = 0; i < avg.length; i++)	if(!Double.isNaN(avg[i]) && countOfDays > 0) avg[i] = avg[i] / countOfDays;
		List<String> listToAdd = new ArrayList<String>();
		for(int i = 0; i < avg.length; i++) listToAdd.add("" + avg[i]);
		ret.data.put(currentTick, listToAdd);

		return ret;
	}

}
