package cnh.crb.analysis.datasets;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Scanner;
import java.util.TreeMap;

public class OutputFileSplitter {

  boolean verbose = false;
	
  int numberOfDigits = 8;
  
  public String getOutputFile(String outputDirectory, String baseOutputName, int runNumber){
	  String runString = "";
	  for(int i = 0; i < numberOfDigits; i++) runString += "0";
	  runString += runNumber;
	  runString = runString.substring(runString.length() - numberOfDigits, runString.length());
	  return outputDirectory + baseOutputName + runString + ".csv";
  }
  
  public SingleDataSet getRun(String inputFile, int runNumber){
	List<Integer> toRetrieve = new ArrayList<Integer>();
	toRetrieve.add(runNumber);
	Map<Integer, SingleDataSet> ret = getRuns(inputFile, toRetrieve);
	return ret.get(runNumber);
  }
  
  
  public Map<Integer, SingleDataSet> getRuns(String inputFile, List<Integer> runs){
	List<Integer> runsToReturn = new ArrayList<Integer>();
	runsToReturn.addAll(runs);
	SingleDataSet out = null;
	FileInputStream inputStream = null;
	Scanner         sc          = null;
	Map<Integer, SingleDataSet> ret = new TreeMap<Integer, SingleDataSet>();
	if(runs.size() == 0) return ret;
	try {
	  File inFile = new File(inputFile);
	  inputStream = new FileInputStream(inFile);
	  sc = new Scanner(inputStream, "UTF-8");
	  if(!sc.hasNextLine()) return null; // No lines?
	  String headerLineIn = sc.nextLine();
	  String[] headerArray = headerLineIn.replace('"', ' ').trim().split(",");
	  List<String> headerList = new ArrayList<String>();
	  int tickColumn = -1;
	  int runColumn  = -1;
	  for(int i = 0; i < headerArray.length; i++){ 
		    String headerTrimmed = headerArray[i].trim();
			if       (headerTrimmed.compareTo("tick") == 0) tickColumn = i;
			else if  (headerTrimmed.compareTo("tick") == 0) runColumn  = i;
			else                                      headerList.add(headerTrimmed);
	  }
		  
	  // Now loop through the data
	  int currentRunNumber = -1;

	  boolean continueLoop = true;
	  while (sc.hasNextLine() && continueLoop) {
		String[] dataArray = sc.nextLine().replace('"', ' ').trim().split(",");
		int newRunNumber = Integer.parseInt(dataArray[runColumn]);
		if(newRunNumber != currentRunNumber){
		  if(runs.contains(currentRunNumber)){
			ret.put(currentRunNumber, out);
			runsToReturn.remove(new Integer(currentRunNumber));
			if(runsToReturn.size() == 0) continueLoop = false;
		  }
		  if(continueLoop){
			out = new SingleDataSet(headerList, null);
			currentRunNumber = newRunNumber;
			System.out.println();
			System.out.print("Scanning " + currentRunNumber + "...");
		  }
		}
		if(continueLoop){
		  double tick = -1;
		  List<String> dataList = new ArrayList<String>();
		  for(int i = 0; i < dataArray.length; i++){
			  if(i != runColumn){
				if(i == tickColumn) tick = Double.parseDouble(dataArray[i]);
				else                dataList.add(dataArray[i]);
			  }
		  }
		  out.data.put(tick, dataList);
		}
      }
	}
	catch (Exception e) {
	  e.printStackTrace();
	} 
	finally {
	  if (inputStream != null) {
	    try {
		  inputStream.close();
		} 
	    catch (IOException e) {
			e.printStackTrace();
		}
	  }
	  if (sc != null) {
		sc.close();
	  }
	}
    return ret;
  }

  
  public void split(String inputFile, String outputDirectory, String baseOutputName, List<Integer> runs){
	  split(inputFile, outputDirectory, baseOutputName, runs, false);
  }
  public void split(String inputFile, String outputDirectory, String baseOutputName, List<Integer> runs, boolean writeAll){
	FileInputStream inputStream = null;
	Scanner         sc          = null;
	if(runs != null && runs.size() == 0) return;
	try {
	  File inFile = new File(inputFile);
	  inputStream = new FileInputStream(inFile);
	  sc = new Scanner(inputStream, "UTF-8");
	  if(!sc.hasNextLine()) return; // No lines?
	  String headerLineIn = sc.nextLine();
	  String[] headerArray = headerLineIn.replace('"', ' ').trim().split(",");
	  List<String> headerList = new ArrayList<String>();
	  int tickColumn = -1;
	  int runColumn  = -1;
	  for(int i = 0; i < headerArray.length; i++){ // Skip 'run' column, column 0
		  String headerTrimmed = headerArray[i].trim();
		  if      (headerTrimmed.compareTo("tick") == 0) tickColumn = i;
		  else if (headerTrimmed.compareTo("run")  == 0) runColumn  = i;
		  else  headerList.add(headerTrimmed);
	  }
	  
	  if(verbose){
		  System.out.print("RETRIEVING RUNS: ");
		  for(Integer run: runs) System.out.print(run + " ");
		  System.out.println();
	  }
	  
	  // Now loop through the data
	  int currentRunNumber = -1;
	  SingleDataSet out = null;
	  boolean continueLoop = true;
	  while (sc.hasNextLine() && continueLoop) {
		String[] dataArray = sc.nextLine().replace('"', ' ').trim().split(",");
		int newRunNumber = Integer.parseInt(dataArray[runColumn]);
		if(newRunNumber != currentRunNumber){
			if((currentRunNumber != -1) &&
			   ((runs == null || runs.contains(currentRunNumber)) || writeAll)){
				if(!(new File(getOutputFile(outputDirectory, baseOutputName, currentRunNumber))).exists()){
				    out.writeToFile(getOutputFile(outputDirectory, baseOutputName, currentRunNumber));
				    if(verbose) System.out.print(" writing...");
				}
				else if(verbose) System.out.print(" already exists; skipping...");
				if(runs != null){
					runs.remove(new Integer(currentRunNumber));
					if(runs.size() == 0) continueLoop = false;
				}
			}
			out = new SingleDataSet(headerList, null);
			currentRunNumber = newRunNumber;
			if(verbose) System.out.println();
			if(continueLoop) if(verbose) System.out.print("Scanning " + currentRunNumber + "...");
		}		
		if(continueLoop){
			double tick = -1;
			List<String> dataList = new ArrayList<String>();
			for(int i = 0; i < dataArray.length; i++){
				if(i != runColumn){
					if(i == tickColumn) tick = Double.parseDouble(dataArray[i]);
					else                dataList.add(dataArray[i]);
				}
			}
			out.data.put(tick, dataList);
	    }
	   }
	   if(continueLoop){ // Has reached last line of file
	     if(!(new File(getOutputFile(outputDirectory, baseOutputName, currentRunNumber))).exists()){
		    out.writeToFile(getOutputFile(outputDirectory, baseOutputName, currentRunNumber));
		    if(verbose) System.out.print(" writing...");
		 }
     }
	   if(verbose) System.out.println("Done loading.");
	}
    catch (Exception e) {
		e.printStackTrace();
	} 
	finally {
	  if (inputStream != null) {
	        try {
				inputStream.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
	    }
	    if (sc != null) {
	        sc.close();
	    }
	}
	System.out.println("DONE.");

  }
  
  public static void main(String[] args){
//	  BatchParametersSet batchParams = new BatchParametersSet("./out/combined_data_2014_09_08/ModelOutput.2014.Sep.08.02_42_01.batch_param_map.txt");
//	  List<Integer> runsOfInterest = batchParams.getRunsWithMatchingValue("priceWeight", "0.025");
	  List<Integer> runsOfInterest = new ArrayList<Integer>();
	  runsOfInterest.add(111);
	  OutputFileSplitter sof = new OutputFileSplitter();
	  sof.split("./out/combined_data_2014_09_08/ModelOutput.2014.Sep.08.02_42_01.txt", "./out/analysis/2014_09_08/", "SEP_08_2014", runsOfInterest);
  }
	
}
