package cnh.crb.analysis.datasets;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;


public class RollingAverageResampler extends DataResampler{

	int window;
	
	public RollingAverageResampler(int windowSize){
		window = windowSize;
	}
	
	public SingleDataSet resample(SingleDataSet dataSet){
		SingleDataSet ret = new SingleDataSet(dataSet);
		Map<String, List<Double>> calcArrays    = new TreeMap<String, List<Double>>();
		Map<String, Double>       currentTotals = new TreeMap<String, Double>();
		for(String header: ret.headers){
			calcArrays.put(header, new ArrayList<Double>());
			currentTotals.put(header, 0.0);
		}
		for(Entry<Double, List<String>> dataLine: dataSet.data.entrySet()){
			List<String> nextLineToAdd = new ArrayList<String>();
			for(int i = 0; i < dataSet.headers.size(); i++){
				String header = dataSet.headers.get(i);
				String value = dataLine.getValue().get(i);
				double val = Double.NaN;
				try{
					val = Double.parseDouble(value);
				}
				catch(NumberFormatException E){} // Should throw an error; only numeric columns should be in original data set
				List<Double> calcArray = calcArrays.get(header);
				calcArray.add(val);
				double currentValue = currentTotals.get(header);
				currentValue += val;
				currentTotals.put(header, currentValue);
				if(calcArray.size() == window){
					double rollingAverage = currentValue / (double)window;
					nextLineToAdd.add("" + rollingAverage);
					// Remove the oldest from the list, grabbing the value
					Double popped = calcArray.remove(0);
					// Remove that from the current total
					currentValue -= popped;
					currentTotals.put(header, currentValue);
				}
			}
			if(nextLineToAdd.size() > 0) ret.data.put(dataLine.getKey(), nextLineToAdd);
		}
		return ret;
	}
	
}
