package cnh.crb.climate;

import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;

public class Climate {

	private class ClimateYear{
	  double annualTemperature        = 0;                                                  // Units are degrees C
	  double[] monthlyTemperature     = new double[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
	
	  double annualPrecipitation      = 0 ;                                                 // Units are mm
	  double[] monthlyPrecipitation   = new double[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
	
	  public ClimateYear(){ }
	  
	  public ClimateYear(double annualTemp, double annualPrecip, double[] monthlyTemp, double[] monthlyPrecip){
		  annualTemperature    = annualTemp;
		  annualPrecipitation  = annualPrecip;
		  monthlyTemperature   = monthlyTemp;
		  monthlyPrecipitation = monthlyPrecip;
	  }
	}
	
	ClimateYear               baseline     = new ClimateYear();
	Map<Integer, ClimateYear> climateYears = new HashMap<Integer, ClimateYear>();
	Calendar                  calendar;
	
	
	public Climate(){}
	
	public Climate(Calendar cal){
		calendar = cal;
	}
	
	
	// Construction
	public void setBaseline(double annualTemp, double annualPrecip, double[] monthlyTemp, double[] monthlyPrecip){
		baseline = new ClimateYear(annualTemp, annualPrecip, monthlyTemp, monthlyPrecip);
	}

	public void addYear(int year, double annualTemp, double annualPrecip, double[] monthlyTemp, double[] monthlyPrecip){
		climateYears.put(year, new ClimateYear(annualTemp, annualPrecip, monthlyTemp, monthlyPrecip));
	}
	
	
	// Update	
	public void setCalendar(Calendar newDate){
		calendar = newDate;
	}
	
	
	// Retrieve values
	public double getBaselineAnnualTemperature(){
		return baseline.annualTemperature;
	}
	
	public double getBaselineMonthlyTemperature(Calendar cal){
		return baseline.monthlyTemperature[cal.get(Calendar.MONTH)];
	}
	
	public double getBaselineAnnualPrecipitation(){
		return baseline.annualPrecipitation;
	}

	public double getBaselineMonthlyPrecipitation(Calendar cal){
		return baseline.monthlyPrecipitation[cal.get(Calendar.MONTH)];
	}
	

	public double getAnnualTemperature(){
		return getAnnualTemperature(calendar);
	}
	
	public double getAnnualTemperature(Calendar cal){
		ClimateYear year = climateYears.get(cal.get(Calendar.YEAR));
		return year != null ? 
				year.annualTemperature : 
				baseline.annualTemperature;
	}
	
	
	public double getMonthlyTemperature(){
		return getMonthlyTemperature(calendar);
	}
	
	public double getMonthlyTemperature(Calendar cal){
		ClimateYear year = climateYears.get(cal.get(Calendar.YEAR));
		return year != null ? 
				year.monthlyTemperature[cal.get(Calendar.MONTH)] : 
				baseline.monthlyTemperature[cal.get(Calendar.MONTH)];
	}
	
	public double getAnnualPrecipitation(){
		return getAnnualPrecipitation(calendar);
	}
	
	public double getAnnualPrecipitation(Calendar cal){
		ClimateYear year = climateYears.get(cal.get(Calendar.YEAR));
		return year != null ? 
				year.annualPrecipitation : 
				baseline.annualPrecipitation;
	}

	public double getMonthlyPrecipitation(){
		return getMonthlyPrecipitation(calendar);
	}
	
	public double getMonthlyPrecipitation(Calendar cal){
		ClimateYear year = climateYears.get(cal.get(Calendar.YEAR));
		return year != null ? 
				year.monthlyPrecipitation[cal.get(Calendar.MONTH)] : 
			    baseline.monthlyPrecipitation[cal.get(Calendar.MONTH)];
	}
	
}
