package cnh.crb.climate;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;

import javax.xml.namespace.QName;
import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.EndElement;
import javax.xml.stream.events.StartElement;
import javax.xml.stream.events.XMLEvent;

/**
 * Loads the SeasonalOffsets from an XML file.
 * 
 * @author nick
 */
public class ClimateLoader {
	
	private static final QName BASELINE     = new QName("baseline");
	private static final QName YEAR         = new QName("year");
	private static final QName MONTH        = new QName("month");
	private static final QName DATE         = new QName("date");
	private static final QName AVG_TEMP     = new QName("annualAvgTemp");
	private static final QName ANN_PRECIP   = new QName("annualPrecipitation");
	private static final QName TEMP         = new QName("temp");
	private static final QName PRECIP       = new QName("precipitation");

	
	public Climate load(String file) throws IOException, XMLStreamException {
		Climate climate = new Climate();
		
		XMLInputFactory factory = XMLInputFactory.newInstance();
		XMLEventReader reader = null;
		int i                 = 0;
		int year              = -1;
        double annualTemp     = 0;
        double annualPrecip  = 0;
        double[] temps      = new double[12];
        double[] precip     = new double[12];
        
		try (BufferedReader br = new BufferedReader(new FileReader(file))){
			reader = factory.createXMLEventReader(br);
			while (reader.hasNext()) {
			      XMLEvent evt = reader.nextEvent();
			      if (evt.isStartElement()) {
			    	  StartElement elm = evt.asStartElement();
			          QName elmName = elm.getName();
			          if(elmName.equals(BASELINE) || elmName.equals(YEAR)){
			        	  i = 0;
			        	  year = elmName.equals(BASELINE) ? -1 : Integer.parseInt(elm.getAttributeByName(DATE).getValue());
				          annualTemp    = Float.parseFloat(elm.getAttributeByName(AVG_TEMP).getValue());
				          annualPrecip  = Float.parseFloat(elm.getAttributeByName(ANN_PRECIP).getValue());
				          temps  = new double[12];
				          precip = new double[12];
			          }
			          else if(elmName.equals(MONTH)){
			        	  temps[i]  = Float.parseFloat(elm.getAttributeByName(TEMP).getValue());
			        	  precip[i] = Float.parseFloat(elm.getAttributeByName(PRECIP).getValue());
			        	  i++;
			          }
			      } 
			      if(evt.isEndElement()){
			    	  EndElement elm = evt.asEndElement();
			    	  QName elmName = elm.getName();
			    	  if(elmName.equals(BASELINE)){
			    		  climate.setBaseline(annualTemp, annualPrecip, temps, precip);
			    	  }
			    	  else if(elmName.equals(YEAR)){
			    		  climate.addYear(year, annualTemp, annualPrecip, temps, precip);
			    	  }
			      }
			 }
		} finally {
			if (reader != null) try {
				reader.close();
			} catch (Exception ex) {}
		}
		return climate;
	}
	
}
