package cnh.crb.econ;

import java.util.Date;

import cnh.crb.hydro.Tap;


/**
 * Generic interface for any class that can take an
 * amount of water consumed and convert it into a bill.
 * Required input is the number of days and the CCF consumed;
 * calculates the amount the customer will be billed.
 * 
 * @author murphy
 *
 */
public interface BillCalculator {

	/**
	 * Gets the bill for the specified number of days and water consumption. 
	 * 
	 * @param startDate the starting date for the service period to calculate the bill for
	 * @param numberOfDays the number of days in the service period for this bill
	 * @param ccf
	 * 
	 * @return the bill for the specified number of days and water consumption. 
	 */
	public Bill getBill(Date startDate, Tap.Usage usage, BillableEntity entity);
	
}
