/**
 * 
 */
package cnh.crb.econ;

import java.util.Date;

/**
 * Creates a SimpleCostCalculator given rate and charge information. 
 * 
 * @author nick
 */
public class CostCalculatorFactory {

	private SimpleCostCalculator calc = new SimpleCostCalculator();
	
	/**
	 * Adds an entry to the cost calculator. The date is the start of the
	 * period for which the value will be returned. The non-inclusive end of the period
	 * is the date of the nearest entry that is greater than this one. 
	 * 
	 * @param date
	 * @param value
	 */
	public void addEntry(Date date, double value) {
		this.addEntry(date, value, Double.NEGATIVE_INFINITY);
	}
	
	/**
	 * Adds an entry to the cost calculator. The date is the start of the
	 * period for which the value will be returned. The non-inclusive end of the period
	 * is the date of the nearest entry that is greater than this one. 
	 * 
	 * @param date
	 * @param value
	 * @param threshold the amount that must be exceeded before any cost 
	 * is incurred.
	 */
	public void addEntry(Date date, double value, double threshold) {
		calc.valueMap.put(date, new SimpleCostCalculator.CostData(value, threshold));
	}
	
	/**
	 * Creates a calculator containing all the entries
	 * added between calls to createCalculator. Any
	 * calls to addEntry subsequent to calling this will
	 * create a new calculator. 
	 * 
	 * @return a calculator containing all the entries
	 * added between calls to createCalculator.
	 */
	public CostCalculator createCalculator() {
		CostCalculator tmp = calc;
		calc = new SimpleCostCalculator();
		return tmp;
	}
}
