/**
 * 
 */
package cnh.crb.econ;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Map;

import cnh.crb.hydro.Tap;
import cnh.crb.hydro.UnitsAndConverters.VolumeUnits;

/**
 * @author nick
 *
 */
public class HouseholdBillCalculator implements BillCalculator{
	
	List<CostCalculator> calculatorList = new ArrayList<CostCalculator>();
	
	public HouseholdBillCalculator() {	}

	/**
	 * Adds a cost calculator to this bill. CostCalculators are all
	 * independent and additive.
	 * @param costCalculator
	 */
	public void addCalculator(CostCalculator costCalculator){
		calculatorList.add(costCalculator);
	}
	
	/**
	 * Calculates the bill for the specified usage..
	 * 
	 * @param usage
	 * 
	 * @return the calculated bill.
	 */
	public Bill getBill(Date startDate, Tap.Usage usage, BillableEntity entity) {
		double amountUsed = usage.getVolumeUsed(VolumeUnits.CCF);
		Date date = usage.getStartDate();
		double cost = 0;
		for(CostCalculator calculator: calculatorList) cost += calculator.calculateCost(date, amountUsed, entity);
		return new Bill(date, usage.getDays(), amountUsed, cost);
	}

//	private double calculateCost(Date date, double surchargeAvg, double sewerAvg, double amtUsed) {
//		cal.setTime(date);
//		int month = cal.get(Calendar.MONTH);
//		double cost = rateCalc.calculateCost(date, amtUsed);
//		// 1 because it's not a per unit charge
//		cost += monthlyChargeCalc.calculateCost(date, 1);
//		cost += capCalc.calculateCost(date, amtUsed);
//		cost += conserveCalc.calculateCost(date, amtUsed);
//		// 1 because not per unit
//		cost += sewerMonthlyChargeCalc.calculateCost(date, 1);
//		// apply the summer surcharge
//		if (month >= Calendar.MAY && month <= Calendar.OCTOBER)
//			cost += surchargeCalc.calculateCost(date, surchargeAvg, amtUsed);
//		cost += sewerRateCalc.calculateCost(date, sewerAvg);
//		return cost;
//	}

}
