package cnh.crb.econ;

import repast.simphony.util.collections.Pair;


/**
 * Block rate calculator that bills at the rate of the highest threshold
 * under which the ccf falls. 
 */
public class IncreasingStepRateCostCalculator extends BlockRateCostCalculator {

	public IncreasingStepRateCostCalculator(double[] thresholds, double[] rates, double flatFee){
		super(thresholds, rates, flatFee);
	}
	
	public IncreasingStepRateCostCalculator(double[] thresholds, double[] rates){
		super(thresholds, rates, 0.0);
	}
	
	
	protected double amount(double ccf) {
		
		double thresholdSum = 0;
		for (Pair<Double, Double> rateBlock : rateBlocks) {
			double threshold = rateBlock.getFirst();
			thresholdSum += threshold;
			if (ccf <= thresholdSum) return ccf * rateBlock.getSecond() + baseFee;
		}
		
		// if gets here then ccf is infinite
		throw new IllegalArgumentException("Bad ccf value: " + ccf);
	}
}
