/**
 * 
 */
package cnh.crb.econ;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Map;

/**
 * CostCalculator that has multiple different rates for different
 * years and months.
 * 
 * @author nick
 */
public class MultiSeasonCostCalculator implements CostCalculator {
	
	private Map<Integer, SeasonalCostCalculator> calcs = new HashMap<>();
	private Calendar cal = GregorianCalendar.getInstance();
	
	/**
	 * Adds a seasonal calculator to be used to calculate the bill for the specified years.
	 * 
	 * @param calc
	 * @param years
	 */
	public void addSeasonalCalculator(SeasonalCostCalculator calc, int...  years) {
		for (int year : years) {
			calcs.put(year, calc);
		}
	}

	/**
	 * Calculates the cost for the specified date and amount.
	 * 
	 * @param date
	 * @param amt
	 * 
	 * @return the cost for the specified date and amount.
	 */
	@Override
	public double calculateCost(Date date, double amt, BillableEntity entity) {
		cal.setTime(date);
		int year = cal.get(Calendar.YEAR);
		SeasonalCostCalculator calc = calcs.get(year);
		if (calc == null)
			throw new IllegalArgumentException("Missing calculator for year: " + year);
		return calc.calculateCost(date, amt, entity);
	}
}
