package cnh.crb.econ.micro;



public class CombinedMarginalCostCurve extends AbstractMarginalCostCurve {

  protected MarginalCostCurve curve1;
  protected MarginalCostCurve curve2;
    
  public CombinedMarginalCostCurve(MarginalCostCurve c1, MarginalCostCurve c2){
    curve1 = c1;
    curve2 = c2;
  }
  
  public double getMarginalCostAt(double amount) {
    double[] allotments = getLowestCostAllotments(amount, true);
    return Math.min(allotments[4], allotments[5]);
  }

  public double getTotalCostUpTo(double amount) {
    double[] allotments = getLowestCostAllotments(amount);
    return allotments[2] + allotments[3];
  }
  
  private double[] getLowestCostAllotments(double amount){
    return getLowestCostAllotments(amount, false);
  }
  
  private double[] getLowestCostAllotments(double amount, boolean doMarginalCost){
    double[] ret = new double[6];
    
    double precision = amount / 1000;
    
    double x = amount;
    double y = 0;
    double minCost = Double.POSITIVE_INFINITY;
    while(x > 0){
      double cost1 = curve1.getCostOfNUnitsAt(0, x);
      double cost2 = curve2.getCostOfNUnitsAt(0, y);
      double cost = cost1 + cost2;
      if(cost < minCost){
        ret[0] = x;
        ret[1] = y;
        ret[2] = cost1;
        ret[3] = cost2;
        minCost = cost;
      }
      x -= precision;
      y += precision;
    }
    if(doMarginalCost){
      ret[4] = curve1.getMarginalCostAt(ret[0]);
      ret[5] = curve2.getMarginalCostAt(ret[1]);
    }
    else{
      ret[4] = -1;
      ret[5] = -1;
    }
    return ret;
  }
  

  public double getAmountAtWhichMarginalCostExceeds(double start, double minCost) {
    
    return 0;
  }

  public String getReport() {
    // TBD
    return null;
  }

  public double getPurchasingPowerOf(double amount, double start) {
    return 0;
  }

}
