package cnh.crb.econ.micro;


public class ConstantMarginalCostCurve extends AbstractMarginalCostCurve{

  protected double baseFee;
  protected double rate;
  
  public ConstantMarginalCostCurve(double ratePerUnit){
    baseFee = 0;
    rate    = ratePerUnit;
  }
  
  public ConstantMarginalCostCurve(double fee, double ratePerUnit){
    baseFee = fee;
    rate    = ratePerUnit;
  }
  
  public double getMarginalCostAt(double amount){
    return (amount > 0 ? rate : baseFee + rate);  // Cost of the first unit is the flat fee plus the rate
  }
  
  public double getTotalCostUpTo(double amount){    
    return (amount > 0 ? baseFee + amount * rate : 0);
  }

  public double getAmountAtWhichMarginalCostExceeds(double start, double minCost){
    return(start == 0 ?                          // If the start point is zero 
              (baseFee > minCost ? 0 :           // then if the base fee is greater than the min Cost, return 0
                  (rate > minCost? 0 : -1)) :    // Otherwise, if the rate is greater than the min cost, return 0
              (rate > minCost ? start : -1));    // If starting past zero, if the rate is greater than the min cost, return the rate
  }
  
  public String getReport(){
    return "FEE: " + baseFee + " RATE: " + rate + " PER UNIT" + System.getProperty("line.separator");
  }
  
  public double getPurchasingPowerOf(double amount, double start){
    return (start == 0 ? 
                (amount > baseFee ? 0 : (amount - baseFee) / rate) :
                (amount / rate) );
  }
  
}
