package cnh.crb.econ.micro;


public class RunningTab {

  protected MarginalCostCurve curve;
  protected double            currentAmount;
  
  public RunningTab(MarginalCostCurve costCurve){
    curve         = costCurve;
    currentAmount = 0;
  }
  
  public double getNextUnitCost(){
    return curve.getMarginalCostAt(currentAmount);
  }
  
  // Returns the amount that can be added to this running tab
  // that will be at a lower marginal cost than if added to the other
  // tab provided
  public double getAmountAddedAtLowerCost(double amount, RunningTab other){
    double intersection = curve.getAmountAtWhichMarginalCostExceeds(currentAmount, other.getNextUnitCost());
    return (intersection == -1 ? // Marginal cost on this tab never exceeds current cost on other tab 
                amount                                            // All of it 
              : Math.min(amount,  intersection - currentAmount)); // If the amount that could be added > amount to be added, add it all
  }
 
  public double purchase(double amount){
    double ret = curve.getCostOfNUnitsAt(currentAmount, amount);
    currentAmount += amount;
    return ret; // Returns the cost incurred
  }
  
  public void reset(){
    currentAmount = 0;
  }
  
}
