package cnh.crb.econ.micro;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;

public class RunningTabCollection {

  public class SourcedOrder{
    public double   amount;
    public String[] sources;
    
    public SourcedOrder(double amt, String[] srces){
      amount  = amt;
      sources = srces;
    }
    
    public int size(){ return sources.length; }
  }

  protected Map<String, RunningTab> sources;
  
  public RunningTabCollection(){ }
  
  public void addSource(String sourceName, MarginalCostCurve curve){
    sources.put(sourceName, new RunningTab(curve));
  }
  
  public void resetAll(){
    for(RunningTab tab : sources.values()) tab.reset();
  }
  
  // Consumes the amount specified at the lowest cost
  // Assumes all sources are valid
  public void consume(double amount){
    consume(new SourcedOrder(amount, sources.keySet().toArray(new String[0])));  
  }
  
  // Consumes the amount specified at the lowest cost,
  // assuming the source must be one of the sources listed
  public void consume(SourcedOrder order){
    List<RunningTab> possibleSources = new ArrayList<RunningTab>();
    for(String source : order.sources){
      if(sources.containsKey(source)) possibleSources.add(sources.get(source));
    }
    // Now find the one with the minimum marginal cost
    
    
    // Must deal with ties...
  }
  
  public void consume(List<SourcedOrder> orders){
    // Sort the list by order of the number of sources;
    // Must deal with those with lowest degrees of freedom first
    Collections.sort(orders, 
        new Comparator<SourcedOrder>(){ 
           public int compare(SourcedOrder so1, SourcedOrder so2){ 
             return so1.size() < so2.size() ? -1 : so2.size() < so1.size() ? 1: 0; }});
    
    for(SourcedOrder sourcedOrder : orders) consume(sourcedOrder);
  }
  
  
  
}
