package cnh.crb.econ.micro;

import java.util.ArrayList;
import java.util.List;

public class VaryingMarginalCostCurve extends AbstractMarginalCostCurve{

  public class MarginalCostCurveSegment implements MarginalCostCurve{
    double            min;
    double            max;
    MarginalCostCurve curve;
    double            totalCost;
    
    public MarginalCostCurveSegment(double minAmount, double maxAmount, MarginalCostCurve marginalCostCurve){
      min       = minAmount;
      max       = maxAmount;
      curve     = marginalCostCurve;
      totalCost = curve.getTotalCostUpTo(max - min); 
    }
    
    public double getMarginalCostAt(double amount){
      return curve.getMarginalCostAt(amount - min);
    }
    
    public double getTotalCostUpTo(double amount){
      return curve.getTotalCostUpTo(amount - min);
      
    }
    
    public double getCostOfNUnitsAt(double start, double N){
      return curve.getCostOfNUnitsAt(start - min, N);
    }
    
    public double getTotalCost(){
      return totalCost;
    }
    
    public String getReport(){
      return "MIN: " + min + " MAX: " + max + " " + curve.getReport();
    }
    
    public double getPurchasingPowerOf(double amount, double start){
      return curve.getPurchasingPowerOf(amount, start - min);
    }
    
    public double getAmountAtWhichMarginalCostExceeds(double start, double minCost){
      return curve.getAmountAtWhichMarginalCostExceeds(start - min, minCost);
    }

  }
  
  List<MarginalCostCurveSegment> segments = new ArrayList<MarginalCostCurveSegment>();
  
  public VaryingMarginalCostCurve(){ }
  
  public void addMarginalCostCurveSegment(double min, double max, MarginalCostCurve curve){
    MarginalCostCurveSegment segment = new MarginalCostCurveSegment(min, max, curve);
    // Check to see that segments do not overlap...
    segments.add(segment);
  }
  
  public double getMarginalCostAt(double amount){
    if(segments.size() == 0) return 0;
    int i = 0;
    while(i < (segments.size() - 1) && segments.get(i).max < amount) i++;
    return segments.get(i).getMarginalCostAt(amount);
  }
  
  public double getTotalCostUpTo(double amount){
    if(segments.size() == 0) return 0;
    double ret = 0;
    int i = 0;
    while(i < (segments.size() - 1) && segments.get(i).max < amount){
      ret += segments.get(i).getTotalCost();
      i++;
    }
    ret += segments.get(i).getTotalCostUpTo(amount);
    return ret;
  }

  public double getAmountAtWhichMarginalCostExceeds(double start, double minCost){
    if(segments.size() == 0) return -1;
    int i = 0;
    while(i < (segments.size() - 1) && segments.get(i).max < start) i++;
    while(i < (segments.size() - 1)){
      double ret = segments.get(i).getAmountAtWhichMarginalCostExceeds(start, minCost); 
      if(ret > -1) return ret;
    }
    return segments.get(i).getAmountAtWhichMarginalCostExceeds(start, minCost);
  }
  
  public double getPurchasingPowerOf(double amount, double start){
    if(segments.size() == 0) return -1;
    double totalToSpend = amount + getTotalCostUpTo(start);
    double amountPurchased = 0;
    int i = 0;
    while(i < (segments.size() - 1) && (totalToSpend > 0) ){
      if(segments.get(i).getTotalCost() < totalToSpend){
        amountPurchased += (segments.get(i).max - segments.get(i).min);
        totalToSpend -= segments.get(i).getTotalCost();
      }
    }
    return amountPurchased;
    
  }
  
  public String getReport(){
    String ret = "";
    for(MarginalCostCurveSegment segment: segments){
      ret += segment.getReport();
    }
    return ret;
  }
  
}
