package cnh.crb.farm;

import java.util.Date;

import cnh.crb.WaterUseAgent;
import cnh.crb.hydro.DemandRequest;
import cnh.crb.hydro.FlowHydrograph;
import cnh.crb.hydro.FlowManager;
import cnh.crb.hydro.FlowRate;
import cnh.crb.hydro.FlowRate.FlowRateUnits;
import cnh.crb.hydro.SourceFlowNode;
import cnh.crb.hydro.UnitsAndConverters.VolumeUnits;
import cnh.crb.wbm.WBMCrop;
import cnh.crb.wbm.WBMNaturalWaterSource;

public class IrrigatedAgFarmer implements WaterUseAgent{

	private   Crop            crop;
	private   WBMNaturalWaterSource groundwaterSource;
	private   boolean         ready                  = false;
	protected SourceFlowNode  irrigationSource;
	protected DemandRequest   expectedInflow;
	private   FlowHydrograph  waterUseHydrograph     = new FlowHydrograph();
	protected double          currentDemand          = 0; // Running total since last Flow
	
	public IrrigatedAgFarmer(Crop crop){
		this.crop = crop;
	}

	public void addWaterSource(SourceFlowNode src) {
		irrigationSource = src;
	}

	public void setGroundWaterSource(WBMNaturalWaterSource gSource){
		groundwaterSource = gSource;
	}
	
	public void registerSupplyToBeReceived(SourceFlowNode upstreamNode,
			FlowHydrograph supplyToBeReceived,
			FlowHydrograph additionalSupplyAvailable, FlowManager flowManager) {
		if (expectedInflow == null) {
			expectedInflow = new DemandRequest(supplyToBeReceived, additionalSupplyAvailable);
			ready = false;
			flowManager.markNonReady(this);
		} else {
			if (expectedInflow.registerSupply(supplyToBeReceived, additionalSupplyAvailable)) {
				ready = false;
				flowManager.markNonReady(this);
			}
		}
	}

	public boolean isReadyForFlow() {
		return ready;
	}

	public boolean prepareFlow(Date start, Date end, FlowManager flowManager) {
		if (ready)
			return true;
		if(irrigationSource != null){
    		if (expectedInflow == null) {
				FlowHydrograph demand = getDemand(start, end);
				irrigationSource.registerDemandRequest(this, demand, flowManager);
				expectedInflow = new DemandRequest(demand, null);
			}
		}
		ready = true;
		return ready;
	}

	public void clearFlow(){
		currentDemand = 0;
		ready = false;
		expectedInflow = null;
		waterUseHydrograph = new FlowHydrograph();
	}

	public void flow(Date end) {
		if(irrigationSource != null){
			double totalInflowInM3 = expectedInflow.hydrograph1.getTotalFlowVolume(VolumeUnits.M3);
			// Apply water to Crop
			crop.applyWater(totalInflowInM3, VolumeUnits.M3);
			clearFlow();
		}
		else{
			System.out.println("APPLYING WATER FROM NOWHERE");
			crop.applyWater(crop.getDemand(VolumeUnits.M3), VolumeUnits.M3); // Equivalent to coming from unsustainable supply
		}
	}

	public String getId() {
		return "IAG";
	}

	public FlowHydrograph getDemand(Date start, Date end){
		if(irrigationSource != null){
    		double demand = crop.getDemand(VolumeUnits.M3); // Convert to per second
			FlowHydrograph use = new FlowHydrograph(start, end, demand  / (60 * 60 * 24),
					FlowRate.FlowRateUnits.M3S);
			waterUseHydrograph.add(use);
			if(demand > 0){
				System.out.println("On " + start + " Farmer needs: " + demand + " m^3 for crop " + ((WBMCrop)crop).id);
				System.out.println(use.report(FlowRateUnits.M3S));
			}
			return waterUseHydrograph.resetBounds(start, end);
		}
		return null; // Do nothing if there is no irrigation source
	}
	
	
	
}
