/**
 * 
 */
package cnh.crb.household;

import repast.simphony.distribution.RSDistribution;
import repast.simphony.parameter.Parameters;
import cnh.crb.CNHConstants;
import cnh.crb.household.Household.HouseholdData;

/**
 * Default household data initializer that uses Parameters to set the household
 * data properties.
 * 
 * @author NC
 */
public class DefaultHouseholdInitializer implements HouseholdDataInitializer {

	private Parameters params;

	public DefaultHouseholdInitializer(Parameters params) {
		this.params = params;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * cnh.crb.household.HouseholdPropertyInitializer#initialize(cnh.crb.household
	 * .Household.HouseholdData)
	 */
	@Override
	public void initialize(HouseholdData data) {
		
		// Social factor from distribution
		double rawSocialFactors = ((RSDistribution)params.getValue(CNHConstants.SOCIAL_FACTOR_DIST_PARAMETER)).nextDouble(); 
		data.socialFactors = rawSocialFactors > 1 ? 1 : rawSocialFactors < -1 ? -1 : rawSocialFactors; 
		
		
		data.priceWeight = params.getDouble(CNHConstants.PRICE_WEIGHT_PARAMETER);
		data.exogenousEffect = params.getDouble(CNHConstants.EXO_EFFECTS_PARAMETER);
		data.exogenousWeight = params.getDouble(CNHConstants.EXO_WEIGHT_PARAMETER);
		data.socialWeight = params.getDouble(CNHConstants.SOCIAL_WEIGHT_PARAMETER);
		RSDistribution distribution = (RSDistribution)params.getValue(CNHConstants.WATER_USE_DIST_PARAMETER);
		double baselineUse  = 0;
		if (distribution.getName().startsWith("Beta_")){
			double betaOutput = ((RSDistribution)params.getValue(CNHConstants.WATER_USE_DIST_PARAMETER)).nextDouble();
			baselineUse = params.getDouble(CNHConstants.BETA_WATERUSE_MAX )* betaOutput;
		}
		else { // Note, that this branch below doesn't use the beta water use max parameter
			baselineUse = ((RSDistribution)params.getValue(CNHConstants.WATER_USE_DIST_PARAMETER)).nextDouble();
			// TODO fix hack
			baselineUse = baselineUse < 0 ? 10 : baselineUse;
		}
		data.baselineWaterUse = baselineUse;
		data.priceMemSize = params.getInteger(CNHConstants.PRICE_MEM_SIZE);
		data.maxPriceFactorIncrease = params.getDouble(CNHConstants.MAX_PRICE_FACTOR_INCREASE);
		data.priceFactorIncreaseScale = params.getDouble(CNHConstants.PRICE_FACTOR_INCREASE_SCALE);
		data.priceFactorDecreaseDecay = params.getDouble(CNHConstants.PRICE_FACTOR_DECREASE_DECAY);
		data.priceFactorBaseDecay = params.getDouble(CNHConstants.PRICE_FACTOR_BASE_DECAY);
		data.lowFlowFixtureWeight = params.getDouble(CNHConstants.LOW_FLOW_WEIGHT);
		data.useFullWaterFlow = params.getBoolean("useFullWaterFlow");
	}
}
