package cnh.crb.household;

import java.util.LinkedList;

import simphony.util.messages.MessageCenter;
import cnh.crb.econ.Bill;

/**
 * A sliding window of bills.
 * 
 * @author murphy
 * 
 */
public class HouseholdWaterBillMemory {
	
	static MessageCenter msg = MessageCenter
			.getMessageCenter(HouseholdWaterBillMemory.class);

	private LinkedList<Bill> bills = new LinkedList<>();
	private int maxSize;
	
	public HouseholdWaterBillMemory(int size) {
		this.maxSize = size;
	}

	// Assumes each added date will be later...
	public void recordBill(Bill bill) {
		bills.add(bill);
		if (bills.size() > maxSize) bills.pop();
	}

	/**
	 * Gets the average unit price of the specified previous number of bills
	 * counting back from the most recent bill.
	 * 
	 * @param numBills
	 * 
	 * @return the average unit price
	 */
	public double getAverageUnitPrice(int numBills) {
		if (bills.size() < numBills) return 0; // Cannot calculate; will not do anything until record is established

		int    i     = bills.size() - 1;
		double sum   = 0;
		double count = 0;

		while(i >= 0 && count < numBills) {
			sum += bills.get(i).getUnitPrice();
			count++;
			i--;
		}
		
		return sum / numBills;
	}
	
	public double getLastBillCost(){
		if (bills.isEmpty()) return 0;
		return bills.getLast().getBillAmount();
	}
	
	/**
	 * Gets the usage in CCF from the last bill.
	 * 
	 * @return the usage in CCF from the last bill.
	 */
	public double getLastBillUsage() {
		if (bills.isEmpty()) return 0;
		return bills.getLast().getConsumptionInCCF();
	}
	
	
	public double getAverageCost(int numBills){
		if (bills.size() < numBills) return 0; // Cannot calculate; will not do anything until record is established

		int    i     = bills.size() - 1;
		double sum   = 0;
		double count = 0;

		while(i >= 0 && count < numBills) {
			sum += bills.get(i).getBillAmount();
			count++;
			i--;
		}

		msg.debug("GETTING AVERAGE COST: " + bills.size() + " " + sum/numBills);
		return sum / numBills;
	}
	
//	// No decay, just max to be recalled
//	public void forget(int maxRecall) {
//		forget(maxRecall, maxRecall, 1);
//	}

	/**
	 * Causes the agent to 'forget' bills, generally keeping the last N bills
	 * where N is the maxRecall parameter. Bills older than the N bills are
	 * deleted from both the dates list and the record of bills. If
	 * minPerfectRecall and decayRate are specified, then the routine guarantees
	 * that the first n bills will be kept (minPerfectRecall = n), but after
	 * that and up to N there is a probability that the bill will be forgotten.
	 * The probability of being kept (1 - prob of being forgotten)is initially
	 * decayRate, which is then multiplied with each bill processed in the list,
	 * so that if the probability of forgetting the first is .9, the next would
	 * be .81, etc.
	 * 
	 * @param minPerfectRecall
	 * @param maxRecall
	 * @param decayRate
	 */
//	public void forget(int maxRecall, int minPerfectRecall, double decayRate) {
//		int i = dates.size() - 1;
//		int count = 0;
//		double prob = 1;
//		Uniform forgetProb = RandomHelper.createUniform();
//		while ((i >= 0)) {
//			count++;
//			if (count > minPerfectRecall)
//				prob *= decayRate;
//			if (count > maxRecall) {
//				record.remove(dates.get(i));
//				dates.remove(i); // Going backwards this is OK...
//			} else if (count > minPerfectRecall && (prob == 1 || forgetProb.nextDouble() > prob)) { // Note:
//																									// prob
//																									// ==
//																									// 1
//																									// is
//																									// for
//																									// performance;
//																									// avoid
//																									// the
//																									// meaningless
//																									// random
//																									// throw
//				record.remove(dates.get(i)); // Note: OK if this doesn't exist
//			}
//			i--;
//		}
//	}

}
