/**
 * 
 */
package cnh.crb.household;

import java.io.BufferedWriter;
import java.io.DataOutputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import org.apache.commons.collections15.Predicate;
import org.opengis.referencing.operation.TransformException;

import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.parameter.ParametersCreator;
import simphony.util.messages.MessageCenter;
import cnh.crb.climate.Climate;
import cnh.crb.climate.ClimateImpacts;
import cnh.crb.household.Household.HouseholdData;

/**
 * Creates a neighborhood file for households.
 * 
 * @author nick
 */
public class NeighborhoodFileCreator {
	
	static MessageCenter msg = MessageCenter
			.getMessageCenter(NeighborhoodFileCreator.class);

	private static class NoOpFilter implements Predicate<Household.HouseholdData> {
		@Override
		public boolean evaluate(HouseholdData arg0) {
			return true;
		}
	}

	private static class NoOpInit implements HouseholdDataInitializer {
		@Override
		public void initialize(HouseholdData data) {
		}
	}

	private List<Household> households;

	static {
		ParametersCreator creator = new ParametersCreator();
		creator.addParameter("epsilon", double.class, 1.0, false);
		RunEnvironment.init(null, null, creator.createParameters(), false);
	}

	private void loadHouseholds(String householdFile, double distance, Calendar calendar) throws IOException {
		// we don't need to intialize the household except from the
		// file so NoOps here
		HouseholdLoader creator = new HouseholdLoader(householdFile, new NoOpFilter(),
				new NoOpInit(), calendar, new Climate(), new ClimateImpacts(0,0));

		households = new ArrayList<>();
		InfluenceSetCreator infCreator = new InfluenceSetCreator();
		while (creator.hasNext()) {
			Household h = creator.nextHousehold();
			households.add(h);
			infCreator.addHousehold(h);
		}

		infCreator.run(distance);
	}

	public void run2(String householdFile, String out, double distance) throws IOException,
			TransformException, NoSuchFieldException, SecurityException, IllegalArgumentException,
			IllegalAccessException {
		msg.debug("Loading households from: " + householdFile);
		msg.debug("Writing " + distance + " meter neighbors to: " + out);

		loadHouseholds(householdFile, distance, null);
		int count = 0;
		for (Household h : households) {
			if (h.getInfluenceSet().size() > 0) count++;
		}
		
		try (DataOutputStream stream = new DataOutputStream(new FileOutputStream(out))) {
			stream.writeInt(count);
			for (Household h : households) {
				InfluenceSet nghs = h.getInfluenceSet();
				if (nghs.size() > 0) {
					stream.writeUTF(h.getId());
					stream.writeInt(nghs.size());
					for (Household other : nghs.households()) {
						stream.writeUTF(other.getId());
					}
				}
			}
		}
	}

	public void run(String householdFile, String out, double distance) throws IOException,
			TransformException, NoSuchFieldException, SecurityException, IllegalArgumentException,
			IllegalAccessException {
		msg.debug("Loading households from: " + householdFile);
		msg.debug("Writing " + distance + " meter neighbors to: " + out);

		loadHouseholds(householdFile, distance, null);
		try (BufferedWriter writer = new BufferedWriter(new FileWriter(out))) {
			for (Household h : households) {

				InfluenceSet nghs = h.getInfluenceSet();
				if (nghs.size() > 0) {

					writer.write(h.getId() + " : ");
					boolean comma = false;
					for (Household other : nghs.households()) {

						if (comma)
							writer.write(",");
						comma = true;
						writer.write(other.getId());

					}
					writer.write("\n");
				}
			}
		}
	}

	public static void main(String[] args) {
		try {
			NeighborhoodFileCreator creator = new NeighborhoodFileCreator();
			creator.run2(args[0], args[1], Double.valueOf(args[2]));
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

}
