package cnh.crb.hydro;

import java.util.Date;

/**
 * Abstract implementation of a Storage unit.
 * 
 * Contains all methods needed to manage water by
 * volume, but does not translate this to a depth.
 * Extensions of this should provide a means to
 * transform volume into a depth value.
 * 
 * @author murphy
 *
 */
public abstract class AbstractStorage implements Storage{

  protected double            currentVolumeStored;
  protected FlowHydrograph    netFlowHydrograph;
  protected Date              currentDate;
  protected String            id;
  
  public AbstractStorage(Date startDate, String id){
    currentVolumeStored = 0;
    currentDate         = startDate;
    netFlowHydrograph = new FlowHydrograph();
    this.id = id;
  }
  
  public AbstractStorage(double initialVolume, Date startDate, String id){
    currentVolumeStored = initialVolume;
    currentDate         = startDate;
    this.id = id;
  }

  public String getId(){ return id; }
  
  public void addInflowHydrograph(FlowHydrograph inflow){      netFlowHydrograph.add(inflow);       }
  public void removeInflowHydrograph(FlowHydrograph inflow){   netFlowHydrograph.subtract(inflow);  }
  public void addOutflowHydrograph(FlowHydrograph outflow){    netFlowHydrograph.subtract(outflow); }
  public void removeOutflowHydrograph(FlowHydrograph outflow){ netFlowHydrograph.add(outflow);      }

  
  public VolumeHydrograph getVolumeHydrograph(Date toDate){
    FlowHydrograph bounded = new FlowHydrograph(netFlowHydrograph);
    bounded.resetBounds(currentDate, toDate);
    return new VolumeHydrograph(currentVolumeStored, bounded);
  }
  
  public boolean flowToDate(Date flowToDate){
    FlowHydrograph netFlow = netFlowHydrograph.resetBounds(currentDate, flowToDate);
    VolumeHydrograph VH = new VolumeHydrograph(currentVolumeStored, netFlow);
    if(VH.isEmpty()) return false;
    currentVolumeStored = VH.getFinalVolume();
    currentDate = flowToDate;    
    return !VH.goesBelowThreshold(0);
  }
  
  public double getCurrentVolume(){
    return currentVolumeStored;
  }
  
  public void setToMax(){
    currentVolumeStored = getMaxVolume();
  }
  
  public void setVolumeStored(double newVolume){
    currentVolumeStored = newVolume;
  }

  
}
