package cnh.crb.hydro;

import java.util.Date;

import cnh.crb.agent.BasicWaterManager;

public class CanalSegmentManager extends BasicWaterManager {

	TrapezoidalCanalSegment seg;
	
	public CanalSegmentManager(TrapezoidalCanalSegment seg, String managerID) {
		super(seg, managerID);
		this.seg = seg;
	}

	
	// CanalSegmentManagers have to worry about loss, so when translating their
	// downstream requests into upstream demands they have to correct
	// for the loss
	protected NotificationStatus calculateAmountsToDemandFromSources(Date start, Date end, FlowManager flowManager){
	  // Assume only one source (for now)
	  SourceFlowNode src = waterSources.get(0);
	  // The amount of the demand will be the sum of all requested amounts
	  // TIMES a correction factor to adjust for loss in transit
	  FlowHydrograph H = getDemand(start, end).getProduct(1/(1 - seg.getLossRate()));
	  expectedInflow.put(src, new DemandRequest(H, null)); // Null for second term = assume it will be met
	  return new NotificationStatus(false, false);
    }
	
	
	// When flowing, must also outflow some to represent loss:
    protected void modifyStorageFlow(){
    	FlowHydrograph H = seg.netFlowHydrograph.getProduct(seg.getLossRate());
    	seg.addOutflowHydrograph(H); 
    }
		
}
