package cnh.crb.hydro;



/**
 * A DemandRequest is a water request used by a RecipientFlowNode
 * for the amount it is demanding and expects to receive from its
 * source.
 * 
 * For a DemandRequest:
 * 
 *    Hydrograph1 = The amount the recipient expects to receive
 *    Hydrograph2 = Additional available amount
 * 
 * @author murphy
 *
 */
public class DemandRequest extends WaterRequest {

  public DemandRequest(FlowHydrograph hydro1, FlowHydrograph hydro2) {
    super(hydro1, hydro2);
  }
  
  /**
   * This is called by the RecipientFlowNode to set a new
   * value that it will be demanding from the associated
   * source. If this is a change from the existing value,
   * return true; return false if there is no change.
   * @param newDemand
   * @return
   */
  public boolean setDemand(FlowHydrograph newDemand){
    boolean ret = !newDemand.equals(hydrograph1);
    hydrograph1 = newDemand;
    return ret;
  }
  
  /**
   * This is called by the RecipientFlowNode to indicate
   * that this is the supply that it will receive from its
   * associated source. If this is a change from the
   * existing value, return true; return false if there is
   * no change.
   * @param newSupply
   * @param additionalAvailable
   * @return
   */
  public boolean registerSupply(FlowHydrograph newSupply, FlowHydrograph additionalAvailable){
    
    if(newSupply.equals(hydrograph1) && additionalAvailable.equals(hydrograph2)){
      return false;
    }
     
    hydrograph1 = newSupply;
    hydrograph2 = additionalAvailable;
    return true;
  }

}
