package cnh.crb.hydro;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.log4j.Logger;

import repast.simphony.engine.environment.RunEnvironment;

public class FlowManager {
	
  private static Logger logger = Logger.getLogger(FlowManager.class);

  public List<FlowNode> nodesToFlow;
  public List<FlowNode> nonReadyNodes;
  public List<FlowNode> readyNodes;
   
  public FlowManager(){
    nodesToFlow   = new ArrayList<FlowNode>();
    nonReadyNodes = new ArrayList<FlowNode>();
    readyNodes    = new ArrayList<FlowNode>();
  }
  
  public void addFlowNode(FlowNode node){
	  nodesToFlow.add(node);
  }
  
  public void flow(Date start, Date end){
    try{
      logger.info("RUNNING FLOW MANAGER FLOW...");

      // Initialize Lists
      nonReadyNodes.addAll(nodesToFlow);
      
      while(nonReadyNodes.size() > 0){
        int i = 0;
        while(i < nonReadyNodes.size()){ // Note: list may grow, but won't 
                                         // shrink except for current node being removed
          FlowNode node = nonReadyNodes.get(i);
          if(node.prepareFlow(start, end, this)){
            readyNodes.add(node);
            nonReadyNodes.remove(node);
          }
          else{
            i++;
          }
        }
      }
      
      // Do all flow
      for(FlowNode node: readyNodes) 
    	  node.flow(end);
      
      // Clear
      readyNodes.clear();
      nonReadyNodes.clear();
      
      logger.info("Tick: " + RunEnvironment.getInstance().getCurrentSchedule().getTickCount() + ", RUNNING FLOW MANAGER FLOW... DONE");
    }
    catch(Exception E){ E.printStackTrace();}
  }
  
  public void markNonReady(FlowNode node){
    if(!nonReadyNodes.contains(node)){
      nonReadyNodes.add(node);
      readyNodes.remove(node);
    }
  }
  
  // Create a Repast-displayable network of nodes;
  // Node weight will equal flow rate (downstream = positive)
  // Parameter indicates the second at which the snapshot
  // is taken...?
  public void setNetworkLinks(long second){
    // Clear all existing links
    
    // Loop through nodes
    //   For each node, get outflows
    //      For each outflow:
    //         Create a new link for each outflow; set weight to flow rate
    
  }
  
  public void removeFlowNode(FlowNode node){
	  logger.debug("REMOVING FLOW NODE: " + node.getId());
	  nodesToFlow.remove(node);
	  nonReadyNodes.remove(node);
	  readyNodes.remove(node);
  }
  
}
