package cnh.crb.hydro;

import java.util.Date;

/**
 * Components of a network of flow
 * nodes implement this interface, which allows
 * a FlowManager to call manage the calculation
 * of how flow will move through the system,
 * determine when all calculations are complete,
 * invoke the 'flow' method, and cleanup after
 * flow is complete.
 * 
 * @author murphy
 *
 */
public interface FlowNode {

  public boolean isReadyForFlow();
  
  /**
   * Prepare to flow water from start date to end date.
   * Will return 'true' if successful and all information
   * needed to perform flow is available, false otherwise.
   * 
   * Typically this is called repeatedly as all Flow Nodes
   * coordinate how much water is being sent from node to
   * node.
   * 
   * @param start
   * @param end
   * @return
   */
  public boolean prepareFlow(Date start, Date end, FlowManager flowManager);
  
  /**
   * Called after flow is complete to re-set to a state
   * from which new flow calculations can be begun. (For
   * example, set 'ready to flow' flag to false.)
   */
  public void clearFlow();
  
  /**
   * Perform flow; adjust internal values based on amount
   * of in flow received and out flow sent.
   * @param end
   */
  public void flow(Date end);
    
  public String getId();
  
}
