package cnh.crb.hydro;

import java.text.DecimalFormat;

import simphony.util.messages.MessageCenter;

public abstract class FlowRate {
	
	static MessageCenter msg = MessageCenter.getMessageCenter(FlowRate.class);

  public enum FlowRateUnits{
    M3S, GPD, GPM, AFY
  }
    // Instance variables
  
  FlowRateUnits nativeUnits;
  protected double m3s;
  protected double gpd;
  protected double gpm;
  protected double afy;
  
  // Factory
  public static FlowRate create(double rate, FlowRateUnits units){
	  switch(units){
	    case M3S: return new FlowRateM3S(rate);
	    case GPD: return new FlowRateGPD(rate);
	    case GPM: return new FlowRateGPM(rate);
	    case AFY: return new FlowRateAFY(rate);
	    default: return null;
	  }
  }
  
  
  // Constructors
  
  public FlowRate(double rateM3S, double rateGPD, double rateGPM, double rateAFY, FlowRateUnits units){
	  m3s = rateM3S;
	  gpd = rateGPD;
	  gpm = rateGPM;
	  afy = rateAFY;
	  nativeUnits = units;
  }
  
  public abstract FlowRate copy();
  
  // Basic Getters
  public FlowRateUnits getNativeUnits(){ return nativeUnits; }
  
  // Operators
  public void add(FlowRate flowRate){
    m3s += flowRate.m3s;
    gpd += flowRate.gpd;
    gpm += flowRate.gpm;
    afy += flowRate.afy;
  }
  
  public void subtract(FlowRate flowRate){
    m3s -= flowRate.m3s;
    gpd -= flowRate.gpd;
    gpm -= flowRate.gpm;
    afy -= flowRate.afy;
  }
  
  public void multiply(double scalar){
    m3s *= scalar;
    gpd *= scalar;
    gpm *= scalar;
    afy *= scalar;
  }
  
  // Calculations (do not modify original)
  public abstract FlowRate getSum(FlowRate flowRate);
  public abstract FlowRate getDiff(FlowRate flowRate);
  public abstract FlowRate getProduct(double scalar);

  public abstract double getRate();
  
  private double getRate(FlowRateUnits units){
    switch(units){
      case GPD: return gpd;
      case M3S: return m3s;
      case AFY: return afy;
      case GPM: return gpm;
      default:  return Double.NaN;
    }
  }

  public double getRateAFY(){ return afy; }
  public double getRateM3S(){ return m3s; }
  public double getRateGPD(){ return gpd; }
  public double getRateGPM(){ return gpm; }
  
  public abstract int compare(FlowRate other);
  
  public String getString(FlowRateUnits units){
    DecimalFormat decimalFormat = new DecimalFormat("#0.000");
    String ret = decimalFormat.format(getRate(units)) + " ";
    switch(units){
      case GPD: ret = ret + "GPD"; break;
      case GPM: ret = ret + "GPM"; break;
      case AFY: ret = ret + "AFY"; break;
      case M3S: 
      default:  ret = ret + "M3S";
    }
    return ret;
  }

}