package cnh.crb.hydro;

import java.util.Date;
import java.util.List;

public class IrregularStorage extends AbstractStorage {

  protected List<Storage> storageComponents;
  protected double maxVolume;
  protected double maxDepth;
  
  
  public IrregularStorage(Date startDate, List<Storage> components, String id){
    super(startDate, id);
    storageComponents.addAll(components);
    for(Storage storage: storageComponents){
      maxVolume += storage.getMaxVolume();
      maxDepth  += storage.getMaxDepth();
    }
  }

  public IrregularStorage(Date startDate, List<Storage> components, double initialVolumeStored, String id){
    super(initialVolumeStored, startDate, id);
    storageComponents.addAll(components);
    for(Storage storage: storageComponents){
      maxVolume += storage.getMaxVolume();
      maxDepth  += storage.getMaxDepth();
    }
  }

  
  public double getMaxVolume() {    return maxVolume;  }
  public double getMaxDepth() {     return maxDepth;   }

  public double getDepthAtVolume(double volume) {
    double v = volume;
    double d = 0;
    int i = 0;
    Storage storage = storageComponents.get(i);
    while( (i < (storageComponents.size() - 1)) && (v > storage.getMaxVolume())){
      d += storage.getMaxDepth();
      v -= storage.getMaxVolume();
      i++;
      storage = storageComponents.get(i);
    }
    return d + storage.getDepthAtVolume(v);
  }

  public double getVolumeAtDepth(double depth) {
    double d = depth;
    double v = 0;
    int i = 0;
    Storage storage = storageComponents.get(i);
    while( (i < (storageComponents.size() - 1)) && (d > storage.getMaxDepth())){
      d -= storage.getMaxDepth();
      v += storage.getMaxVolume();
      i++;
      storage = storageComponents.get(i);
    }
    return v + storage.getVolumeAtDepth(d);
  }

}
