package cnh.crb.hydro;

import java.util.Date;

/**
 * A Reservoir is essentially a Storage
 * that has a known elevation for the stage
 * when volume contained is zero (i.e. the
 * 'bottom' of the reservoir). This allows
 * tracking stage through time.
 * 
 * @author murphy
 *
 */
public class Reservoir implements Storage{

  protected Storage storage;
  protected double  stageAtZero;
  
  public Reservoir(double altitudeOfBottom, Storage storageProfile){
    stageAtZero = altitudeOfBottom;
    storage     = storageProfile;
  }

  // Convert depth to stage
  public double getMaxStage(){                   return storage.getMaxDepth() + stageAtZero;            }
  public double getStageAtVolume(double volume){ return storage.getDepthAtVolume(volume) + stageAtZero; }
  public double getVolumeAtStage(double stage){  return storage.getVolumeAtDepth(stage - stageAtZero);  }
  public double getCurrentDepth(){               return getDepthAtVolume(storage.getCurrentVolume());   }
  public double getCurrentStage(){               return getStageAtVolume(storage.getCurrentVolume());   }
  
  // Basic Storage methods
  public void setToMax(){                                      storage.setToMax();                         }
  public void addInflowHydrograph(FlowHydrograph inflow){      storage.addInflowHydrograph(inflow);        }
  public void removeInflowHydrograph(FlowHydrograph inflow){   storage.removeInflowHydrograph(inflow);     }
  public void addOutflowHydrograph(FlowHydrograph outflow){    storage.addOutflowHydrograph(outflow);      }
  public void removeOutflowHydrograph(FlowHydrograph outflow){ storage.removeOutflowHydrograph(outflow);   }
  public void setVolumeStored(double newVolume){               storage.setVolumeStored(newVolume);         }

  public double  getMaxVolume(){                               return storage.getMaxVolume();              }
  public double  getMaxDepth(){                                return storage.getMaxDepth();               }
  public double  getDepthAtVolume(double volume){              return storage.getDepthAtVolume(volume);    }
  public double  getVolumeAtDepth(double depth){               return storage.getVolumeAtDepth(depth);     }
  public boolean flowToDate(Date flowToDate){                  return storage.flowToDate(flowToDate);      }
  public double  getCurrentVolume(){                           return storage.getCurrentVolume();          }
  
  public VolumeHydrograph getVolumeHydrograph(Date toDate){    return storage.getVolumeHydrograph(toDate); }
}
