package cnh.crb.hydro;

import java.util.Date;

public class TrapezoidalCanalSegment extends AbstractStorage {

  protected double bottomWidth;
  protected double topWidth;
  protected double height;
  protected double length;
  protected double xsArea;
  protected double maxVolume;
  
  protected double lossRate = 0; // Fraction of water lost in transit (evaporation, seepage)
	
  public TrapezoidalCanalSegment(Date startDate, double top, double bottom, double channelHeight, double segmentLength, String id){
    super(startDate, id);
    bottomWidth = bottom;
    topWidth    = top;
    height      = channelHeight;
    length      = segmentLength;
    xsArea      = ((top + bottom) / 2) * height;
    maxVolume   = xsArea * length;
  }
		  
  public TrapezoidalCanalSegment(double initialVolume, Date startDate, double top, double bottom, double channelHeight, double segmentLength, String id){
    super(initialVolume, startDate, id);
    bottomWidth = bottom;
    topWidth    = top;
    height      = channelHeight;
    length      = segmentLength;
    xsArea      = ((top + bottom) / 2) * height;
    maxVolume   = xsArea * length;
  }
  
  public double getLossRate(){     return lossRate; }
  
  public void   setLossRate(double rate){ lossRate = rate; }
  
  public double getMaxVolume(){
	return maxVolume;
  }

  public double getMaxDepth(){
	return height;
  }

  public double getDepthAtVolume(double volume){
	double area = volume/length;
	
	// To solve for depth given area is a quadratic where:
	double a = (topWidth - bottomWidth) / (2 * height);
	double b = bottomWidth;
	double c = -1 * area;
	
	// We know it will be the positive result
	double d = (-1 * b + Math.sqrt(b * b - 4 * a * c) ) / (2 * a);
	return d;
  }

  public double getVolumeAtDepth(double depth){
	double topWidthAtDepth = bottomWidth + (topWidth - bottomWidth) * (depth / height);
	double area = ((topWidthAtDepth + bottomWidth) / 2) * depth;
	return area * length;
  }
  
}
