package cnh.crb.hydro;

public class UnitsAndConverters {


	public enum VolumeUnits{
		GALLONS, CCF, M3, AF
	}

// Volume measurements
	public static double GallonsToCCF(double gallons){	return gallons / 748.052d; 	  }
	public static double GallonsToM3(double gallons){   return gallons * 0.00378451;  }
	public static double GallonsToAF(double gallons){   return gallons / 325851.429d; }
	
	public static double CCFToGallons(double CCF){      return CCF * 748.052d;        }
	public static double CCFToM3(double CCF){           return CCF / 2.68;            }
	public static double CCFToAF(double CCF){           return CCF * 0.00229568411d;  }
 	
	public static double M3ToCCF(double M3){            return M3 * 2.68;             }
	public static double M3ToGallons(double M3){        return M3 / 0.00378451d;      }
	public static double M3ToAF(double M3){             return M3 * 0.000810713194d;  }

	public static double AFToGallons(double AF){        return AF * 325851.429d;      }
	public static double AFToCCF(double AF){            return AF / 0.00229568411d;   }
	public static double AFToM3(double AF){             return AF * 1233.48184d;      } 

	
	public static double convertVolume(double value, VolumeUnits originalUnits, VolumeUnits newUnits){
		if(originalUnits == newUnits) return value;
		switch(originalUnits){
			case GALLONS:
				switch(newUnits){
					case CCF:       return GallonsToCCF(value);
					case M3:        return GallonsToM3(value);
					case AF:        return GallonsToAF(value);
				}
			case CCF:
				switch(newUnits){
					case GALLONS:   return CCFToGallons(value);
					case M3:        return CCFToM3(value);
					case AF:        return CCFToAF(value);
				}
			case M3:
				switch(newUnits){   
					case GALLONS:   return M3ToGallons(value); 
					case CCF:       return M3ToCCF(value);
					case AF:        return M3ToAF(value);
				}
			case AF:
		    	switch(newUnits){
		    	    case GALLONS:   return AFToGallons(value);
		    	    case CCF:       return AFToCCF(value);
		    	    case M3:        return AFToM3(value);
		    	}
	        }
		return value; // Should not occur
	}
	
	// Flow Rates
	public static double M3S_To_GPD(double cubicMetersPerSecond){    return 22824465.3d     * cubicMetersPerSecond;    }
	public static double M3S_To_GPM(double cubicMetersPerSecond){    return 15850.3231d     * cubicMetersPerSecond;    }
	public static double M3S_To_AFY(double cubicMetersPerSecond){    return 25583.6162d     * cubicMetersPerSecond;    }
	public static double GPD_To_M3S(double gallonsPerDay){           return 4.38126364E-08d * gallonsPerDay;           }
	public static double GPD_To_GPM(double gallonsPerDay){           return 0.000694444444d * gallonsPerDay;           }
	public static double GPD_To_AFY(double gallonsPerDay){           return 0.00112088568d  * gallonsPerDay;           }
	public static double GPM_To_M3S(double gallonsPerMinute){        return 6.30901964E-05d * gallonsPerMinute;        }
	public static double GPM_To_GPD(double gallonsPerMinute){        return 1440d           * gallonsPerMinute;        }
	public static double GPM_To_AFY(double gallonsPerMinute){        return 1.61407537d     * gallonsPerMinute;        }
	public static double AFY_To_M3S(double acreFeetPerYear){         return 3.90875156E-05d * acreFeetPerYear;         }
	public static double AFY_To_GPD(double acreFeetPerYear){         return 892.151645d     * acreFeetPerYear;         }
	public static double AFY_To_GPM(double acreFeetPerYear){         return 0.619549754d    * acreFeetPerYear;         }
	
}

