package cnh.crb.hydro;

import java.util.Date;
import java.util.Vector;

import cnh.crb.hydro.FlowRate.FlowRateUnits;

public class VolumeHydrograph {

  public class VolumeHydrographInterval{
    protected Date      startTime;
    protected Date      endTime;
    protected long      startSecond;
    protected long      endSecond;
    protected double    duration;

    double initialVolume;
    double slope;
    double intercept;
    double finalVolume;
  
    boolean goesNegative;
    
    public VolumeHydrographInterval(double initialVol, FlowInterval flowInterval){
      startTime     = flowInterval.startTime;
      endTime       = flowInterval.endTime;
      startSecond   = flowInterval.startSecond;
      endSecond     = flowInterval.endSecond;
      duration      = flowInterval.durationInMilliSeconds/1000;
      
      initialVolume = initialVol;
      double startRate = flowInterval.startFlowRate.getRateM3S();
      double endRate   = flowInterval.startFlowRate.getRateM3S();
      
      intercept = startRate;
      slope     = (endRate - startRate) / duration;
      
      finalVolume = getVolumeAtTime(duration); 
    }
    
    double getVolumeAtTime(double deltaT){
      return initialVolume + intercept * deltaT + (slope/2.0) * deltaT * deltaT;
    }
    
    public double getMinimum(){
      double endY = intercept + duration * slope;
      if((intercept >= 0 ) && (endY >= 0)) return initialVolume; // Only increasing
      if((intercept <= 0 ) && (endY <= 0)) return finalVolume;   // Only decreasing
      return getVolumeAtTime(duration * (Math.abs(intercept) / Math.abs(endY - intercept) ) );
    }
    
    public double getMaximum(){
      double endY = intercept + duration * slope;
      if((intercept >= 0 ) && (endY >= 0)) return finalVolume;     // Only increasing
      if((intercept <= 0 ) && (endY <= 0)) return initialVolume;   // Only decreasing
      return getVolumeAtTime(duration * (Math.abs(intercept) / Math.abs(endY - intercept) ) );
    }
    
    public boolean goesBelowThreshold(double threshold){
      return getMinimum() < threshold;
    }
    
    public boolean exceedsThreshold(double threshold){
      return getMaximum() > threshold;
    }
    
  }
  
  Vector<VolumeHydrographInterval> intervals = new Vector<VolumeHydrographInterval>();
  
  public VolumeHydrograph(double originalVolume, FlowHydrograph H){
    double initialVol = originalVolume;
    for(FlowInterval fi: H.flowIntervals){
      VolumeHydrographInterval sgi = new VolumeHydrographInterval(initialVol, fi);
      intervals.add(sgi);
      initialVol = sgi.finalVolume;
    }
  }
  
  public double getMaximum(){
    double max = Double.NEGATIVE_INFINITY;
    for(VolumeHydrographInterval sgi: intervals) max = Math.max(max, sgi.getMaximum());
    return max;
  }
  
  public double getMinimum(){
    double min = Double.POSITIVE_INFINITY;
    for(VolumeHydrographInterval sgi: intervals) min = Math.min(min, sgi.getMinimum());
    return min;
  }
  
  public boolean goesBelowThreshold(double threshold){
    for(VolumeHydrographInterval sgi: intervals) if(sgi.goesBelowThreshold(threshold)) return true;
    return false;
  }
  
  public double getFinalVolume(){
    return intervals.size() > 0 ? intervals.lastElement().finalVolume : Double.NEGATIVE_INFINITY;
  }
  
  public boolean isEmpty(){
    return (intervals.size() == 0);
  }
}
