package cnh.crb.messaging;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.apache.poi.ss.formula.functions.T;

import repast.simphony.context.Context;
import repast.simphony.random.RandomHelper;
import cnh.crb.household.Household;

public class HouseholdMessager implements Messager {

	Context context;
	List<Household> targets;

	double waterUseTop, waterUseBottom, intensity, effectiveness, subsampling;
	boolean sameHouseholds;

	/**
	 * waterUseTop and waterUseBottom define the range of households to message.  
	 * 
	 * @param waterUseTop 
	 * @param waterUseBottom
	 * @param intensity
	 * @param effectiveness
	 * @param subsampling
	 * @param sameHouseholds
	 */
	public HouseholdMessager(double waterUseTop, double waterUseBottom,
			double intensity, double effectiveness, double subsampling, boolean sameHouseholds) {
		this.waterUseTop = waterUseTop;
		this.waterUseBottom = waterUseBottom;
		this.intensity = intensity;
		this.effectiveness = effectiveness;
		this.subsampling = subsampling;
		this.sameHouseholds = sameHouseholds;
	}

	@Override
	public void initialize(Context context) {
		this.context = context;
	}

	@Override
	public void message() {
		// Update targets as necessary
		updateTargets();
		// For all targets, message with specific intensity and effectiveness)
		for (Household h : targets){
			if (RandomHelper.nextDouble() < effectiveness){
				h.updateSocialFactorsWithIntensity(intensity);
			}
		}
	}

	boolean firstTime = true;

	protected void updateTargets() {
		if (firstTime) {
			targets = createTargets();
			firstTime = false;
		} else {
			if (!sameHouseholds) {
				targets = createTargets();
			}
		}
	}

	protected List<Household> createTargets() {
		List<Household> temp = new ArrayList<>();
		for (Object obj : context.getObjects(Household.class)) {
			temp.add((Household) obj);
		}

		// Sort ascending
		Collections.sort(temp, new Comparator<Household>() {
			@Override
			public int compare(Household h1, Household h2) {
				return Double.compare(h1.getBillMemory().getLastBillCost(), h2
						.getBillMemory().getLastBillCost());
			}
		});
		return getSublist(temp);
	}

	protected <T> List<T> getSublist(List<T> list) {
		int len = list.size();
		// Calculate start index, lowest consumption
		int startIndex = (int) Math.floor(len * waterUseBottom);

		// Calculate end index, highest consumption
		int endIndex = (int) Math.ceil(len * waterUseTop);
		
		List<T> temp = new ArrayList<T>();
		for(T t: list.subList(startIndex, endIndex)) if(RandomHelper.nextDouble() < subsampling) temp.add(t);
		return temp;
	}

}
