package cnh.crb.regional;

import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;

import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.parameter.Parameters;
import repast.simphony.random.RandomHelper;
import cnh.crb.SeasonalWaterUse;
import cnh.crb.SeasonalWaterUseInstance;
import cnh.crb.agent.BasicWaterManager;
import cnh.crb.climate.TemperatureStation;
import cnh.crb.hydro.DemandRequest;
import cnh.crb.hydro.FlowHydrograph;
import cnh.crb.hydro.FlowManager;
import cnh.crb.hydro.FlowRate.FlowRateUnits;
import cnh.crb.hydro.SourceFlowNode;
import cnh.crb.hydro.Storage;
import cnh.crb.hydro.UnitsAndConverters;
import cnh.crb.hydro.UnitsAndConverters.VolumeUnits;
import cnh.crb.hydro.WaterSink.WATER_SINK_METHOD;

public class CAPSubcontractor extends BasicWaterManager{
	
	private static final double SHORTAGE_THRESH_RED_ADJ_RATE = 0; // Default = 2
	private static final double AMT_REQ_RED_ADJ_RATE = 2;         // Default = 2
	private static final double OVER_THRESH_RED_ADJ_RATE = 0;     // Default = 1
	private static final double AMT_RET_INC_ADJ_RATE = 1;         // Default = 1
	
	private static final double OVER_THRESH_INC_ADJ_RATE = 0;     // Default = 2
	private static final double AMT_RET_RED_ADJ_RATE = 2;         // Default = 2
	private static final double SHORTAGE_THRESH_INC_ADJ_RATE = 0; // Default = 1
	private static final double AMT_REQ_INC_ADJ_RATE = 1;         // Default = 1
	
	
  // FOR TESTING ONLY: OVERRIDES CAPSubcontractors.xml...
  private static final String[] subcontractors = { 
	  "ApacheJunction","Avondale","Carefree","CaveCreek","Chandler","Gilbert","Glendale","Goodyear","Mesa","Peoria","Phoenix","Scottsdale","Surprise","Tempe"};
	
  // Note: These are drawn from Phoenix's data
  private static final double[] defaultGPCDIntercepts = {
	  /* Jan Feb Mar */ 78.339326755617,   61.9306374171928,  42.7522402410892,
	  /* Apr May Jun */ 46.7734023786695,  61.5644449866587,  60.2518400647242,
	  /* Jul Aug Sep */ 26.9636839708681, -57.5876518003275, 121.937352072417,
	  /* Oct Nov Dec */ 62.539397933882,   60.8295013974454, 101.342996251129 
  };
  
  // Note: these data are from WBM averages, 2002-2013
  private static final double[][] defaultTemperatureSeries = {
	  /* ApacheJunction*/ {10.89, 11.99, 16.22, 20.26, 25.98, 31.68, 32.81, 31.64, 28.85, 22.47, 16.23, 10.38 },
	  /* Avondale*/       {13.39, 14.59, 18.86, 22.87, 28.49, 33.96, 35.97, 34.82, 31.8 , 25.06, 18.68, 12.76 },
	  /* Carefree*/       {10.59, 11.52, 15.65, 19.53, 25.26, 31.01, 32.72, 31.5 , 28.44, 21.83, 15.65,  9.94 },
	  /* CaveCreek*/      {10.59, 11.52, 15.65, 19.53, 25.26, 31.01, 32.72, 31.5 , 28.44, 21.83, 15.65,  9.94 },
	  /* Chandler*/       {12.49, 13.65, 17.94, 22   , 27.62, 33.19, 34.78, 33.62, 30.72, 24.11, 17.76, 11.9} ,
	  /* Gilbert*/        {12.2 , 13.33, 17.59, 21.64, 27.3,  32.92, 34.37, 33.2 , 30.33, 23.79, 17.49, 11.64 },
	  /* Glendale*/       {13.43, 14.56, 18.81, 22.8 , 28.46, 33.99, 35.93, 34.75, 31.72, 25.01, 18.68, 12.79 },
	  /* GoodYear*/       {13.18, 14.44, 18.73, 22.71, 28.29, 33.71, 35.78, 34.67, 31.68, 24.92, 18.48, 12.56 },
	  /* Mesa*/           {12.3 , 13.41, 17.66, 21.71, 27.39, 33.03, 34.46, 33.28, 30.4 , 23.89, 17.61, 11.74 },
	  /* Peoria*/         {12.71, 13.75, 17.94, 21.82, 27.54, 33.17, 35.06, 33.86, 30.79, 24.11, 17.85, 12.07 },
	  /* Phoenix*/        {13.27, 14.4 , 18.67, 22.71, 28.35, 33.9,  35.74, 34.56, 31.55, 24.87, 18.54, 12.63 },
	  /* Scottsdale*/     {10.04, 10.99, 15.11, 19.03, 24.73, 30.44, 32.11, 30.9 , 27.9 , 21.33, 15.15,  9.42 },
	  /* Surprise*/       {12.82, 13.94, 18.16, 22.07, 27.75, 33.28, 35.32, 34.14, 31.06, 24.34, 18.03, 12.19 },
	  /* Tempe*/          {12.7 , 13.86, 18.14, 22.19, 27.81, 33.38, 35.11, 33.94, 30.97, 24.32, 17.97, 12.09 }
  };
  
  
  /* These methods allow you to get a temperature for a city in a given month,
   * using a set of pre-packaged defaults
   */
  private static Map<String, Integer> tempSeqIndex = new HashMap<String, Integer>();
  
  private static int getTempSequenceIndex(String cityName){
	  if(tempSeqIndex.containsKey(cityName)) return tempSeqIndex.get(cityName);
	  for(int i = 0; i < subcontractors.length; i++){
		  if(subcontractors[i].compareTo(cityName) == 0){
			  tempSeqIndex.put(cityName, i);
			  return i;
		  }
	  }
	  return -1; // Note: No error trap; should never happen...
  }
  
  public static double getDefaultTemp(String cityName, int month){
	  return defaultTemperatureSeries[getTempSequenceIndex(cityName)][month];
  }
  	
  private static Logger LOGGER = Logger.getLogger(CAPSubcontractor.class);

  public static double globalNoise;         // Noise that is the same in a given year or month for all managers
  public static double globalNoiseRange; 
  
  double   returnPercent = 0.5;
  double   population;
  double   populationGrowthRate;
  double   gpd;
  double[] gpdIntercepts      = new double[12];
  double[] gpdSlopes          = new double[12];
  double   gpdTrend;
  double   noisiness;
  double   currentRate;
  double[] schedule           = new double[12]; // This year's schedule         Units are total gallons per day
  double[] nextSchedule       = new double[12]; // Next year's schedule                    "
  double[] demandRecord       = new double[12]; // Record of what was demanded             "
  double[] supplyRecord       = new double[12]; // Record of what was supplied             "
  double   totalScheduledAF   = 0;
  double   runningTotalAF     = 0;
  Calendar cal;  
  
  double overageThreshold  =  1.05;
  double amountToReturn    =  1.10;
  double shortageThreshold =  0.95;
  double amountToObtain    = -0.90;
  
  // Values for reporting
  double totalSimToDateShortage            = 0;
  double totalSimToDateShortagePerCapita   = 0;
  double totalSimToDateShortagePercentGPCD = 0;
  int    years                             = 1;
  
  double lastYearScheduledAF = 0;
  double lastYearDeliveredAF = 0;
  
  
  double[] AFDonated   = new double[12];
  double[] AFReceived  = new double[12];
  double[] AFOffered   = new double[12];
  double[] AFRequested = new double[12];
  
  double temperatureRecord[] = new double[12]; 
  double popRecord[]         = new double[12];
  double minShortageFraction = 1;
  double maxOverageFraction  = 1;
  
  private class AnnualWaterRecord{
	  public double[] demand      = new double[12];
	  public double[] delivered   = new double[12];
	  public double[] origSched   = new double[12];
	  public double[] scheduled   = new double[12];
	  public double[] AFoffered   = new double[12];
	  public double[] AFdonated   = new double[12];
	  public double[] AFrequested = new double[12];
	  public double[] AFreceived  = new double[12];
	  public double totalDemand      = 0;
	  public double totalDelivery    = 0;
	  public double totalScheduledAF = 0;
	  public double net; // Surplus or shortage
  }
  
  Map<Integer, AnnualWaterRecord > pastYearsDemandRecord = new HashMap<Integer, AnnualWaterRecord >(); 
		  
  // Pass-through constructor
  public CAPSubcontractor(Storage storage, String managerID, double population, double populationGrowthRate, 
		  double gpd, double gpdTrend, double noiseRange, Calendar cal){
	  this(storage, managerID, population, populationGrowthRate, gpd, gpdTrend, noiseRange, cal, "", "");
  }
  
  public CAPSubcontractor(Storage storage, String managerID, double population, double populationGrowthRate, 
		  double gpd, double gpdTrend, double noiseRange, Calendar cal, String intercepts, String slopes){
	
	/* First invoke the superclass constructor for Basic Water Manager */
    super(storage, managerID);
    
    
    /* Next set the instance variables from the arguments passed */
    this.population           = population;
    this.populationGrowthRate = populationGrowthRate;
    this.gpd                  = gpd;
    this.gpdTrend             = gpdTrend;
    this.cal                  = cal;
    this.noisiness            = noiseRange;
    
    /* Grab the parameters values */
    Parameters params = RunEnvironment.getInstance().getParameters();
    
    /* Testing: Re-Scale values for pop and gpd to some set of values away from the averages given */
    // Note that minimum value for GPD is 40, and for population is 1,000
    double scale     = params.getDouble("gpcdAndPopScale");
    if(scale != 1.0){
      double AVERAGE_GPCD = params.getDouble("avgGPCD"); //    132.7;
      double AVERAGE_POP  = params.getDouble("avgPop");  // 243461;
      this.population  = Math.max(1000,  AVERAGE_POP  + (population - AVERAGE_POP) * scale);
      this.gpd         = Math.max(40,    AVERAGE_GPCD + (gpd - AVERAGE_GPCD)       * scale);
      LOGGER.info("Setting scaled gpcd/pop for " + id + " to POP = " + this.population + " GPCD = " + this.gpd);    	
    }
    
    boolean useDefaultGPCDIntercepts = params.getBoolean("CAPSubsUseDefaultGPCDIntercepts");
    if(useDefaultGPCDIntercepts) LOGGER.info("Using default values for GPCD intercepts.");
      
	if(intercepts != null && intercepts.length() > 0){
	  String[] ints = intercepts.split(",");
	  String[] slps = slopes.split(",");
	  for(int c = 0; c < 12; c++){
		  setInterceptAndSlope(c, (useDefaultGPCDIntercepts ? defaultGPCDIntercepts[c]: Double.parseDouble(ints[c])), Double.parseDouble(slps[c]));
	  }
    }

	// Set the population record:
	for(int i = 0; i < 12; i++) popRecord[i] = this.population; // Note: this is last year's population record; could work backward to calculate based on growth rate...
	  
	// Need to initialize values for schedule and demand, supply records
	if(params.getBoolean("UseSeasonalWaterUseCurve")){
		Calendar cal2 = (Calendar)cal.clone();
	    cal2.set(Calendar.MONTH, 0);
	    SeasonalWaterUse seasonal = SeasonalWaterUseInstance.getWaterUse();
	    for(int i = 0; i < 12; i++){
	    	double dailyDemand = this.gpd * this.population * seasonal.getMonthlyScaleFactorFor(cal2);
			LOGGER.debug("CAPSubcontractor constructor SETTING demandRecord VALUES FOR " + getId() + " to daily demand = " + dailyDemand);
	    	demandRecord[i] = dailyDemand;
	    	cal2.add(Calendar.MONTH, 1);
	    }
	}
	else{
		String referenceCity = params.getBoolean("UseSingleTemperature") ? params.getString("UseSingleTemperatureCity") : id;
		for(int i = 0; i < 12; i++){
			temperatureRecord[i] = CAPSubcontractor.getDefaultTemp(referenceCity, i);
			double dailyDemand =  (gpdIntercepts[i] + gpdSlopes[i] * temperatureRecord[i]) * this.population; // No noise...
			LOGGER.debug("CAPSubcontractor constructor SETTING VALUES FOR " + getId() + " to daily demand = " + dailyDemand + " using reference city: " + referenceCity);
			demandRecord[i] = dailyDemand;
		}
		
	}
    
	// All values from last year match, and this drives schedule:
	for(int i = 0; i < 12; i++){
		supplyRecord[i]   = demandRecord[i];
		schedule[i]       = demandRecord[i]; // Note: Could drive by pop growth figures... 
    	nextSchedule[i]   = demandRecord[i]; //
	}
    
  }
  
  
  public double getOverageThreshold(){    return overageThreshold; }
  public double getAmountToReturn(){      return amountToReturn; }
  public double getShortageThreshold(){   return shortageThreshold; }
  public double getAmountToObtain(){      return amountToObtain; }
  public double getLastYearScheduledAF(){ return lastYearScheduledAF; }
  public double getLastYearDeliveredAF(){ return lastYearDeliveredAF; }
  public double getPopulation(){          return population; }
  public double getGPCD(){                return gpd; }  
  
  
  public void setInterceptAndSlope(int month, double newIntercept, double newSlope){
	  gpdIntercepts[month] = newIntercept;
	  gpdSlopes[month]     = newSlope;
  }
  
  // This is called each new year
  public void newYearRollover(){
	  // Keep a record of last year's demand and amount supplied
	  int lastYear = cal.get(Calendar.YEAR) - 1;
	  AnnualWaterRecord newRecord = new AnnualWaterRecord();
	  
	  for(int i = 0; i < 12; i++){ 
		  newRecord.totalDemand   += demandRecord[i];
		  newRecord.totalDelivery += supplyRecord[i];
		  newRecord.totalScheduledAF = this.totalScheduledAF;
		  
		  newRecord.demand[i]      = demandRecord[i];
		  newRecord.delivered[i]   = supplyRecord[i];
		  newRecord.origSched[i]   = schedule[i];
		  newRecord.scheduled[i]   = nextSchedule[i]; 
		  newRecord.AFoffered[i]   = AFOffered[i];
		  newRecord.AFdonated[i]   = AFDonated[i];
		  newRecord.AFrequested[i] = AFRequested[i];
		  newRecord.AFreceived[i]  = AFReceived[i];
		  
		  AFOffered[i]   = 0;
		  AFDonated[i]   = 0;
		  AFRequested[i] = 0;
		  AFReceived[i]  = 0;
	  }
	  pastYearsDemandRecord.put(lastYear, newRecord);
	  
	  if(LOGGER.isDebugEnabled()){
		  double totalDemand = 0;
		  double totalDelivered = 0;
		  double totalOrigScheduled = 0;
		  double totalDonated = 0;
		  double totalReceived = 0;
		  double totalOffered = 0;
		  double totalRequested = 0;
		  Calendar lastYearCal = (Calendar)cal.clone();
		  lastYearCal.add(Calendar.YEAR, -1); // Go back in time
		  for(int i = 0; i < 12; i++){
			  int daysInMonth = lastYearCal.getActualMaximum(Calendar.DATE);
			  totalDemand        += UnitsAndConverters.GallonsToAF(newRecord.demand[i]    * daysInMonth);
			  totalDelivered     += UnitsAndConverters.GallonsToAF(newRecord.delivered[i] * daysInMonth);;
			  totalOrigScheduled += UnitsAndConverters.GallonsToAF(newRecord.origSched[i] * daysInMonth);;
			  totalDonated       += newRecord.AFdonated[i];
			  totalReceived      += newRecord.AFreceived[i];
			  totalOffered       += newRecord.AFoffered[i];
			  totalRequested     += newRecord.AFrequested[i];
			  lastYearCal.add(Calendar.MONTH, 1);
		  }
		  double tick = RunEnvironment.getInstance().getCurrentSchedule().getTickCount();
		  LOGGER.debug("YEAR ROLLOVER: TICK= " + tick + " YEAR= " + lastYear + " City= " + this.getId() + " OrigSched= " + totalOrigScheduled + " Demand= " + totalDemand + " Delivered= " + totalDelivered  + " Scheduled= " + this.totalScheduledAF + " Donated= " + totalDonated + " Received= " + totalReceived + " ShortageThresh= " + this.shortageThreshold + " OverageThresh= " + this.overageThreshold + " AmtToRequest= " + this.amountToObtain + " AmtToReturn= " + this.amountToReturn + " totalOffered= " + totalOffered + " totalRequested= " + totalRequested);
	  }
	  
	  // Deal with cumulative
	  updateSimToDateTotals(); // SHOULD NOT RUN ON FIRST INSTANCE
	  
	  Parameters params = RunEnvironment.getInstance().getParameters();
	  if(params.getBoolean("OptimizeThresholds")) optimizeThresholds(newRecord);
		  
	  
	  // And rollover to new values
	  population *= (1 + populationGrowthRate);
	  gpd        *= (1 + gpdTrend);
	  for(int i = 0; i < 12; i++) schedule[i] = nextSchedule[i];
	  runningTotalAF = 0;
	  
  }
  
  public static void initGlobalNoise(){
	  if(globalNoise == 0) resetGlobalNoise();
  }
  
  public static void resetGlobalNoise(){
	  // And reset the global noise. Note that this happens for all managers, but only the last one matters.
	  globalNoise = ((RandomHelper.nextDoubleFromTo(-1.0, 1.0)) * globalNoiseRange) + 1.0;
  }
  
  // This is called by the CAWCD each Oct 1.
  public double[] getNextYearSchedule(){
	  // System.out.println(getId() + " RUNNING GET NEXT YEAR SCHEDULE " + RunEnvironment.getInstance().getCurrentSchedule().getTickCount());
	calculateNextYearSchedule();
	return nextSchedule;
  }
  
  public void calculateNextYearSchedule(){
	  for(int i = 0; i < 12; i++) nextSchedule[i] = demandRecord[i]; // Note: actually has last year's Oct/Nov/Dec values (!)
  }
  
  // Note: This is called as soon as the simulation starts; newYearRollover is only called after the first year ends
  public void resetYear(){
	  // System.out.println(getId() + " RUNNING RESET YEAR " + RunEnvironment.getInstance().getCurrentSchedule().getTickCount());
	  totalScheduledAF = 0;
	  Calendar calendar = (Calendar)cal.clone();
	  for(int i = 0; i < 12; i++){
		  calendar.set(Calendar.MONTH, i);
		  totalScheduledAF += UnitsAndConverters.GallonsToAF(schedule[i]) * calendar.getActualMaximum(Calendar.DATE);
	  }
//	  System.out.println(getId() + " totalScheduledAF = " + totalScheduledAF);
  }
  
  // This should be called each month
  public void resetMonth(){
	  double tick = RunEnvironment.getInstance().getCurrentSchedule().getTickCount();
	  int month = cal.get(Calendar.MONTH);
	  LOGGER.info("CALLING RESET MONTH FOR " + tick + " " + this.getId() + " MONTH = " + month);
	  if(month == 0) resetYear();

	  int daysInMonth = cal.getActualMaximum(Calendar.DATE);
	  // Calculate the total scheduled acre feet for the year
		
	  double noise        = ((RandomHelper.nextDoubleFromTo(-1.0, 1.5)) * noisiness) + 1.0;
	  LOGGER.debug("TICK: " + tick + " NEW NOISE VALUE FOR " + this.getId() + " IS " + noise + " FROM " + noisiness);
	  
	  double newDailyDemand  = gpd * population * SeasonalWaterUseInstance.getWaterUse().getMonthlyScaleFactorFor(cal) * globalNoise * noise;
	  popRecord[month] = population; // In case this changes...
	  if(this.sink != null){
		  LOGGER.debug(getId() + " SINK IS NOT NULL");
		  if(this.sink instanceof TemperatureStation){
			  LOGGER.debug(getId() + " SINK IS NOT NULL");  
			  double temp = ((TemperatureStation)sink).getTemperature();
			  if(RunEnvironment.getInstance().getParameters().getBoolean("UseRepeatedTemperature")){
				  LOGGER.debug("USING REPEATED TEMPERATURE");
				  if(temperatureRecord[month] == Double.NEGATIVE_INFINITY){
					  LOGGER.debug("MAKING RECORD OF TEMPERATURE FOR MONTH " + month + ": T = " + temp);
					  temperatureRecord[month] = temp;
				  }
				  else{
					  temp = temperatureRecord[month];
					  LOGGER.debug("USING RECORDED TEMPERATURE FOR MONTH " + month + ": T = " + temp);
				  }
			  }
			  if(temp > 0){ // Never really zero in PHX for an entire month...
				  double originalDemand = newDailyDemand;
				  newDailyDemand = (gpdIntercepts[month] + gpdSlopes[month] * temp) * population * globalNoise * noise;
//				  LOGGER.debug("SETTING DAILY DEMAND BASED ON TEMPERATURE: T = " + temp);
//				  LOGGER.debug("    PREVIOUS     = " + originalDemand + " gallons/day"); 
//				  LOGGER.debug("    TEMP         = " + temp);
//				  LOGGER.debug("    POPULATION   = " + population);
//				  LOGGER.debug("    GLOBAL NOISE = " + globalNoise);
//				  LOGGER.debug("    NOISE        = " + noise);
//				  LOGGER.debug("    INTERCEPT    = " + gpdIntercepts[month] + " for month " + month);
//				  LOGGER.debug("    SLOPE        = " + gpdSlopes[month] + " for month " + month);
//				  LOGGER.debug("    NEW DEMAND   = " + newDailyDemand + " gallons/day = " + UnitsAndConverters.GallonsToAF(newDailyDemand) * cal.getActualMaximum(Calendar.DATE) + " AF this month");
				  
				  LOGGER.debug("MONTH RESET " + tick + " " + cal.get(Calendar.YEAR)+ " " + month + " " + getId() + " " + originalDemand + " " + gpdIntercepts[month] + " " + gpdSlopes[month] + " " + temp + " " + globalNoise + " " + noise + " " + population + " " + newDailyDemand);
			  }
			  else{
				  // This is likely to happen first time through...
				  LOGGER.warn("TEMP IS ZERO- KEEPING ORIGINAL VALUE FOR dailyDemand = " + newDailyDemand + " gallons/day");
			  }
		  }
	  }
	  
	  

	  // Obsolete:
      // double dailyDemand  = record[month] *noise;
	  demandRecord[month] = newDailyDemand;
	  currentRate         = getNewMonthRate(newDailyDemand);
	  supplyRecord[month] = currentRate;
	  runningTotalAF += UnitsAndConverters.GallonsToAF(currentRate) * daysInMonth;
//	  LOGGER.debug("RUNNING RESETMONTH DATA:" + getId()); 
//      LOGGER.debug("   TICK: " + RunEnvironment.getInstance().getCurrentSchedule().getTickCount()+ " YEAR: " + cal.get(Calendar.YEAR) + " MONTH: " + month);
//      LOGGER.debug("   DEMAND RECORD           = " + demandRecord[month] + " (GALLONS/DAY)");
//      LOGGER.debug("   SUPPLY RECORD           = " + supplyRecord[month] + " (GALLONS/DAY)"); 
//      LOGGER.debug("   SCHEDULE                = " + schedule[month] + " (GALLONS/DAY)");
//      LOGGER.debug("   RUNNING TOTAL ACRE FEET = " + runningTotalAF + " (AF/MONTH)");
      if(LOGGER.isDebugEnabled()){
    	  
    	  Calendar calTemp = (Calendar)cal.clone();
	      String logOut = "DEMAND_RECORD "  + tick + " " + cal.get(Calendar.YEAR)+ " " + month + " " + getId();
	      calTemp.set(Calendar.MONTH, 0);
	      for(int i =0; i < 12; i++){
	    	  logOut += " " + UnitsAndConverters.GallonsToAF(demandRecord[i]) * calTemp.getActualMaximum(Calendar.DATE);
	    	  calTemp.add(Calendar.MONTH, 1);
	      }
	      LOGGER.debug(logOut);
	      
	      logOut = "SUPPLY_RECORD "  + tick + " " + cal.get(Calendar.YEAR)+ " " + month + " " + getId();
	      calTemp = (Calendar)cal.clone();
	      calTemp.set(Calendar.MONTH, 0);
	      for(int i =0; i < 12; i++){
	    	  logOut += " " + UnitsAndConverters.GallonsToAF(supplyRecord[i]) * calTemp.getActualMaximum(Calendar.DATE);
	    	  calTemp.add(Calendar.MONTH, 1);
	      }
	      LOGGER.debug(logOut);
	      
	      logOut = "SCHEDULE_RECORD "  + tick + " " + cal.get(Calendar.YEAR)+ " " + month + " " + getId();
	      calTemp = (Calendar)cal.clone();
	      calTemp.set(Calendar.MONTH, 0);
	      for(int i =0; i < 12; i++){
	    	  logOut += " " + UnitsAndConverters.GallonsToAF(schedule[i]) * calTemp.getActualMaximum(Calendar.DATE);
	    	  calTemp.add(Calendar.MONTH, 1);
	      }
	      LOGGER.debug(logOut);
	      
	      logOut = "NEXTSCHED_RECORD "  + tick + " " + cal.get(Calendar.YEAR)+ " " + month + " " + getId();
	      calTemp = (Calendar)cal.clone();
	      calTemp.set(Calendar.MONTH, 0);
	      for(int i =0; i < 12; i++){
	    	  logOut += " " + UnitsAndConverters.GallonsToAF(nextSchedule[i]) * calTemp.getActualMaximum(Calendar.DATE);
	    	  calTemp.add(Calendar.MONTH, 1);
	      }
	      LOGGER.debug(logOut);
      }
      LOGGER.info("REMAINING_RECORD " + tick + " " + month + " " + this.getId() + " " + getRemainingScheduled());
      
  }
  
  /**
   * This method can capture any strategic decisions that the agent would like
   * to make. The only restriction is that the amount withdrawn cannot exceed
   * the maximum allowed by the original scheduled amount.
   * @return
   */
  private double getNewMonthRate(double fullDemand){
	  int daysInMonth = cal.getActualMaximum(Calendar.DATE);
      LOGGER.debug("GETTING NEW MONTH RATE: FULL DEMAND = " + fullDemand + " Scheduled " + totalScheduledAF + " running " + runningTotalAF);
	  double rate = fullDemand;
	  double acreFeetRemaining = totalScheduledAF - runningTotalAF;  // Total remaining for the year
	  if(acreFeetRemaining <= 0) rate = 0;
	  else{
	      double acreFeetProposed = UnitsAndConverters.GallonsToAF(rate) * daysInMonth;  // Proposed for this month
	      if(acreFeetProposed > acreFeetRemaining){
	    	  rate = UnitsAndConverters.AFToGallons(acreFeetRemaining) / ((double)daysInMonth);
	      }
	  }
	  return rate;
  }
  
  // Taken from DataDrivenDemandWaterManager  
  protected NotificationStatus calculateAmountsToDemandFromSources(Date start, Date end, FlowManager flowManager){
	  // Assume only one source (for now)
	  SourceFlowNode src = waterSources.get(0);
	  // The amount of the demand will be the sum of all requested amounts
	  FlowHydrograph H = getDemand(start, end);
	  expectedInflow.put(src, new DemandRequest(H, null)); // Assume it will be met
	  return new NotificationStatus(false, false);
  }
  
  public FlowHydrograph getDemand(Date start, Date end){
	  super.getDemand(start, end); // Calculate as per usual, but this will be EMPTY:
	  FlowHydrograph H = new FlowHydrograph(start, end, currentRate, FlowRateUnits.GPD);
	  //System.out.println(this.id + "| " + start + " ACRE-FEET " + UnitsAndConverters.GallonsToAF(currentRate) + " (" + currentRate + ")");
	  return H;
  }
  
  public void sink(){
	  if(sink != null){
		  double storageVol = storage.getCurrentVolume();
		  double previousStorageVol = storageVol - deltaStorage; // Note: will always be positive storage growth
		  
		  // Some of the inflow (which is currently in storage) is automatically redirected back to the sink
		  double toReturnToSurface      = deltaStorage * .20; // 20% goes back via surface
		  double toReturnToBase         = deltaStorage * .05; //  5% goes back via groundwater
		  double toReturnViaEvaporation = deltaStorage * .10; // 10% goes back via evaporation
		  
		  double storageReduction = toReturnToSurface + toReturnToBase + toReturnViaEvaporation;
		  
		  // Adjust actualStorage values:
		  storageVol -= storageReduction;
		  storage.setVolumeStored(storageVol);
		  
		  // After this, choose a target storage volume:
		  double storagePercent = storageVol / storage.getMaxVolume();
		  double targetStoragePct = 0.5;
//		  System.out.println("CAP SUBCONTRACTOR " + getId() + " storagePct = " + storagePercent + " target = " + targetStoragePct + " diff = " + (storagePercent - targetStoragePct));
		  if(storagePercent > targetStoragePct){
			  double pctReduction = storagePercent - targetStoragePct;
			  double reductionAmount = pctReduction * storage.getMaxVolume();
			  toReturnToSurface += reductionAmount * 0.5; // Half goes to the surface
			  toReturnToBase    += reductionAmount * 0.5; // Half goes to the base
			  
			  storageVol -= reductionAmount;
			  storage.setVolumeStored(storageVol);			  
		  }
		  deltaStorage = previousStorageVol - storageVol;
		  sink.eliminateWater(toReturnToSurface,      VolumeUnits.M3, WATER_SINK_METHOD.RETURN_TO_SURFACE);
		  sink.eliminateWater(toReturnToBase,         VolumeUnits.M3, WATER_SINK_METHOD.RETURN_TO_GROUNDWATER);
		  sink.eliminateWater(toReturnViaEvaporation, VolumeUnits.M3, WATER_SINK_METHOD.EVAPORATE);
		  sink.eliminateWater(deltaStorage,           VolumeUnits.M3, WATER_SINK_METHOD.DELTA_STORE);
//		  System.out.println("CAP SUBCONTRACTOR " + getId() + " SINKING: To surface: " + toReturnToSurface + " to base: " + toReturnToBase + " evap: " + toReturnViaEvaporation + " deltaStore: " + deltaStorage);
	  }
  }
  

  public double withinYearPreferredAdjustment(){
	  double tick = RunEnvironment.getInstance().getCurrentSchedule().getTickCount();
	  // Compare amount scheduled to be delivered with total acre-feet remaining and propose an adjustment
	  int firstMonthForExchange = RunEnvironment.getInstance().getParameters().getInteger("FirstCAPExchangeMonth");
	  int lastMonthForExchange  = RunEnvironment.getInstance().getParameters().getInteger("LastCAPExchangeMonth");
	  int month = cal.get(Calendar.MONTH);
	  if((month < firstMonthForExchange) || (month > lastMonthForExchange)) return 0;
	  int year = cal.get(Calendar.YEAR);
	  double originallyScheduled = 0;
	  double remainingScheduledSupply = 0;
	  Calendar cal2 = (Calendar)cal.clone();
	  for(int i = 0; i < 12; i++){
		  cal2.set(Calendar.MONTH, i);
		  double monthScheduled = UnitsAndConverters.GallonsToAF(schedule[i]) * cal2.getActualMaximum(Calendar.DAY_OF_MONTH);
		  if(i < month) originallyScheduled      += monthScheduled;
		  else          remainingScheduledSupply += monthScheduled;
	  }
	  double projectedTotalUsage = runningTotalAF + remainingScheduledSupply;
	  double fraction = totalScheduledAF / projectedTotalUsage;
	  maxOverageFraction = Math.max(maxOverageFraction, fraction);
	  minShortageFraction = Math.min(minShortageFraction, fraction);
	  
//	  LOGGER.debug("POLLING: " + getId() + ": ");
//	  LOGGER.debug("      Year/Month: " + year + "/" + month); 
//	  LOGGER.debug("      Orig Sched: " + originallyScheduled +" (AF), "); 
//	  LOGGER.debug("      TotalSched: " + totalScheduledAF + " (AF), ");
//	  LOGGER.debug("      RunTotal:   " + runningTotalAF + " (AF), ");
//	  LOGGER.debug("      Remaining:  " + remainingScheduledSupply + " (AF), ");
//	  LOGGER.debug("      Projected:  " + projectedTotalUsage + " (AF),");
//	  LOGGER.debug("      Ratio:      " + fraction + ", Over? " + (fraction > overageThreshold) + " Under? " + (fraction < shortageThreshold));
	  double toReturn = 0;
	  double diff = (projectedTotalUsage - totalScheduledAF);
	  if     (fraction > overageThreshold){
		  toReturn =  Math.min(0, Math.max(diff * amountToReturn, (remainingScheduledSupply * -1))); // Can't offer more than you have
		  AFOffered[month] = toReturn;
//		  LOGGER.debug("    " + getId() + " POLLING: RETURNING OVERAGE " + toReturn + " (Multiplier = " + amountToReturn + ")");
	  }
	  else if(fraction < shortageThreshold){
		  toReturn = Math.max(0,  Math.min(diff  * amountToObtain, remainingScheduledSupply));
		  AFRequested[month] = toReturn;
//		  LOGGER.debug("    " + getId() + " POLLING: REPORTING SHORTAGE " + toReturn + " (Multiplier = " + amountToObtain + ")");
	  }
//	  else{
//		  LOGGER.debug("    " + getId() + " POLLING: REPORTING NO CHANGE");
//	  }
	  LOGGER.debug("POLL_RECORD " + tick + " " + year + " " + month + " " + getId() + " " + originallyScheduled + " " + totalScheduledAF + " " + runningTotalAF + " " + remainingScheduledSupply + " " + projectedTotalUsage + " " + fraction + " " + (diff < -0.000000001 ? diff : 0) + " " + (toReturn < 0.0000000001 ? toReturn : 0) + " " + (diff > 0.000000001 ? diff : 0) + " " + (toReturn > 0.000000001 ? toReturn : 0));
	  return toReturn;
  }
  
  public void adjust(double AFToAddOrSubtract){
	  double tick = RunEnvironment.getInstance().getCurrentSchedule().getTickCount();
	  int month = cal.get(Calendar.MONTH);
	  if(AFToAddOrSubtract > 0){
		  AFDonated[month]  = AFToAddOrSubtract;
		  AFReceived[month] = 0;
	  }
	  else{
		  AFDonated[month]  = 0;
		  AFReceived[month] = Math.abs(AFToAddOrSubtract);		  
	  }
	  int monthsRemaining = 12 - month; // Note: usually run on 1st of month for remaining months
	  double adjustPerMonth = UnitsAndConverters.AFToGallons(AFToAddOrSubtract / monthsRemaining);
	  LOGGER.debug("    ADJUSTING: " + tick + " " + getId() + " " + AFToAddOrSubtract + " CHANGING totalScheduleAF from " + totalScheduledAF + " to " + (totalScheduledAF + AFToAddOrSubtract) + " for " + monthsRemaining + " remaining months at " + adjustPerMonth + " per month");
	  // At this point, assess the amount returned and apportion it through the remainder
	  // of the year. Note that this could be done in a variety of ways, but the easiest is
	  // to just apply it evenly to the months remaining
	  double originalTotalScheduled = totalScheduledAF;
	  totalScheduledAF += AFToAddOrSubtract;
	  Calendar localCal = (Calendar)cal.clone();
	  for(int i = month + 1; i < 12; i++){
		  localCal.add(Calendar.MONTH, 1);
		  LOGGER.debug("      CHANGING SCHEDULE FOR " + getId() + " FOR MONTH " + i + " FROM " + schedule[i] + " TO " + (adjustPerMonth/(localCal.getActualMaximum(Calendar.DATE))));
		  schedule[i] += adjustPerMonth/(localCal.getActualMaximum(Calendar.DATE));
	  }
//	  LOGGER.debug("  ADJUSTING: " + getId() + " MONTH = " + month + ", YEAR = " + cal.get(Calendar.YEAR) + "," + "," + adjustPerMonth);
	  LOGGER.debug("ADJUST_RECORD " + tick + " " + cal.get(Calendar.YEAR) + " " + month + " " + getId() + " " + originalTotalScheduled + " " + AFToAddOrSubtract + " " + (AFToAddOrSubtract < 0 ? AFToAddOrSubtract : 0) + " " + (AFToAddOrSubtract > 0 ? AFToAddOrSubtract : 0) + " " + totalScheduledAF + " " + monthsRemaining + " " + adjustPerMonth + " " + (adjustPerMonth < 0 ? adjustPerMonth : 0) + " " + (adjustPerMonth > 0 ? adjustPerMonth : 0));
  }


  
  public void setOverageThreshold(double newVal){
	  overageThreshold = newVal; 
  }
  public void setShortageThreshold(double newVal){
	  shortageThreshold = newVal; 
  }
  public void setAmountToReturn(double newVal){
	  amountToReturn = newVal; 
  }
  public void setAmountToObtain(double newVal){
	  amountToObtain = newVal; 
  }
  public void setNoisiness(double newVal){
	  noisiness = newVal; 
  }


  public void updateSimToDateTotals(){
	  Calendar localCal = (Calendar)cal.clone();
	  localCal.add(Calendar.DAY_OF_YEAR, -1);
	  LOGGER.debug("Updating to-date totals A for " + getId() + " in month " + cal.get(Calendar.MONTH) + " -> " + localCal.get(Calendar.MONTH));
	  
	  double totalShortageInc            = getShortageYTD(localCal);
	  double totalShortagePerCapitaInc   = getShortageYTDPerCapita(localCal);
	  double totalShortagePercentGPCDInc = getShortageYTDPerCapitaFractionGPCD(localCal); 

	  
	  LOGGER.debug("    TotalSimToDateShortage:            " + totalSimToDateShortage            + " + " + totalShortageInc            + " => " + (totalSimToDateShortage + totalShortageInc));
	  LOGGER.debug("    TotalSimToDateShortagePerCapita:   " + totalSimToDateShortagePerCapita   + " + " + totalShortagePerCapitaInc   + " => " + (totalSimToDateShortage + totalShortagePerCapitaInc));
	  LOGGER.debug("    TotalSimToDateShortagePercentGPCD: " + totalSimToDateShortagePercentGPCD + " + " + totalShortagePercentGPCDInc + " => " + (totalSimToDateShortage + totalShortagePercentGPCDInc));
	  
	  totalSimToDateShortage            += totalShortageInc;
	  totalSimToDateShortagePerCapita   += totalShortagePerCapitaInc;
	  totalSimToDateShortagePercentGPCD += totalShortagePercentGPCDInc;
	  years++;
  }
  
  public void optimizeThresholds(AnnualWaterRecord record){
	  double tick = RunEnvironment.getInstance().getCurrentSchedule().getTickCount();
//	  LOGGER.debug(this.getId() + " Year " + cal.get(Calendar.YEAR)+ " Optimizing Thresholds: BEFORE ShortageThresh= " + this.shortageThreshold + " OverageThresh= " + this.overageThreshold + " AmtToRequest= " + this.amountToObtain + " AmtToReturn= " + this.amountToReturn);
	  // Did you end last year with a surplus or a shortage?
	  double totalDemand    = 0;
	  double totalDelivered = 0;
	  double totalOffered   = 0;
	  double totalRequested = 0;
	  double totalDonated   = 0;
	  double totalReceived  = 0;
	  Calendar calDup = (Calendar)cal.clone();
	  calDup.add(Calendar.YEAR, -1); // Drop back one year; note that this is called on Jan 1 of the new year
	  for(int i = 0; i < 12; i++){
		  int daysInMonth = calDup.getActualMaximum(Calendar.DATE);
		  totalDemand    += UnitsAndConverters.GallonsToAF(record.demand[i] * daysInMonth);     // Actual demand; convert from Gallons per day to AF
		  totalDelivered += UnitsAndConverters.GallonsToAF(record.delivered[i] * daysInMonth);  // Actual delivered
		  totalOffered   += record.AFoffered[i];
		  totalRequested += record.AFrequested[i];
		  totalDonated   += record.AFdonated[i];
		  totalReceived  += record.AFreceived[i];
		  calDup.add(Calendar.MONTH, 1);
	  }
	  double totalScheduled = record.totalScheduledAF;	  
	  double surplusFraction  = ((totalScheduled - totalDelivered)/totalDelivered);
	  double shortageFraction = ((totalDemand - totalDelivered)/totalDelivered);
	  LOGGER.debug(this.getId() + " Year " + cal.get(Calendar.YEAR)+ " Optimizing Thresholds: TotalDelivered = " + totalDelivered + " totalScheduled " + totalScheduled + " totalDemand " + totalDemand + " surplusFrac " + surplusFraction + " shortageFrac " + shortageFraction + " NET: " + (totalScheduled > totalDemand ? " SURPLUS " : "SHORTAGE"));
	  
	  double shortageThresholdReduction = 0;
	  double amountToObtainReduction    = 0;
	  double overageThresholdReduction  = 0;
	  double amountToReturnIncrease     = 0;
	  double overageThresholdIncrease   = 0;
	  double amountToReturnReduction    = 0;
	  double shortageThresholdIncrease  = 0;
	  double amountToObtainIncrease     = 0;
	  
	  // Set values for reporting
	  this.lastYearDeliveredAF = totalDelivered;
	  this.lastYearScheduledAF = totalScheduled;
	  
	  // Surplus?
	  if((totalScheduled - totalDelivered) > .00005){ // Note: This avoids rounding errors by double math; represents: "if scheduled > delivered"
		  
		  // Did you request some during this year?
		  if(totalRequested > 0){ 
			  // Will need to raise the shortage threshold and lower the amount to request
			  double choiceWeight = RandomHelper.nextDoubleFromTo(0.25, 0.75);
			  
			  // Should have refrained from requesting;
			  // If you can move the threshold, do so
			  if(shortageThreshold > minShortageFraction){
				  shortageThresholdReduction = (surplusFraction * SHORTAGE_THRESH_RED_ADJ_RATE) * choiceWeight;
				  shortageThreshold -=  shortageThresholdReduction;
				  shortageThreshold = Math.max(minShortageFraction, shortageThreshold); // Can't go below floor
			  }
			  
			  // And should have requested less
			  amountToObtainReduction = (surplusFraction * AMT_REQ_RED_ADJ_RATE) * (1 - choiceWeight); 
			  amountToObtain -= amountToObtainReduction;
		      amountToObtain = Math.max(0, amountToObtain); // Can't go below zero
		  }
		  
		  // Whether you requested or not, you should have offered more
		  double choiceWeight = RandomHelper.nextDoubleFromTo(0.25, 0.75);
		  
		  // Be more eager to offer
		  overageThresholdReduction = (surplusFraction * OVER_THRESH_RED_ADJ_RATE) * choiceWeight; 
		  overageThreshold -=  overageThresholdReduction;
		  overageThreshold = Math.max(minShortageFraction, overageThreshold); // Can't go below the floor
		  shortageThreshold = Math.min(overageThreshold, shortageThreshold); // Move both downward if necessary
		  
		  // And offer more
		  amountToReturnIncrease = (surplusFraction * AMT_RET_INC_ADJ_RATE) * (1 - choiceWeight);
		  amountToReturn += amountToReturnIncrease;
	  }
	  
	  // Shortage?
	  else if((totalDemand - totalDelivered) > .00005){ // Avoid rounding errors from double math
		  // Did you offer to return some during this year?
		  if(Math.abs(totalOffered) > 0.00005){ 
			  // Will need to raise the overage threshold and lower the amount to offer
			  double choiceWeight = RandomHelper.nextDoubleFromTo(0.25, 0.75);
			  
			  // Should have refrained from offering
			  if(overageThreshold < maxOverageFraction){
				  overageThresholdIncrease = (shortageFraction * OVER_THRESH_INC_ADJ_RATE) * choiceWeight;
				  overageThreshold += overageThresholdIncrease;
				  overageThreshold = Math.min(maxOverageFraction, overageThreshold); // Can't go above ceiling
			  }
			  
			  // And should have offered less
			  amountToReturnReduction = (shortageFraction * AMT_RET_RED_ADJ_RATE) * (1 - choiceWeight); 
			  LOGGER.debug("REDUCING AMOUNT TO RETURN FOR " + this.getId() + " BY " + amountToReturnReduction);
			  amountToReturn -= amountToReturnReduction;
			  amountToReturn = Math.max(0, amountToReturn); // Can't go below zero
		  }
		  
		  double choiceWeight = RandomHelper.nextDoubleFromTo(0.25, 0.75);
		  
		  // be more eager to request
		  shortageThresholdIncrease = (shortageFraction * SHORTAGE_THRESH_INC_ADJ_RATE) * choiceWeight; 
		  shortageThreshold += shortageThresholdIncrease;
		  shortageThreshold = Math.min(shortageThreshold, maxOverageFraction); // Can't go above ceiling
		  overageThreshold = Math.max(overageThreshold,shortageThreshold); // Move both upward if necessary
		  
		  
		  // And requested more
		  amountToObtainIncrease = (shortageFraction * AMT_REQ_INC_ADJ_RATE) * (1 - choiceWeight);
		  amountToObtain += amountToObtainIncrease;
		    
	  }

	  
	  LOGGER.debug("OPTIMIZE_RECORD " + 
	      " Tick= " + tick + 
	      " Year= " + cal.get(Calendar.YEAR) + 
	      " Month= " + cal.get(Calendar.MONTH) + 
	      " City= " + this.getId() + 
	      " TotalDelivered= " + totalDelivered + 
	      " totalScheduled= " + totalScheduled + 
	      " totalDemand= " + totalDemand + 
	      " surplusFrac= " + surplusFraction + 
	      " shortageFrac= " + shortageFraction + 
	      " NET= " + (totalScheduled > totalDemand ? " SURPLUS " : "SHORTAGE") + 
	      " offered= " + totalOffered +
	      " donated= " + totalDonated +
	      " requested= " + totalRequested +
	      " received= " + totalReceived + 
	      " shortageThreshold= "  + this.shortageThreshold + 
	      " OverageThresh= " + this.overageThreshold + 
	      " AmtToRequest= " + this.amountToObtain + 
	      " AmtToReturn= " + this.amountToReturn + 
		  " shortageThresholdReduction= " + shortageThresholdReduction + 
	      " amountToObtainReduction= " + amountToObtainReduction +
	      " overageThresholdReduction= " + overageThresholdReduction +
	      " amountToReturnIncrease= " + amountToReturnIncrease +
	      " overageThresholdIncrease= " + overageThresholdIncrease +
	      " amountToReturnReduction= " + amountToReturnReduction +
	      " shortageThresholdIncrease= " + shortageThresholdIncrease +
	      " amountToObtainIncrease= " + amountToObtainIncrease
	  );
	  
	  
  }
  
  
  /**
   * Returns the shortage for the ytd in gallons
   * @param localCal
   * @return
   */
  public double getShortageYTD(Calendar localCal){
	  if(localCal == null) localCal = (Calendar)cal.clone(); 
      int month = localCal.get(Calendar.MONTH);
      localCal.set(Calendar.MONTH, 0);
	  double ret = 0;
	  int i = 0;
	  for( ; i < month; i++){
		  ret += (supplyRecord[i] - demandRecord[i]) * localCal.getActualMaximum(Calendar.DATE);
		  localCal.add(Calendar.MONTH, 1);
	  }
	  ret += (supplyRecord[i] - demandRecord[i]) * localCal.getActualMaximum(Calendar.DATE); // Add the current month's values
	  return ret;
  }
  
  public double getRemainingScheduled(){
	  return totalScheduledAF - this.runningTotalAF;
  }
  
  
  /**
   * Returns the shortage YTD, in gallons, divided by the population
   * @param localCal
   * @return
   */
  public double getShortageYTDPerCapita(Calendar localCal){
	  return getShortageYTD(localCal) / population;
  }
  
  /**
   * Returns the shortage as fraction of the (statistical baseline) gallons per capita per day
   * through the END of the current month
   * @param localCal
   * @return
   */
  public double getShortageYTDPerCapitaFractionGPCD(Calendar localCal){
	  if(localCal == null) localCal = (Calendar)cal.clone();
	  localCal.set(Calendar.DATE, localCal.getMaximum(Calendar.DATE));
	  int daysSoFar = localCal.get(Calendar.DAY_OF_YEAR);
	  return getShortageYTD(localCal) / (double)daysSoFar  / (double)population / (double)gpd;
  }
  
  
  /**
   * Gets the total simulation shortage as percent GPCD, averaged over all simulation years 
   * @return
   */
  public double getSimToDateShortageYTDPerCapitaFractionGPCDPerYear(){
	  return totalSimToDateShortagePercentGPCD / (double)years;
  }
  
  
  
  /* Repast-Accessible Reporting */
  // These getters take no arguments and return positive values;
  // all water units are acre-feet
  
  /**
   * Returns the shortage for the current month in acre-feet
   * @return
   */
  public double getShortage(){
	  int month = -1;
	  double ret = -1;
	  if (cal != null){
		  month = cal.get(Calendar.MONTH);
		  ret = Math.abs(UnitsAndConverters.GallonsToAF((supplyRecord[month] - demandRecord[month]) * cal.getMaximum(Calendar.DATE)));
	  }
//	  LOGGER.debug("Getting Shortage for " + getId() + " for month " + month + ": " + ret);
	  if(Math.abs(ret) < .00000001) ret = 0;
	  return ret;
  }
    
  /**
   * Returns the shortage, YTD, in acre-feet
   * @return
   */
  public double getShortageYTD(){
	  double ret = Math.abs(UnitsAndConverters.GallonsToAF(getShortageYTD(null)));
//	  LOGGER.debug("Getting Shortage YTD for " + getId() + ": " + ret);
	  if(Math.abs(ret) < .00000001) ret = 0;
	  return ret;
  }
  
  /**
   * Returns the shortage, YTD, in acre-feet per person
   * @return
   */
  public double getShortageYTDPerCapita(){
	  double ret = Math.abs(UnitsAndConverters.GallonsToAF(getShortageYTDPerCapita(null)));
	  LOGGER.debug("Getting Shortage YTD Per Capita for " + getId() + ": " + ret);
	  return ret;
  }
  
  public double getShortageYTDPerCapitaFractionGPCD(){
	  double ret = Math.abs(getShortageYTDPerCapitaFractionGPCD(null));
	  LOGGER.debug("Getting Shortage YTD Per Capita Fraction GPCD for " + getId() + ": " +ret);
	  return ret;
  }

  public double getFullSimShortageYTDPerYear(){
	  double ret = Math.abs(totalSimToDateShortage / (double)years);
	  LOGGER.debug("Getting Shortage YTD per year for " + id + " = " + ret);
	  return ret;
  }

  public double getFullSimShortageYTDPerCapitaPerYear(){
	  double ret = Math.abs(totalSimToDateShortagePerCapita / (double)years);
	  LOGGER.debug("Getting Shortage YTD Per Capita Fraction Per year for " + getId() + ": " + ret);
	  return ret;
  }
  
}
