package cnh.crb.regional;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.log4j.Logger;

import repast.simphony.engine.environment.RunEnvironment;
import cnh.crb.agent.BasicWaterManager;
import cnh.crb.hydro.RecipientFlowNode;
import cnh.crb.hydro.Storage;

public class CAWCD extends BasicWaterManager{

  private static Logger LOGGER = Logger.getLogger(CAWCD.class);
	
  List<CAPSubcontractor>           subcontractors          = new ArrayList<CAPSubcontractor>();
  Map<CAPSubcontractor, double[]>  scheduleAmounts         = new HashMap<CAPSubcontractor, double[]>();
  Map<CAPSubcontractor, double[]>  scheduleAmountsNextYear = new HashMap<CAPSubcontractor, double[]>();
  Map<CAPSubcontractor, double[]>  deliveredAmounts        = new HashMap<CAPSubcontractor, double[]>();
	
  
  private static double getTotal(Map<? extends Object, Double> map){
	  double ret = 0;
	  for(Double val : map.values()) ret += val;
	  return ret;
  }
  
  private static double getMax(Map<? extends Object, Double> map){
	  double ret = Double.NEGATIVE_INFINITY;
	  for(Double val : map.values()) ret = Math.max(ret, val);
	  return ret;
  }
  
  private static double getMin(Map<? extends Object, Double> map){
	  double ret = Double.POSITIVE_INFINITY;
	  for(Double val : map.values()) ret = Math.min(ret, val);
	  return ret;
  }
  
  // Assumes totalAvailable AND all values in map are >= 0
  private static <T> Map<T, Double> apportion_1(Map<T, Double> map, double totalAvailable){
	  Map<T, Double> ret = new HashMap<T, Double>();
	  double totalInMap = getTotal(map);
	  if(totalAvailable >= totalInMap){
		  ret.putAll(map); // If there is enough available that all get what they need, it's simple
		  return ret; 
	  }
	  double fraction = totalAvailable / totalInMap;
	  for(T item: map.keySet()) ret.put(item, map.get(item) * fraction);
	  return ret;
  }
  
  private static <T> Map<T, Double> apportion_2(Map<T, Double> map, double totalAvailable){
	  Map<T, Double> ret = new HashMap<T, Double>();
	  double totalInMap = getTotal(map);
	  if(totalAvailable >= totalInMap){
		  ret.putAll(map); // If there is enough available that all get what they need, it's simple
		  return ret; 
	  }
	  Map<T, Double> work = new HashMap<T, Double>();
	  work.putAll(map);
	  for(T item: map.keySet()) ret.put(item, 0d);
	  double remainingToDistribute = totalInMap;
	  while(work.size() > 0 && remainingToDistribute > 0.0000001){
	    double smallestValue = getMin(work);
	    remainingToDistribute -= smallestValue * work.size();
	    List<T> workList = new ArrayList<T>(work.keySet());
		for(T item: workList){
		  double newVal = work.get(item) - smallestValue;
		  if(newVal > 0.00000001) work.put(item, newVal);
		  ret.put(item, ret.get(item) + smallestValue);
		}
	  }
	  
	  return ret;
  }
   
  
  public CAWCD(Storage storage, String managerID){
    super(storage, managerID);
  }
  
  public void addWaterRecipient(RecipientFlowNode subcontractor){
	  super.addWaterRecipient(subcontractor);
	  if(subcontractor instanceof CAPSubcontractor) subcontractors.add((CAPSubcontractor)subcontractor);
  }

  // This should be called on October 1st
  public void getSchedules(){
//	  System.out.println("CAWCD GETTING SCHEDULES");
	  for(CAPSubcontractor subcontractor: subcontractors) scheduleAmountsNextYear.put(subcontractor, subcontractor.getNextYearSchedule());
  }
  
  // This should be called on the new year
  public void reschedule(){
	  scheduleAmounts.clear();
	  scheduleAmounts.putAll(scheduleAmountsNextYear);
	  scheduleAmountsNextYear.clear();
	  deliveredAmounts.clear();
  }
  
  public void pollForAdjustments(Calendar cal){
	  // Note: Calendar is only used for debugging
//	  System.out.println("CAWCD is Polling Subcontractors for adjustments...");
	  Map<CAPSubcontractor, Double> returns  = new HashMap<CAPSubcontractor, Double>();
	  Map<CAPSubcontractor, Double> requests = new HashMap<CAPSubcontractor, Double>();
	  double totalAvailable = 0;
	  double totalRequested = 0;
	  for(CAPSubcontractor subcontractor: subcontractors){
		  double revisionAmount = subcontractor.withinYearPreferredAdjustment();
		  if(revisionAmount > 0){
			  totalRequested += revisionAmount;
			  requests.put(subcontractor, revisionAmount);
		  }
		  if(revisionAmount < 0){
			  revisionAmount = -1 * revisionAmount;
			  totalAvailable += revisionAmount;
			  returns.put(subcontractor, revisionAmount);
		  }
	  }
	  double tick = RunEnvironment.getInstance().getCurrentSchedule().getTickCount();
	  LOGGER.debug("CAWCD_POLL " + tick + " " + cal.get(Calendar.YEAR) + " " + cal.get(Calendar.MONTH) + " " + totalAvailable + " " + totalRequested + " " + (totalAvailable > totalRequested ? totalRequested : totalAvailable));
	  
	  if((totalAvailable == 0) || (totalRequested == 0)){
		  LOGGER.debug("Polling = Nothing to do: Available = " + totalAvailable + " Requested: " + totalRequested);
		  return; // Nothing to do
	  }
	  else{
		  LOGGER.debug("Polling = REALLOCATING: Available = " + totalAvailable + " Requested: " + totalRequested);
	  }
	  
	  
	  if(totalAvailable > totalRequested) totalAvailable = totalRequested;

	  // Note that something subtle is going on here:
	  // This statement:
	  returns  = apportion_1(returns, totalAvailable);
	  // takes the amount of water being offered in return to the CAP, per subcontractor,
	  // and arrives at the amount of water that is being claimed _from_ each of these
	  // subcontractors.
	  // Conversely, this statement:
	  requests = apportion_1(requests, totalAvailable);
	  // takes the amount of water being requested, per subcontractor, and arrives at
	  // the amount of water that is being given to them in response to these requests

	  // After this, you can just distribute, making sure to use negative values for the donors:
	  for(Entry<CAPSubcontractor, Double> entry: returns.entrySet())  entry.getKey().adjust(entry.getValue() * -1);
	  for(Entry<CAPSubcontractor, Double> entry: requests.entrySet()) entry.getKey().adjust(entry.getValue());	  
  }
}
