package cnh.crb.regional.tucson;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import cnh.crb.econ.BillableEntity;
import cnh.crb.econ.SimpleCostCalculator;
import cnh.crb.hydro.Tap;
import cnh.crb.hydro.UnitsAndConverters.VolumeUnits;

/**
 * Tucson Sewer charges are a rate based on winter usage/month
 * 
 * @author murphy
 *
 */
public class TucsonHouseholdSewerChargeCalculator extends SimpleCostCalculator{

	public double globalDefault = 0;
	
	
	public void setGlobalDefault(double newGlobalDefault){
		globalDefault = newGlobalDefault;
	}
	
	public double calculateCost(Date date, double amt, BillableEntity entity){
		GregorianCalendar calendar = new GregorianCalendar();
		calendar.setTime(date);
		int year = calendar.get(Calendar.YEAR);
		// If it's before Mar 1, then you're still using last year's
		if(calendar.get(Calendar.MONTH) < 2) year--;

		Tap.Usage sewerUsage = entity.getMeteredWaterUse("SEWER_START_" + (year -1), "SEWER_END_" + year);
		
		// CORRECT IMPLEMENTATION
		// If it is valid, use it for the sewer maximum
//		double maxMonthlyAmount = TucsonWaterDept.sewerUsageIsValid(sewerUsage, year) ? sewerUsage.getMonthlyAverage(VolumeUnits.CCF) : globalDefault;
//		return super.calculateCost(date, Math.min(maxMonthlyAmount,  amt), entity);
		
		// CURRENT IMPLEMENTATION
		double sewerAmount = TucsonWaterDept.sewerUsageIsValid(sewerUsage, year) ? sewerUsage.getMonthlyAverage(VolumeUnits.CCF) : globalDefault;
		return super.calculateCost(date, sewerAmount, entity);

	}
	
}
