package cnh.crb.regional.tucson;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Map;

import cnh.crb.econ.BillableEntity;
import cnh.crb.econ.SurchargeCalculator;
import cnh.crb.hydro.Tap;
import cnh.crb.hydro.UnitsAndConverters.VolumeUnits;

public class TucsonHouseholdSummerSurchargeCalculator extends SurchargeCalculator{

	public double globalDefault; // The winter average of all Tucson users
	
	public TucsonHouseholdSummerSurchargeCalculator(Map<Date, Surcharge> surcharges) {
		super(surcharges);
		globalDefault = -1;
	}

	public void setGlobalDefault(double newGlobalDefault){ globalDefault = newGlobalDefault; }
	
	public double getAmountToCompare(Date date, double amt,	BillableEntity entity) {
		// Get the appropriate winter value from the household
		// Must know what years to use
		GregorianCalendar calendar = new GregorianCalendar();
		calendar.setTime(date);
		int year = calendar.get(Calendar.YEAR);
		// If it's before May 1, then you're still using last year's
		if(calendar.get(Calendar.MONTH) < 4) year--;
		Tap.Usage winterUsage = entity.getMeteredWaterUse("WINTER_START_" + (year -1), "WINTER_END_" + year);
		
		// Must determine if this is valid or not.
		boolean usageIsValid = TucsonWaterDept.winterUsageIsValid(winterUsage, year);

		// If it is valid, then just return it
		if(usageIsValid) return winterUsage.getMonthlyAverage(VolumeUnits.CCF);
		
		// If it is invalid, then maybe you will use the global default
		if(globalDefault > 0) return globalDefault;
		
		// If the global default is not yet set, then use an invalid but non-zero usage
		if(winterUsage.getDays() > 0) return winterUsage.getMonthlyAverage(VolumeUnits.CCF);
		
		// But if the usage is zero then just don't apply a surcharge		
		return -1;
	}

}
