/**
 * 
 */
package cnh.crb.stats;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

/**
 * Simple histogram class.
 * 
 * @author nick
 */
public class Histogram {

	private static interface Adder {

		void add(double val, TreeMap<Double, MutableDouble> map);
	}

	private class CountAdder implements Adder {

		public void add(double val, TreeMap<Double, MutableDouble> map) {
			Map.Entry<Double, MutableDouble> entry = (val == bins.get(bins.size() - 1) ? map.lowerEntry(val) : map.floorEntry(val));
			if (entry == null)
				throw new IllegalArgumentException("Value " + val + " is outside of bin range");
			++entry.getValue().val;
		}
	}

	private class SumAdder implements Adder {

		public void add(double val, TreeMap<Double, MutableDouble> map) {
			Map.Entry<Double, MutableDouble> entry = (val == bins.get(bins.size() - 1) ? map.lowerEntry(val) : map.floorEntry(val));
			if (entry == null)
				throw new IllegalArgumentException("Value " + val + " is outside of bin range");
			entry.getValue().val += val;
		}
	}

	private static class MutableDouble {
		double val = 0;
	}

	private TreeMap<Double, MutableDouble> map = new TreeMap<>();
	private List<Double> bins = new ArrayList<>();
	private Adder adder = new CountAdder();

	/**
	 * 
	 * @param edges
	 *            sorted list of bin edges (0.1, 0.2 ...)
	 * @param maxIsOpen
	 * @param sumBinValues
	 */
	public Histogram(double[] edges, boolean sumBinValues) {
		for (double val : edges) {
			map.put(val, new MutableDouble());
		}

		if (sumBinValues)
			adder = new SumAdder();

		bins.addAll(map.keySet());
	}

	public void addValue(double value) {
		adder.add(value, map);
	}

	public int binCount() {
		return map.size() - 1;
	}

	public String getBinTitle(int binIndex) {
		double lower = bins.get(binIndex);
		double higher = bins.get(binIndex + 1);
		return "[" + lower + " - " + higher + (binIndex + 1 == bins.size() - 1 ? "]" : ")");
	}

	public double getValue(int binIndex) {
		return map.get(bins.get(binIndex)).val;
	}
	
	public void reset() {
		for (Map.Entry<Double, MutableDouble> entry : map.entrySet()) {
			entry.getValue().val = 0;
		}
	}
}
