package cnh.crb.util;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import simphony.util.messages.MessageCenter;

public class RateBlocksFromRatesXLS {
	
	static MessageCenter msg = MessageCenter
			.getMessageCenter(RateBlocksFromRatesXLS.class);

	private Map<Integer, Double> infMap = new HashMap<>();

	private static final int YEAR = 0;
	private static final int BLOCK_1 = 3;
	private static final int BLOCK_2 = 4;
	private static final int BLOCK_3 = 5;
	private static final int BLOCK_4 = 6;
	private static final int BLOCK_RANGE_1 = 7;
	private static final int BLOCK_RANGE_2 = 8;
	private static final int BLOCK_RANGE_3 = 9;
	private static final int BLOCK_RANGE_4 = 10;

	public RateBlocksFromRatesXLS() {
		infMap.put(1993, 1.62);
		infMap.put(1994, 1.58);
		infMap.put(1995, 1.53);
		infMap.put(1996, 1.49);
		infMap.put(1997, 1.46);
		infMap.put(1998, 1.44);
		infMap.put(1999, 1.40);
		infMap.put(2000, 1.36);
		infMap.put(2001, 1.32);
		infMap.put(2002, 1.30);
		infMap.put(2003, 1.27);
		infMap.put(2004, 1.24);
		infMap.put(2005, 1.20);
		infMap.put(2006, 1.16);
		infMap.put(2007, 1.13);
		infMap.put(2008, 1.09);
		infMap.put(2009, 1.09);
		infMap.put(2010, 1.07);
		infMap.put(2011, 1.04);
		infMap.put(2012, 1.02);
		infMap.put(2013, 1.00);
	}

	public void run(String ratesFile) throws IOException {
		Workbook book = new XSSFWorkbook(new FileInputStream(ratesFile));
		//runRates(book);
		//runSewerRates(book);
		//runSewerCharges(book);
		//runMonthlyCharges(book);
		//runSurchargeCharges(book);
		runMiscFees(book);
		
	}
	
	private void runMiscFees(Workbook book) {
		Sheet sheet = book.getSheet("MiscFeesandBase");
		// iterate over the rows, skipping the first one
		
		for (Row row : sheet) {
			if (row.getRowNum() > 0) {
				int year = (int)row.getCell(YEAR).getNumericCellValue();
				double infRate = infMap.get(year);
				int month = (int)row.getCell(1).getNumericCellValue() - 1;
				double thresh = row.getCell(2).getNumericCellValue();
				double rate_1 = row.getCell(3).getNumericCellValue();
				double rate_2 = row.getCell(4).getNumericCellValue();
				
				String line = String.format("<entry year=\"%d\" month=\"%d\" rate_1=\"%.2f\" rate_2=\"%.2f\" threshold=\"%.2f\"/>", 
						year, month, rate_1 * infRate, rate_2 * infRate, thresh);
				msg.debug(line);
				
				if (year == 2013) break;
				
			}
		}
	}
	
	private void runSurchargeCharges(Workbook book) {
		Sheet sheet = book.getSheet("SurCharge");
		// iterate over the rows, skipping the first one
		
		for (Row row : sheet) {
			if (row.getRowNum() > 0) {
				int year = (int)row.getCell(YEAR).getNumericCellValue();
				double infRate = infMap.get(year);
				int month = (int)row.getCell(1).getNumericCellValue() - 1;
				double rate1 = row.getCell(2).getNumericCellValue();
				double rate2 = row.getCell(3).getNumericCellValue();
				double rate2Thresh = row.getCell(4).getNumericCellValue();
				
				String line = String.format("<entry year=\"%d\" month=\"%d\" rate_1=\"%.2f\" rate_2=\"%.2f\" rate_2_threshold=\"%.2f\"/>", 
						year, month, rate1 * infRate, rate2 * infRate, rate2Thresh);
				msg.debug(line);
				
				if (year == 2013) break;
				
			}
		}
	}
	
	private void runMonthlyCharges(Workbook book) {
		Sheet sheet = book.getSheet("Monthly Service Charge");
		// iterate over the rows, skipping the first one
		
		for (Row row : sheet) {
			if (row.getRowNum() > 0) {
				int year = (int)row.getCell(YEAR).getNumericCellValue();
				//System.out.println(year);
				double infRate = infMap.get(year);
				int month = (int)row.getCell(1).getNumericCellValue() - 1;
				double rate = row.getCell(3).getNumericCellValue();
				
				String line = String.format("<entry year=\"%d\" month=\"%d\" cost=\"%.2f\"/>", year, month, rate * infRate);
				msg.debug(line);
				
				if (year == 2013) break;
			}
		}
		
	}
	
	private void runSewerCharges(Workbook book) {
		Sheet sheet = book.getSheet("SewerMonthlyCharge");
		// iterate over the rows, skipping the first one
		
		for (Row row : sheet) {
			if (row.getRowNum() > 0) {
				int year = (int)row.getCell(YEAR).getNumericCellValue();
				//System.out.println(year);
				double infRate = infMap.get(year);
				int month = (int)row.getCell(1).getNumericCellValue() - 1;
				double rate = row.getCell(4).getNumericCellValue();
				
				String line = String.format("<entry year=\"%d\" month=\"%d\" cost=\"%.2f\"/>", year, month, rate * infRate);
				msg.debug(line);
				
				if (year == 2013) break;
			}
		}
		
	}
	
	private void runSewerRates(Workbook book) {
		Sheet sheet = book.getSheet("SewerCommodityRates");
		// iterate over the rows, skipping the first one
		
		for (Row row : sheet) {
			if (row.getRowNum() > 0) {
				int year = (int)row.getCell(YEAR).getNumericCellValue();
				//System.out.println(year);
				double infRate = infMap.get(year);
				int month = (int)row.getCell(1).getNumericCellValue() - 1;
				double rate = row.getCell(4).getNumericCellValue();
				
				String line = String.format("<entry year=\"%d\" month=\"%d\" cost=\"%.2f\"/>", year, month, rate * infRate);
				msg.debug(line);
				
			}
		}
		
	}
	
	private void runRates(Workbook book) {
		
		// get the first worksheet
		Sheet sheet = book.getSheet("Singlefamily");
		// iterate over the rows, skipping the first one
		
		for (Row row : sheet) {
			if (row.getRowNum() > 0) {
				int year = (int)row.getCell(YEAR).getNumericCellValue();
				//System.out.println(year);
				double infRate = infMap.get(year);
				
				Double block1 = getValue(row, BLOCK_1, infRate);
				Double block2 = getValue(row, BLOCK_2, infRate);
				Double block3 = getValue(row, BLOCK_3, infRate);
				Double block4 = getValue(row, BLOCK_4, infRate);

				String thresh_1 = parseThreshold(row, BLOCK_RANGE_1);
				String thresh_2 = parseThreshold(row, BLOCK_RANGE_2);
				String thresh_3 = parseThreshold(row, BLOCK_RANGE_3);
				String thresh_4 = parseThreshold(row, BLOCK_RANGE_4);

				StringBuilder buf = new StringBuilder("<rate_table type=\"block\" id=\"block_" + year
						+ "\" flat_fee=\"0\">\n");
				buf.append(String.format("<rate_block threshold=\"%s\" rate=\"%.2f\"/>%n", thresh_1,
						block1.doubleValue()));
				buf.append(String.format("<rate_block threshold=\"%s\" rate=\"%.2f\"/>%n", thresh_2,
						block2.doubleValue()));
				buf.append(String.format("<rate_block threshold=\"%s\" rate=\"%.2f\"/>%n", thresh_3,
						block3.doubleValue()));
				if (block4 != null)
					buf.append(String.format("<rate_block threshold=\"%s\" rate=\"%.2f\"/>%n",
							thresh_4, block4.doubleValue()));

				buf.append("</rate_table>");

				msg.debug(buf);
			}
		}

	}

	private String parseThreshold(Row row, int index) {
		Cell cell = row.getCell(index);
		if (cell.getCellType() == Cell.CELL_TYPE_BLANK)
			return null;
		String val = cell.getStringCellValue().trim();
		if (val.contains("Over"))
			return "max";
		return val.substring(val.lastIndexOf(" ") + 1, val.length()).trim();
	}

	public Double getValue(Row row, int index, double infRate) {
		Cell cell = row.getCell(index);
		if (cell.getCellType() == Cell.CELL_TYPE_BLANK)
			return null;
		return cell.getNumericCellValue() * infRate;
	}

	public static void main(String[] args) {
		try {
			new RateBlocksFromRatesXLS().run("./data/tucson_rates.xlsx");
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

}
