/**
 * 
 */
package cnh.crb.util;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Serializable;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Queue;
import java.util.concurrent.Callable;
import java.util.concurrent.CompletionService;
import java.util.concurrent.Executor;
import java.util.concurrent.ExecutorCompletionService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

import org.geotools.data.DataStore;
import org.geotools.data.DataStoreFinder;
import org.geotools.data.DataUtilities;
import org.geotools.data.DefaultTransaction;
import org.geotools.data.FeatureSource;
import org.geotools.data.Transaction;
import org.geotools.data.collection.ListFeatureCollection;
import org.geotools.data.shapefile.ShapefileDataStore;
import org.geotools.data.shapefile.ShapefileDataStoreFactory;
import org.geotools.data.simple.SimpleFeatureCollection;
import org.geotools.data.simple.SimpleFeatureSource;
import org.geotools.data.simple.SimpleFeatureStore;
import org.geotools.feature.FeatureCollection;
import org.geotools.feature.FeatureIterator;
import org.geotools.feature.SchemaException;
import org.geotools.feature.simple.SimpleFeatureBuilder;
import org.geotools.filter.text.cql2.CQL;
import org.geotools.filter.text.cql2.CQLException;
import org.geotools.geometry.jts.JTS;
import org.geotools.geometry.jts.JTSFactoryFinder;
import org.geotools.referencing.CRS;
import org.geotools.referencing.crs.DefaultGeographicCRS;
import org.opengis.feature.simple.SimpleFeature;
import org.opengis.feature.simple.SimpleFeatureType;
import org.opengis.geometry.MismatchedDimensionException;
import org.opengis.referencing.FactoryException;
import org.opengis.referencing.crs.CoordinateReferenceSystem;
import org.opengis.referencing.operation.MathTransform;
import org.opengis.referencing.operation.TransformException;

import simphony.util.messages.MessageCenter;
import au.com.bytecode.opencsv.CSVReader;
import au.com.bytecode.opencsv.CSVWriter;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.Point;

/**
 * Trims the PIMA county synth population to only those in the tuscon water
 * area.
 * 
 * @author NC
 */
public class TucsonHouseholdsCreator {
	
	static MessageCenter msg = MessageCenter
			.getMessageCenter(TucsonHouseholdsCreator.class);

	private static class Checker implements Callable<Checker> {

		private String[] line;
		private List<Geometry> geoms;
		private Point pt;
		private boolean passed = false;

		public Checker(List<Geometry> geoms) {
			this.geoms = new ArrayList<Geometry>();
			for (Geometry geom : geoms) {
				this.geoms.add((Geometry) geom.clone());
			}
		}

		public void reset(String[] line, Point pt) {
			passed = false;
			this.line = line;
			this.pt = pt;
		}

		@Override
		public Checker call() throws Exception {
			for (Geometry geom : geoms) {
				if (geom.contains(pt)) {
					passed = true;
					break;
				}

			}
			return this;
		}
	}

	// private static final int ID_IDX = 0;
	// private static final int RACE_IDX = 3;
	// private static final int INCOME_IDX = 4;
	// private static final int SIZE_IDX = 5;
	// private static final int AGE_IDX = 6;
	private static final int LAT_IDX = 7;
	private static final int LON_IDX = 8;

	private static final int THREAD_COUNT = 6;

	private List<Geometry> geoms = new ArrayList<>();
	private List<SimpleFeature> features;
	private SimpleFeatureType type;
	private Executor executor = Executors.newFixedThreadPool(THREAD_COUNT);
	
	@SuppressWarnings({ "rawtypes", "unchecked" })
	private void readGeometry() throws IOException, FactoryException, MismatchedDimensionException,
			TransformException, CQLException {
		File file = new File("./data/gisdata/AZ_Water_Municipal_Service_Areas/muniServiceAreas.shp");
		Map<String, URL> map = new HashMap<>();
		map.put("url", file.toURI().toURL());

		DataStore dataStore = DataStoreFinder.getDataStore(map);
		String typeName = dataStore.getTypeNames()[0];

		FeatureSource source = dataStore.getFeatureSource(typeName);

		// get only the water co rows for 2008
		FeatureCollection collection = source.getFeatures(CQL.toFilter("YEAR_ = 2008"));
		CoordinateReferenceSystem crs = collection.getSchema().getCoordinateReferenceSystem();

		MathTransform transform = CRS.findMathTransform(crs, DefaultGeographicCRS.WGS84, true);

		FeatureIterator<SimpleFeature> results = collection.features();
		try {
			while (results.hasNext()) {
				SimpleFeature feature = (SimpleFeature) results.next();
				// System.out.println(feature.getAttribute("the_geom"));
				String waterco = feature.getAttribute("WATERCO").toString().toLowerCase();
				// only areas for tucson but not the prisons
				if (waterco.contains("tucson") && !waterco.contains("prison")) {
					Geometry geom = (Geometry) feature.getDefaultGeometry();
					Geometry wgsGeom = JTS.transform(geom, transform);
					geoms.add(wgsGeom);
				}
			}
		} finally {
			results.close();
			dataStore.dispose();
		}

	}

	private boolean isInTucson(Point pt) {
		for (Geometry geom : geoms) {
			if (geom.contains(pt))
				return true;
		}
		return false;
	}

	private void processHouseholds() throws IOException, SchemaException {
		type = DataUtilities.createType("household",
				"the_geom:Point:srid=4326,hid:String,race:int,income:int," + "size:int,age:int");

		List<SimpleFeature> features = new ArrayList<SimpleFeature>();
		GeometryFactory geometryFactory = JTSFactoryFinder.getGeometryFactory();
		SimpleFeatureBuilder fb = new SimpleFeatureBuilder(type);

		Queue<Checker> checkers = new LinkedList<>();
		for (int i = 0; i < THREAD_COUNT; ++i) {
			checkers.add(new Checker(geoms));
		}

		try (CSVReader reader = new CSVReader(new BufferedReader(new FileReader(
				"./data/population/AZ_Pima_2005_2009_ver2_04019_synth_households.txt")));
				CSVWriter writer = new CSVWriter(new BufferedWriter(new FileWriter(
						"./data/population/Tucson_2005_2009_ver2_04019_synth_households.txt")))) {
			// header
			String[] line = reader.readNext();
			writer.writeNext(line);
			int count = 0;
			int c = 0;
			
			CompletionService<Checker> service = new ExecutorCompletionService<>(executor);

			boolean done = false;
			while (!done) {
				int running = 0;
				for (int i = 0; i < THREAD_COUNT; ++i) {
					line = reader.readNext();
					if (line == null) {
						done = true;
						break;
					} else {
						Checker checker = checkers.poll();
						double lat = Double.parseDouble(line[LAT_IDX]);
						double lon = Double.parseDouble(line[LON_IDX]);
						Point point = geometryFactory.createPoint(new Coordinate(lon, lat));
						checker.reset(line, point);
						++running;
						service.submit(checker);
					}
				}

				for (int i = 0; i < running; ++i) {
					Future<Checker> future = service.take();
					Checker checker = future.get();
					if (checker.passed) {
						writer.writeNext(checker.line);
						if (++count % 1000 == 0) {
							msg.debug(count + "...");
							if (++c == 10) {
								msg.debug("");
								c = 0;
							}
						}
					}
					checkers.add(checker);
				}
			}

//			while ((line = reader.readNext()) != null) {
//				Checker checker = checkers.take();
//				double lat = Double.parseDouble(line[LAT_IDX]);
//				double lon = Double.parseDouble(line[LON_IDX]);
//				Point point = geometryFactory.createPoint(new Coordinate(lon, lat));
//				checker.reset(line, point);
//				service.submit(checker);
//
//				if (isInTucson(point)) {
//					writer.writeNext(line);
//					// fb.add(point);
//					// fb.add(line[ID_IDX].trim());
//					// fb.add(Integer.parseInt(line[RACE_IDX]));
//					// fb.add(Integer.parseInt(line[INCOME_IDX]));
//					// fb.add(Integer.parseInt(line[SIZE_IDX]));
//					// fb.add(Integer.parseInt(line[AGE_IDX]));
//					// features.add(fb.buildFeature("fid." + i));
//					// print out some status
//
//				}
//			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}

		// writeShapeFile();

	}

	private void writeShapeFile() throws IOException {
		File newFile = new File("./data/gisdata/tucson_households/tucson_synth_households.shp");
		if (!newFile.getParentFile().exists()) {
			newFile.getParentFile().mkdirs();
		}

		ShapefileDataStoreFactory dataStoreFactory = new ShapefileDataStoreFactory();
		Map<String, Serializable> params = new HashMap<String, Serializable>();
		params.put("url", newFile.toURI().toURL());
		params.put("create spatial index", Boolean.TRUE);

		ShapefileDataStore newDataStore = (ShapefileDataStore) dataStoreFactory
				.createNewDataStore(params);
		newDataStore.createSchema(type);

		Transaction transaction = new DefaultTransaction("create");

		String typeName = newDataStore.getTypeNames()[0];
		SimpleFeatureSource featureSource = newDataStore.getFeatureSource(typeName);

		SimpleFeatureStore featureStore = (SimpleFeatureStore) featureSource;
		SimpleFeatureCollection collection = new ListFeatureCollection(type, features);
		featureStore.setTransaction(transaction);
		try {
			featureStore.addFeatures(collection);
			transaction.commit();
		} catch (Exception problem) {
			problem.printStackTrace();
			transaction.rollback();
		} finally {
			transaction.close();
		}
	}

	public void run() {
		try {
			readGeometry();
			processHouseholds();
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	public static void main(String[] args) {
		new TucsonHouseholdsCreator().run();
	}

}
