package cnh.crb.util;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;

import repast.simphony.parameter.ParameterSchema;
import repast.simphony.parameter.Parameters;
import repast.simphony.parameter.ParametersParser;
import repast.simphony.parameter.Schema;
import au.com.bytecode.opencsv.CSVReader;

public class UPFfromCSVandSweep {

	private static final String UPF_FILE_NAME = "upf_from_csv.txt";

	Parameters parameters;
	String csvFilePath, sweepParamsFilePath;

	String outPath;
	String outFile;

	public UPFfromCSVandSweep(String paramsFilePath, String csvFilePath,
			String sweepParamsFilePath, String outPath, String outFile) {
		try {
			File paramsFile = new File(paramsFilePath);
			ParametersParser pp;
			pp = new ParametersParser(paramsFile);
			parameters = pp.getParameters();
			this.outFile = outFile == null ? UPF_FILE_NAME : outFile;
			this.csvFilePath = csvFilePath;
			this.sweepParamsFilePath = sweepParamsFilePath;
			this.outPath = outPath;

		} catch (ParserConfigurationException | SAXException | IOException e) {
			e.printStackTrace();
		}
	}

	String[] sweepParamNames;
	List<String[]> sweepParamValueCombinations;

	protected void readSweepParams() {
		sweepParamValueCombinations = new ArrayList<>();
		List<List<String>> sweepParamValues = new ArrayList<>();
		Path path = Paths.get(sweepParamsFilePath);
		Charset charset = Charset.forName("UTF-8");
		try {
			List<String> lines = Files.readAllLines(path, charset);
			List<String> paramNamesList = new ArrayList<>();
			for (String line : lines) {
				if (line.indexOf(":") != -1) {
					String[] result = line.split(":");
					paramNamesList.add(result[0]);
					String[] vals = result[1].trim().split(",");
					sweepParamValues.add(Arrays.asList(vals));
				}
			}
			sweepParamNames = paramNamesList.toArray(new String[0]);
			List<List<String>> productResult = ListProductUtil
					.product(sweepParamValues);
			for (List<String> combination : productResult){
				sweepParamValueCombinations.add(combination.toArray(new String[0]));
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void create() {
		readSweepParams();
		Charset charset = Charset.forName("UTF-8");
		Path path = Paths.get(outPath, outFile);
		List<String> paramNames = new ArrayList<>();
		for (String pName : parameters.getSchema().parameterNames()) {
			paramNames.add(pName);
		}

		try (BufferedWriter writer = Files.newBufferedWriter(path, charset);
				CSVReader reader = new CSVReader(new FileReader(csvFilePath))) {
			String[] nextLine;
			int lineNumber = 0;
			String[] csvParamNames = null;
			while ((nextLine = reader.readNext()) != null) {
				if (lineNumber == 0) {
					csvParamNames = nextLine;
					lineNumber++;
				} else {
					setParameters(csvParamNames, nextLine);
					for (String [] combination : sweepParamValueCombinations) {
						setParameters(sweepParamNames, combination);
						// nextLine[] is an array of values from the line
						writer.write(String.valueOf(lineNumber));
						writer.write("\t");

						boolean addComma = false;
						for (String pName : paramNames) {
							if (addComma)
								writer.write(",");
							writer.write(pName);
							writer.append("\t");
							writer.append(parameters.getValueAsString(pName));
							addComma = true;
						}
						writer.append("\n");
						lineNumber++;
					}
				}
				
			}
			writer.flush();
		} catch (IOException x) {
			System.err.format("IOException: %s%n", x);
		}

	}

	private void setParameters(String[] paramNames, String[] paramVals) {
		Map<String, Object> pvs = new LinkedHashMap<String, Object>();
		Schema schema = parameters.getSchema();
		for (int i = 0; i < paramNames.length; i++) {
			String p = paramNames[i];
			ParameterSchema pSchema = schema.getDetails(p);
			Object obj = pSchema.fromString(paramVals[i]);
			pvs.put(p, obj);
		}
		// set params with pvs
		for (String key : pvs.keySet()) {
			Object obj = pvs.get(key);
			parameters.setValue(key, obj);
		}
	}

	public static void main(String[] args) {
		new UPFfromCSVandSweep("CNH_CRB.rs/parameters_20140905.xml", "misc/onlyParams_top10.csv",
				"misc/sweepParams.txt", "out", "upf_20140905out.txt").create();
	}

}
