package cnh.crb.util;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

public class Utilities {

	private static final int[] DAYS_IN_MONTH = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
	
	public static long daysBetweenDates(GregorianCalendar start, GregorianCalendar end){
		GregorianCalendar tracker = new GregorianCalendar(start.get(Calendar.YEAR), start.get(Calendar.MONTH), start.get(Calendar.DAY_OF_MONTH));

		Calendar endCal = Calendar.getInstance();
		endCal.set(end.get(Calendar.YEAR), end.get(Calendar.MONTH), end.get(Calendar.DAY_OF_MONTH),0,0,0);
		endCal.set(Calendar.MILLISECOND, 0);
		
		int count = 0;
		while(tracker.before(endCal)){
			// If it's Jan 1, you may be able to skip a year
			boolean skippedYear = false;
			if(tracker.get(Calendar.DAY_OF_YEAR) == 0){
				Calendar skip = (Calendar)tracker.clone();
				skip.add(Calendar.YEAR, 1);
				if(skip.before(endCal)){
					count+= (tracker.isLeapYear(tracker.get(Calendar.YEAR)) ? 366 : 365);
					tracker.add(Calendar.YEAR, 1);
					skippedYear = true;
				}
			}
			// If you skipped one year, you'd prefer to stay on Jan 1- maybe you can skip another year
			boolean skippedMonth = false;
			if(!skippedYear && tracker.get(Calendar.DAY_OF_MONTH) == 0){
				Calendar skip = (Calendar)tracker.clone();
				skip.add(Calendar.MONTH, 1);
				if(skip.before(endCal)){
					count+= daysInMonth(tracker);
					tracker.add(Calendar.MONTH, 1);
					skippedMonth = true;
				}
			}
			// If you didn't do any skipping, just add a day
			if(!skippedYear && ! skippedMonth){
				tracker.add(Calendar.DAY_OF_YEAR, 1);
				count++;
			}
		}
		return count;
  
	}
	
	public static long daysBetweenDates(Date start, Date end){
		GregorianCalendar startCal = (GregorianCalendar)GregorianCalendar.getInstance();
		startCal.setTime(start);
		GregorianCalendar endCal = (GregorianCalendar)GregorianCalendar.getInstance();
		endCal.setTime(end);
		return daysBetweenDates(startCal, endCal);  
	}
	
	public static int daysInMonth(GregorianCalendar cal){
		int month = cal.get(Calendar.MONTH);
		return (cal.isLeapYear(cal.get(Calendar.YEAR)) && month == 1) ? 29 : DAYS_IN_MONTH[month];
	}
	

	
	/**
	 * Clamps the val to a range between min and max, inclusive.
	 * 
	 * @param val
	 * @param min
	 * @param max
	 * 
	 * @return the clamped value.
	 */
	public static double clamp(double val, double min, double max) {
		if (val < min) return min;
		if (val > max) return max;
		return val;
	}
	
}
