/**
 * 
 */
package cnh.crb.util;

import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import au.com.bytecode.opencsv.CSVReader;

/**
 * Reads the yearly by month water data csv file and into an xml format file.
 * 
 * @author nick
 * 
 */
public class WaterDataFileCreator {

	private static final int SERVICES = 3;
	private static final int TOTAL_DEVS = 4;
	private static final int CFS_PER_SERV = 5;
	private static final int GALLONS_PER_DAY = 6;

	static class Month {
		int month;
		float services, totalDeliveries;
		float cfsPerService, gallonsPerDay;
		float fracOfMean;

		public Month(int month, String[] vals) {
			this.month = month;
			services = Float.parseFloat(vals[SERVICES].trim());
			totalDeliveries = Float.parseFloat(vals[TOTAL_DEVS].trim());
			cfsPerService = Float.parseFloat(vals[CFS_PER_SERV].trim());
			gallonsPerDay = Float.parseFloat(vals[GALLONS_PER_DAY].trim());
		}
	}

	static class Year {
		int year;
		float mean;
		List<Month> months = new ArrayList<>();

		void add(Month month) {
			months.add(month);
		}

		void run() {
			float sum = 0;
			for (Month month : months) {
				sum += month.gallonsPerDay;
			}
			mean = sum / months.size();

			for (Month month : months) {
				month.fracOfMean = month.gallonsPerDay / mean;
			}
		}
	}

	public void run(String in, String out) throws IOException {
		List<Year> years = readData(in);
		writeData(out, years);
	}

	private void writeData(String out, List<Year> years) throws IOException {
		try (BufferedWriter writer = new BufferedWriter(new FileWriter(out))) {
			writer.write("<years>\n");
			for (Year year : years) {
				writer.write(String
						.format("<year date=\"%d\" mean=\"%f\">%n", year.year, year.mean));
				for (Month month : year.months) {
					writer.write(String.format(
							"<month date=\"%d\" fracOfMean=\"%f\" gallonsPerDay=\"%f\" "
									+ "services=\"%f\" deliveries=\"%f\" cfsPerService=\"%f\"/>%n",
							month.month, month.fracOfMean, month.gallonsPerDay, month.services,
							month.totalDeliveries, month.cfsPerService));
				}
				writer.write("</year>\n");
			}

			writer.write("</years>\n");

		}
	}

	private List<Year> readData(String in) throws IOException {
		List<Year> years = new ArrayList<>();
		try (CSVReader reader = new CSVReader(new FileReader(in))) {
			List<String[]> data = reader.readAll();

			Year year = new Year();
			int month = 1;
			for (int i = 1; i < data.size(); ++i) {
				String[] line = data.get(i);
				int iYear = Integer.parseInt(line[0].trim());
				if (year.year != iYear) {
					year = new Year();
					years.add(year);
					year.year = iYear;
					month = 1;
				}
				year.add(new Month(month, line));
				++month;
			}
		}

		for (Year year : years) {
			year.run();
		}
		return years;
	}
	
	public static void main(String[] args) {
		WaterDataFileCreator creator = new WaterDataFileCreator();
		try {
			creator.run(args[0], args[1]);
		} catch (IOException ex) {
			ex.printStackTrace();
		}
	}
}
