package cnh.crb.wbm;

import repast.simphony.context.Context;
import cnh.crb.farm.Crop;
import cnh.crb.farm.IrrigatedAgFarmer;
import cnh.crb.hydro.FlowManager;
import cnh.crb.hydro.UnitsAndConverters;
import cnh.crb.hydro.UnitsAndConverters.VolumeUnits;
import cnh.crb.regional.CAWCD;

public class WBMCrop implements Crop{

	public static boolean useWBMDemand = true;
	
	// Provided by WBM
	public String  ID;
	public int     id;                       // ID
	public double  fraction;                 // Fraction of cell occupied by this field
	public double  soilMoisture;             // Current water in field, in mm
	public double  soilMoistureFraction;     // Fraction of water vs. total possible
	public double  wbmDemand;                // Amount the WBM has calculated is needed
	
	// Calculated
	public double  area;                     // Area of this field, in sq m
	public double  fieldCapacity;            // Deepest water possible in field, in mm
	public boolean active;                   // When inactive, may be culled
	public double  mmApplied = 0;            // Volume of water applied
	public IrrigatedAgFarmer farmer;
	public Context context;
	
	
	public WBMCrop(String ID, int id, double fraction, double area, double soilMoisture, double soilMoistureFraction, double wbmDemand, Context context){
//		System.out.println("Creating crop with ID: " + id);
		this.ID = ID;
		this.id = id;
		this.fraction = fraction;
		this.area = area;
		this.soilMoisture = soilMoisture;
		this.soilMoistureFraction = soilMoistureFraction;
		this.wbmDemand = wbmDemand;
		this.context = context;
		
		fieldCapacity = soilMoisture / soilMoistureFraction;
		mmApplied = 0;
		farmer = new IrrigatedAgFarmer(this);
		context.add(farmer);
		for(Object mgr: context.getObjects(CAWCD.class)){
			CAWCD cawcd = (CAWCD)mgr;
			cawcd.addWaterRecipient(farmer);
			break;
		}
		for(Object mgr: context.getObjects(FlowManager.class)){
			FlowManager fmgr = (FlowManager)mgr;
			fmgr.addFlowNode(farmer);
			break;
		}

	}
	
	public void set(double fraction, double area, double soilMoisture, double soilMoistureFraction, double wbmDemand){
		clear();
		this.fraction = fraction;
		this.area = area;
		this.soilMoisture = soilMoisture;
		this.soilMoistureFraction = soilMoistureFraction;
		this.wbmDemand = wbmDemand;
		fieldCapacity = soilMoisture / soilMoistureFraction;
	}
	
	public double getDemand(VolumeUnits units){
		double d = useWBMDemand ? UnitsAndConverters.convertVolume(((wbmDemand / 0.6) / 1000) * area, VolumeUnits.M3, units) : 0; // Note: 0.6 is 'irrigation efficiency'
		return d;
	}
	
	public void applyWater(double volume, VolumeUnits units){
		/*if(volume > 0) */System.out.println("CROP " + ID + " with crop " + id + " RECEIVING " + volume + " water with area " + area);
		if(area <= 0) return;
		// Convert to m^3
		double m3Vol = UnitsAndConverters.convertVolume(volume, units, VolumeUnits.M3);
		// Convert to mm
		double mm = (m3Vol / area) * 1000;
		mmApplied += mm;
		System.out.println("ADDING " + mm + " millimeters");
		soilMoisture += mm;
		soilMoistureFraction = soilMoisture / fieldCapacity;
	}
	
	public void clear(){
		mmApplied = 0;
	}
	
	public void removeManager(){
		for(Object mgr: context.getObjects(FlowManager.class)){
			FlowManager fmgr = (FlowManager)mgr;
			fmgr.removeFlowNode(farmer);
			break;
		}
		for(Object mgr: context.getObjects(CAWCD.class)){
			CAWCD cawcd = (CAWCD)mgr;
			cawcd.removeWaterRecipient(farmer);
			break;
		}
		context.remove(farmer);
	}
}
