/**
 * 
 */
package cnh.crb.wbm;

import java.awt.geom.Point2D;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.zeromq.ZMQ;

import repast.simphony.util.collections.Pair;
import simphony.util.messages.MessageCenter;

/**
 * @author nick
 * 
 */
public class WBMProxy {
	
	static MessageCenter msg = MessageCenter.getMessageCenter(WBMProxy.class);

	public static final String HEADER = "date,lat,lon,,gridID,gridullat,gridullong,gridlrlat,gridlrlong,available\n";

	int steps = 0;
	
	class ReturnData{
		double surfaceFlow;
		double baseFlow;
	}
	
	private ZMQ.Context                                 context;
	private ZMQ.Socket                                  responder;
	private Map<Point2D.Double, WBMNaturalWaterSource>  sources     = new HashMap<Point2D.Double, WBMNaturalWaterSource>();

	public WBMProxy(){
		connect();
	}
	
	public WBMProxy(WBMWaterSourceCollection collection) {
		for(Point2D.Double key: collection.waterSources.keySet()) sources.put(key,  collection.waterSources.get(key));
		connect();
	}
	
	private void connect(){
		context = ZMQ.context(1);
		responder = context.socket(ZMQ.REP);
		responder.bind("tcp://0.0.0.0:5562");
	}

	public void run() {
		// Wait for a send
		msg.debug("PROXY WAITING...");
		String requested = responder.recvStr(StandardCharsets.UTF_8);
		if(requested.compareTo("END") == 0){
			close();
			return;
		}
		List<Pair<Point2D.Double, ReturnData> > ret = new ArrayList<Pair<Point2D.Double, ReturnData> >();
		try (BufferedReader reader = new BufferedReader(new StringReader(requested))) {
			String line = reader.readLine();  	// Skips header
			while((line = reader.readLine()) != null){
				String[] vals = line.split(",");
				Point2D.Double point =  new Point2D.Double(Double.parseDouble(vals[1].trim()), Double.parseDouble(vals[0].trim()));
				WBMNaturalWaterSource nws = sources.get(point); 
			    if(nws == null){
			    	WBMNaturalWaterSource newSource = new WBMNaturalWaterSource("PROXY", point);
			    	newSource.setDefaultFlow(10000000);
					sources.put(point, newSource);
					nws = newSource;
				}
			    ReturnData rd = new ReturnData();
			    rd.surfaceFlow = nws.getDefaultFlow();
			    rd.baseFlow = nws.getDefaultFlow() / 100;
				ret.add(new Pair<Point2D.Double, ReturnData>(point, rd));
			}
		} catch (IOException e) {}
		
		String valsToSend = "";
		for(Pair<Point2D.Double, ReturnData> pointVal: ret){
			String gridID = (int)(pointVal.getFirst().y * 10) + "_" + (int)(pointVal.getFirst().x * 10);
			valsToSend += "YYYY_MM_DD," +
					      pointVal.getFirst().y + "," + pointVal.getFirst().x + 
					      ",GRID_ID_"  + gridID + "," +                                                              // Some will have unique ids, but to test can make it not 
		                  pointVal.getFirst().y + "," + pointVal.getFirst().x + "," +
			              pointVal.getFirst().y + "," + pointVal.getFirst().x + "," + 
		                  "10000," + // Cell area; needs to be > zero
		                  pointVal.getSecond().surfaceFlow + "," +
			              pointVal.getSecond().baseFlow;
			if((steps > 1) && (steps < 4)){
				valsToSend+= ",5,.2,70,30,.30"; // Crop Id = 5, 20% of cell, at 30mm which is .3 of capacity, needs 70mm
			}
			valsToSend += "\n";
		}
		responder.send(HEADER + valsToSend + "\n");
		msg.debug("PROXY SENT: " + HEADER+valsToSend);
		steps++;
	}
	
	public void close() {
		responder.close();
		context.term();
	}
}
