package cnh.crb.wbm;

import java.awt.geom.Point2D;
import java.util.Date;

import cnh.crb.agent.BasicWaterManager;
import cnh.crb.climate.TemperatureStation;
import cnh.crb.hydro.CylindricalStorage;
import cnh.crb.hydro.Storage;
import cnh.crb.hydro.UnitsAndConverters.VolumeUnits;
import cnh.crb.hydro.WaterSink;

/**
 * Mediates between multiple water recipients and a single WBM water source.
 * 
 * Water sources can only have one recipient, but WBM Water sources can have
 * multiple recipients. This is partly because multiple natural sources
 * being modeled on the ABM side fall within the same grid cell boundaries
 * on the WBM side.
 * 
 * This class also permits return flow to be added to the WBM Natural Watersource
 * it is associated with
 * 
 * @author murphy
 *
 */
public class WBMWaterSourceManager extends BasicWaterManager implements WaterSink, TemperatureStation{

  // Memoize
  WBMNaturalWaterSource wbmSource;
  
  public WBMWaterSourceManager(String managerID, WBMNaturalWaterSource source){
    super(getDummyStorage(), managerID);
    addWaterSource(source);
    wbmSource = source;
  }

  private static Storage getDummyStorage(){
	  return new CylindricalStorage(new Date(), 10, 10, "Dummy");
  }
  
  
  public WBMNaturalWaterSource getSource(){
	  return waterSources.size() > 0 ? (WBMNaturalWaterSource)waterSources.get(0) : null;
  }

  public void eliminateWater(double volume, VolumeUnits units, WATER_SINK_METHOD method) {
	  wbmSource.eliminateWater(volume, units, method);
  }
  
  public void addWaterSinkAmounts(WaterSink other){
	  wbmSource.addWaterSinkAmounts(other);
  }

  public void notifyStorage(double storage){
	  wbmSource.notifyStorage(storage);
  }

  public void setTemperature(double newTemp) {
	wbmSource.setTemperature(newTemp);
  }

  public double getTemperature() {
	return wbmSource.getTemperature();
  }
  
  public Point2D.Double getLatLon(){ 
	return wbmSource.getLatLon(); 
  }
  
  public String getTemperatureStationId(){
	  return wbmSource.getTemperatureStationId();
  }
  
  
}
